/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the Sku class.
 * @constructor
 * The account SKU.
 *
 * @member {string} name Gets or sets the SKU name of the account. Possible
 * values include: 'Free', 'Basic'
 * @member {string} [family] Gets or sets the SKU family.
 * @member {number} [capacity] Gets or sets the SKU capacity.
 */
export interface Sku {
  name: string;
  family?: string;
  capacity?: number;
}

/**
 * @class
 * Initializes a new instance of the Resource class.
 * @constructor
 * The core properties of ARM resources
 *
 * @member {string} [id] Fully qualified resource Id for the resource
 * @member {string} [name] The name of the resource
 * @member {string} [type] The type of the resource.
 */
export interface Resource extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
}

/**
 * @class
 * Initializes a new instance of the TrackedResource class.
 * @constructor
 * The resource model definition for a ARM tracked top level resource
 *
 * @member {object} [tags] Resource tags.
 * @member {string} [location] The Azure Region where the resource lives
 */
export interface TrackedResource extends Resource {
  tags?: { [propertyName: string]: string };
  location?: string;
}

/**
 * @class
 * Initializes a new instance of the AutomationAccount class.
 * @constructor
 * Definition of the automation account type.
 *
 * @member {object} [sku] Gets or sets the SKU of account.
 * @member {string} [sku.name] Gets or sets the SKU name of the account.
 * Possible values include: 'Free', 'Basic'
 * @member {string} [sku.family] Gets or sets the SKU family.
 * @member {number} [sku.capacity] Gets or sets the SKU capacity.
 * @member {string} [lastModifiedBy] Gets or sets the last modified by.
 * @member {string} [state] Gets status of account. Possible values include:
 * 'Ok', 'Unavailable', 'Suspended'
 * @member {date} [creationTime] Gets the creation time.
 * @member {date} [lastModifiedTime] Gets the last modified time.
 * @member {string} [description] Gets or sets the description.
 * @member {string} [etag] Gets or sets the etag of the resource.
 */
export interface AutomationAccount extends TrackedResource {
  sku?: Sku;
  lastModifiedBy?: string;
  readonly state?: string;
  readonly creationTime?: Date;
  readonly lastModifiedTime?: Date;
  description?: string;
  etag?: string;
}

/**
 * @class
 * Initializes a new instance of the AutomationAccountCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update automation account
 * operation.
 *
 * @member {object} [sku] Gets or sets account SKU.
 * @member {string} [sku.name] Gets or sets the SKU name of the account.
 * Possible values include: 'Free', 'Basic'
 * @member {string} [sku.family] Gets or sets the SKU family.
 * @member {number} [sku.capacity] Gets or sets the SKU capacity.
 * @member {string} [name] Gets or sets name of the resource.
 * @member {string} [location] Gets or sets the location of the resource.
 * @member {object} [tags] Gets or sets the tags attached to the resource.
 */
export interface AutomationAccountCreateOrUpdateParameters {
  sku?: Sku;
  name?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the OperationDisplay class.
 * @constructor
 * Provider, Resource and Operation values
 *
 * @member {string} [provider] Service provider: Microsoft.Automation
 * @member {string} [resource] Resource on which the operation is performed:
 * Runbooks, Jobs etc.
 * @member {string} [operation] Operation type: Read, write, delete, etc.
 */
export interface OperationDisplay {
  provider?: string;
  resource?: string;
  operation?: string;
}

/**
 * @class
 * Initializes a new instance of the Operation class.
 * @constructor
 * Automation REST API operation
 *
 * @member {string} [name] Operation name: {provider}/{resource}/{operation}
 * @member {object} [display] Provider, Resource and Operation values
 * @member {string} [display.provider] Service provider: Microsoft.Automation
 * @member {string} [display.resource] Resource on which the operation is
 * performed: Runbooks, Jobs etc.
 * @member {string} [display.operation] Operation type: Read, write, delete,
 * etc.
 */
export interface Operation {
  name?: string;
  display?: OperationDisplay;
}

/**
 * @class
 * Initializes a new instance of the Statistics class.
 * @constructor
 * Definition of the statistic.
 *
 * @member {string} [counterProperty] Gets the property value of the statistic.
 * @member {number} [counterValue] Gets the value of the statistic.
 * @member {date} [startTime] Gets the startTime of the statistic.
 * @member {date} [endTime] Gets the endTime of the statistic.
 * @member {string} [id] Gets the id.
 */
export interface Statistics {
  readonly counterProperty?: string;
  readonly counterValue?: number;
  readonly startTime?: Date;
  readonly endTime?: Date;
  readonly id?: string;
}

/**
 * @class
 * Initializes a new instance of the UsageCounterName class.
 * @constructor
 * Definition of usage counter name.
 *
 * @member {string} [value] Gets or sets the usage counter name.
 * @member {string} [localizedValue] Gets or sets the localized usage counter
 * name.
 */
export interface UsageCounterName {
  value?: string;
  localizedValue?: string;
}

/**
 * @class
 * Initializes a new instance of the Usage class.
 * @constructor
 * Definition of Usage.
 *
 * @member {string} [id] Gets or sets the id of the resource.
 * @member {object} [name] Gets or sets the usage counter name.
 * @member {string} [name.value] Gets or sets the usage counter name.
 * @member {string} [name.localizedValue] Gets or sets the localized usage
 * counter name.
 * @member {string} [unit] Gets or sets the usage unit name.
 * @member {number} [currentValue] Gets or sets the current usage value.
 * @member {number} [limit] Gets or sets max limit. -1 for unlimited
 * @member {string} [throttleStatus] Gets or sets the throttle status.
 */
export interface Usage {
  id?: string;
  name?: UsageCounterName;
  unit?: string;
  currentValue?: number;
  limit?: number;
  throttleStatus?: string;
}

/**
 * @class
 * Initializes a new instance of the Key class.
 * @constructor
 * Automation key which is used to register a DSC Node
 *
 * @member {string} [keyName] Automation key name. Possible values include:
 * 'Primary', 'Secondary'
 * @member {string} [permissions] Automation key permissions. Possible values
 * include: 'Read', 'Full'
 * @member {string} [value] Value of the Automation Key used for registration.
 */
export interface Key {
  readonly keyName?: string;
  readonly permissions?: string;
  readonly value?: string;
}

/**
 * @class
 * Initializes a new instance of the KeyListResult class.
 * @constructor
 * @member {array} [keys] Lists the automation keys.
 */
export interface KeyListResult {
  keys?: Key[];
}

/**
 * @class
 * Initializes a new instance of the AutomationAccountUpdateParameters class.
 * @constructor
 * The parameters supplied to the update automation account operation.
 *
 * @member {object} [sku] Gets or sets account SKU.
 * @member {string} [sku.name] Gets or sets the SKU name of the account.
 * Possible values include: 'Free', 'Basic'
 * @member {string} [sku.family] Gets or sets the SKU family.
 * @member {number} [sku.capacity] Gets or sets the SKU capacity.
 * @member {string} [name] Gets or sets the name of the resource.
 * @member {string} [location] Gets or sets the location of the resource.
 * @member {object} [tags] Gets or sets the tags attached to the resource.
 */
export interface AutomationAccountUpdateParameters {
  sku?: Sku;
  name?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the ProxyResource class.
 * @constructor
 * ARM proxy resource.
 *
 */
export interface ProxyResource extends Resource {
}

/**
 * @class
 * Initializes a new instance of the ErrorResponse class.
 * @constructor
 * Error response of an operation failure
 *
 * @member {string} [code] Error code
 * @member {string} [message] Error message indicating why the operation
 * failed.
 */
export interface ErrorResponse {
  code?: string;
  message?: string;
}

/**
 * @class
 * Initializes a new instance of the CertificateCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update or replace certificate
 * operation.
 *
 * @member {string} name Gets or sets the name of the certificate.
 * @member {string} base64Value Gets or sets the base64 encoded value of the
 * certificate.
 * @member {string} [description] Gets or sets the description of the
 * certificate.
 * @member {string} [thumbprint] Gets or sets the thumbprint of the
 * certificate.
 * @member {boolean} [isExportable] Gets or sets the is exportable flag of the
 * certificate.
 */
export interface CertificateCreateOrUpdateParameters {
  name: string;
  base64Value: string;
  description?: string;
  thumbprint?: string;
  isExportable?: boolean;
}

/**
 * @class
 * Initializes a new instance of the Certificate class.
 * @constructor
 * Definition of the certificate.
 *
 * @member {string} [thumbprint] Gets the thumbprint of the certificate.
 * @member {date} [expiryTime] Gets the expiry time of the certificate.
 * @member {boolean} [isExportable] Gets the is exportable flag of the
 * certificate.
 * @member {date} [creationTime] Gets the creation time.
 * @member {date} [lastModifiedTime] Gets the last modified time.
 * @member {string} [description] Gets or sets the description.
 */
export interface Certificate extends ProxyResource {
  readonly thumbprint?: string;
  readonly expiryTime?: Date;
  readonly isExportable?: boolean;
  readonly creationTime?: Date;
  readonly lastModifiedTime?: Date;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the CertificateUpdateParameters class.
 * @constructor
 * The parameters supplied to the update certificate operation.
 *
 * @member {string} [name] Gets or sets the name of the certificate.
 * @member {string} [description] Gets or sets the description of the
 * certificate.
 */
export interface CertificateUpdateParameters {
  name?: string;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the ConnectionTypeAssociationProperty class.
 * @constructor
 * The connection type property associated with the entity.
 *
 * @member {string} [name] Gets or sets the name of the connection type.
 */
export interface ConnectionTypeAssociationProperty {
  name?: string;
}

/**
 * @class
 * Initializes a new instance of the ConnectionCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update connection operation.
 *
 * @member {string} name Gets or sets the name of the connection.
 * @member {string} [description] Gets or sets the description of the
 * connection.
 * @member {object} connectionType Gets or sets the connectionType of the
 * connection.
 * @member {string} [connectionType.name] Gets or sets the name of the
 * connection type.
 * @member {object} [fieldDefinitionValues] Gets or sets the field definition
 * properties of the connection.
 */
export interface ConnectionCreateOrUpdateParameters {
  name: string;
  description?: string;
  connectionType: ConnectionTypeAssociationProperty;
  fieldDefinitionValues?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the Connection class.
 * @constructor
 * Definition of the connection.
 *
 * @member {object} [connectionType] Gets or sets the connectionType of the
 * connection.
 * @member {string} [connectionType.name] Gets or sets the name of the
 * connection type.
 * @member {object} [fieldDefinitionValues] Gets the field definition values of
 * the connection.
 * @member {date} [creationTime] Gets the creation time.
 * @member {date} [lastModifiedTime] Gets the last modified time.
 * @member {string} [description] Gets or sets the description.
 */
export interface Connection extends ProxyResource {
  connectionType?: ConnectionTypeAssociationProperty;
  readonly fieldDefinitionValues?: { [propertyName: string]: string };
  readonly creationTime?: Date;
  readonly lastModifiedTime?: Date;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the ConnectionUpdateParameters class.
 * @constructor
 * The parameters supplied to the update connection operation.
 *
 * @member {string} [name] Gets or sets the name of the connection.
 * @member {string} [description] Gets or sets the description of the
 * connection.
 * @member {object} [fieldDefinitionValues] Gets or sets the field definition
 * values of the connection.
 */
export interface ConnectionUpdateParameters {
  name?: string;
  description?: string;
  fieldDefinitionValues?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the FieldDefinition class.
 * @constructor
 * Definition of the connection fields.
 *
 * @member {boolean} [isEncrypted] Gets or sets the isEncrypted flag of the
 * connection field definition.
 * @member {boolean} [isOptional] Gets or sets the isOptional flag of the
 * connection field definition.
 * @member {string} type Gets or sets the type of the connection field
 * definition.
 */
export interface FieldDefinition {
  isEncrypted?: boolean;
  isOptional?: boolean;
  type: string;
}

/**
 * @class
 * Initializes a new instance of the ConnectionType class.
 * @constructor
 * Definition of the connection type.
 *
 * @member {string} [id] Gets the id of the resource.
 * @member {string} [name] Gets the name of the connection type.
 * @member {string} [type] Resource type
 * @member {boolean} [isGlobal] Gets or sets a Boolean value to indicate if the
 * connection type is global.
 * @member {object} [fieldDefinitions] Gets the field definitions of the
 * connection type.
 * @member {date} [creationTime] Gets the creation time.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time.
 * @member {string} [description] Gets or sets the description.
 */
export interface ConnectionType {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
  isGlobal?: boolean;
  readonly fieldDefinitions?: { [propertyName: string]: FieldDefinition };
  readonly creationTime?: Date;
  lastModifiedTime?: Date;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the ConnectionTypeCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update connection type operation.
 *
 * @member {string} name Gets or sets the name of the connection type.
 * @member {boolean} [isGlobal] Gets or sets a Boolean value to indicate if the
 * connection type is global.
 * @member {object} fieldDefinitions Gets or sets the field definitions of the
 * connection type.
 */
export interface ConnectionTypeCreateOrUpdateParameters {
  name: string;
  isGlobal?: boolean;
  fieldDefinitions: { [propertyName: string]: FieldDefinition };
}

/**
 * @class
 * Initializes a new instance of the CredentialCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update credential operation.
 *
 * @member {string} name Gets or sets the name of the credential.
 * @member {string} userName Gets or sets the user name of the credential.
 * @member {string} password Gets or sets the password of the credential.
 * @member {string} [description] Gets or sets the description of the
 * credential.
 */
export interface CredentialCreateOrUpdateParameters {
  name: string;
  userName: string;
  password: string;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the Credential class.
 * @constructor
 * Definition of the credential.
 *
 * @member {string} [userName] Gets the user name of the credential.
 * @member {date} [creationTime] Gets the creation time.
 * @member {date} [lastModifiedTime] Gets the last modified time.
 * @member {string} [description] Gets or sets the description.
 */
export interface Credential extends ProxyResource {
  readonly userName?: string;
  readonly creationTime?: Date;
  readonly lastModifiedTime?: Date;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the CredentialUpdateParameters class.
 * @constructor
 * The parameters supplied to the Update credential operation.
 *
 * @member {string} [name] Gets or sets the name of the credential.
 * @member {string} [userName] Gets or sets the user name of the credential.
 * @member {string} [password] Gets or sets the password of the credential.
 * @member {string} [description] Gets or sets the description of the
 * credential.
 */
export interface CredentialUpdateParameters {
  name?: string;
  userName?: string;
  password?: string;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the ContentHash class.
 * @constructor
 * Definition of the runbook property type.
 *
 * @member {string} algorithm Gets or sets the content hash algorithm used to
 * hash the content.
 * @member {string} value Gets or sets expected hash value of the content.
 */
export interface ContentHash {
  algorithm: string;
  value: string;
}

/**
 * @class
 * Initializes a new instance of the ContentSource class.
 * @constructor
 * Definition of the content source.
 *
 * @member {object} [hash] Gets or sets the hash.
 * @member {string} [hash.algorithm] Gets or sets the content hash algorithm
 * used to hash the content.
 * @member {string} [hash.value] Gets or sets expected hash value of the
 * content.
 * @member {string} [type] Gets or sets the content source type. Possible
 * values include: 'embeddedContent', 'uri'
 * @member {string} [value] Gets or sets the value of the content. This is
 * based on the content source type.
 * @member {string} [version] Gets or sets the version of the content.
 */
export interface ContentSource {
  hash?: ContentHash;
  type?: string;
  value?: string;
  version?: string;
}

/**
 * @class
 * Initializes a new instance of the DscConfigurationParameter class.
 * @constructor
 * Definition of the configuration parameter type.
 *
 * @member {string} [type] Gets or sets the type of the parameter.
 * @member {boolean} [isMandatory] Gets or sets a Boolean value to indicate
 * whether the parameter is madatory or not.
 * @member {number} [position] Get or sets the position of the parameter.
 * @member {string} [defaultValue] Gets or sets the default value of parameter.
 */
export interface DscConfigurationParameter {
  type?: string;
  isMandatory?: boolean;
  position?: number;
  defaultValue?: string;
}

/**
 * @class
 * Initializes a new instance of the DscConfigurationCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update configuration operation.
 *
 * @member {boolean} [logVerbose] Gets or sets verbose log option.
 * @member {boolean} [logProgress] Gets or sets progress log option.
 * @member {object} source Gets or sets the source.
 * @member {object} [source.hash] Gets or sets the hash.
 * @member {string} [source.hash.algorithm] Gets or sets the content hash
 * algorithm used to hash the content.
 * @member {string} [source.hash.value] Gets or sets expected hash value of the
 * content.
 * @member {string} [source.type] Gets or sets the content source type.
 * Possible values include: 'embeddedContent', 'uri'
 * @member {string} [source.value] Gets or sets the value of the content. This
 * is based on the content source type.
 * @member {string} [source.version] Gets or sets the version of the content.
 * @member {object} [parameters] Gets or sets the configuration parameters.
 * @member {string} [description] Gets or sets the description of the
 * configuration.
 * @member {string} [name] Gets or sets name of the resource.
 * @member {string} [location] Gets or sets the location of the resource.
 * @member {object} [tags] Gets or sets the tags attached to the resource.
 */
export interface DscConfigurationCreateOrUpdateParameters {
  logVerbose?: boolean;
  logProgress?: boolean;
  source: ContentSource;
  parameters?: { [propertyName: string]: DscConfigurationParameter };
  description?: string;
  name?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the DscConfiguration class.
 * @constructor
 * Definition of the configuration type.
 *
 * @member {string} [provisioningState] Gets or sets the provisioning state of
 * the configuration. Possible values include: 'Succeeded'
 * @member {number} [jobCount] Gets or sets the job count of the configuration.
 * @member {object} [parameters] Gets or sets the configuration parameters.
 * @member {object} [source] Gets or sets the source.
 * @member {object} [source.hash] Gets or sets the hash.
 * @member {string} [source.hash.algorithm] Gets or sets the content hash
 * algorithm used to hash the content.
 * @member {string} [source.hash.value] Gets or sets expected hash value of the
 * content.
 * @member {string} [source.type] Gets or sets the content source type.
 * Possible values include: 'embeddedContent', 'uri'
 * @member {string} [source.value] Gets or sets the value of the content. This
 * is based on the content source type.
 * @member {string} [source.version] Gets or sets the version of the content.
 * @member {string} [state] Gets or sets the state of the configuration.
 * Possible values include: 'New', 'Edit', 'Published'
 * @member {boolean} [logVerbose] Gets or sets verbose log option.
 * @member {date} [creationTime] Gets or sets the creation time.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time.
 * @member {number} [nodeConfigurationCount] Gets the number of compiled node
 * configurations.
 * @member {string} [description] Gets or sets the description.
 * @member {string} [etag] Gets or sets the etag of the resource.
 */
export interface DscConfiguration extends TrackedResource {
  provisioningState?: string;
  jobCount?: number;
  parameters?: { [propertyName: string]: DscConfigurationParameter };
  source?: ContentSource;
  state?: string;
  logVerbose?: boolean;
  creationTime?: Date;
  lastModifiedTime?: Date;
  nodeConfigurationCount?: number;
  description?: string;
  etag?: string;
}

/**
 * @class
 * Initializes a new instance of the DscConfigurationUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update configuration operation.
 *
 * @member {boolean} [logVerbose] Gets or sets verbose log option.
 * @member {boolean} [logProgress] Gets or sets progress log option.
 * @member {object} source Gets or sets the source.
 * @member {object} [source.hash] Gets or sets the hash.
 * @member {string} [source.hash.algorithm] Gets or sets the content hash
 * algorithm used to hash the content.
 * @member {string} [source.hash.value] Gets or sets expected hash value of the
 * content.
 * @member {string} [source.type] Gets or sets the content source type.
 * Possible values include: 'embeddedContent', 'uri'
 * @member {string} [source.value] Gets or sets the value of the content. This
 * is based on the content source type.
 * @member {string} [source.version] Gets or sets the version of the content.
 * @member {object} [parameters] Gets or sets the configuration parameters.
 * @member {string} [description] Gets or sets the description of the
 * configuration.
 * @member {string} [name] Gets or sets name of the resource.
 * @member {object} [tags] Gets or sets the tags attached to the resource.
 */
export interface DscConfigurationUpdateParameters {
  logVerbose?: boolean;
  logProgress?: boolean;
  source: ContentSource;
  parameters?: { [propertyName: string]: DscConfigurationParameter };
  description?: string;
  name?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the RunAsCredentialAssociationProperty class.
 * @constructor
 * Definition of runas credential to use for hybrid worker.
 *
 * @member {string} [name] Gets or sets the name of the credential.
 */
export interface RunAsCredentialAssociationProperty {
  name?: string;
}

/**
 * @class
 * Initializes a new instance of the HybridRunbookWorker class.
 * @constructor
 * Definition of hybrid runbook worker.
 *
 * @member {string} [name] Gets or sets the worker machine name.
 * @member {string} [ip] Gets or sets the assigned machine IP address.
 * @member {date} [registrationTime] Gets or sets the registration time of the
 * worker machine.
 * @member {date} [lastSeenDateTime] Last Heartbeat from the Worker
 */
export interface HybridRunbookWorker {
  name?: string;
  ip?: string;
  registrationTime?: Date;
  lastSeenDateTime?: Date;
}

/**
 * @class
 * Initializes a new instance of the HybridRunbookWorkerGroup class.
 * @constructor
 * Definition of hybrid runbook worker group.
 *
 * @member {string} [id] Gets or sets the id of the resource.
 * @member {string} [name] Gets or sets the name of the group.
 * @member {array} [hybridRunbookWorkers] Gets or sets the list of hybrid
 * runbook workers.
 * @member {object} [credential] Sets the credential of a worker group.
 * @member {string} [credential.name] Gets or sets the name of the credential.
 * @member {string} [groupType] Type of the HybridWorkerGroup. Possible values
 * include: 'User', 'System'
 */
export interface HybridRunbookWorkerGroup {
  id?: string;
  name?: string;
  hybridRunbookWorkers?: HybridRunbookWorker[];
  credential?: RunAsCredentialAssociationProperty;
  groupType?: string;
}

/**
 * @class
 * Initializes a new instance of the HybridRunbookWorkerGroupUpdateParameters class.
 * @constructor
 * Parameters supplied to the update operation.
 *
 * @member {object} [credential] Sets the credential of a worker group.
 * @member {string} [credential.name] Gets or sets the name of the credential.
 */
export interface HybridRunbookWorkerGroupUpdateParameters {
  credential?: RunAsCredentialAssociationProperty;
}

/**
 * @class
 * Initializes a new instance of the ScheduleAssociationProperty class.
 * @constructor
 * The schedule property associated with the entity.
 *
 * @member {string} [name] Gets or sets the name of the Schedule.
 */
export interface ScheduleAssociationProperty {
  name?: string;
}

/**
 * @class
 * Initializes a new instance of the RunbookAssociationProperty class.
 * @constructor
 * The runbook property associated with the entity.
 *
 * @member {string} [name] Gets or sets the name of the runbook.
 */
export interface RunbookAssociationProperty {
  name?: string;
}

/**
 * @class
 * Initializes a new instance of the JobSchedule class.
 * @constructor
 * Definition of the job schedule.
 *
 * @member {string} [id] Gets the id of the resource.
 * @member {string} [name] Gets the name of the variable.
 * @member {string} [type] Resource type
 * @member {string} [jobScheduleId] Gets or sets the id of job schedule.
 * @member {object} [schedule] Gets or sets the schedule.
 * @member {string} [schedule.name] Gets or sets the name of the Schedule.
 * @member {object} [runbook] Gets or sets the runbook.
 * @member {string} [runbook.name] Gets or sets the name of the runbook.
 * @member {string} [runOn] Gets or sets the hybrid worker group that the
 * scheduled job should run on.
 * @member {object} [parameters] Gets or sets the parameters of the job
 * schedule.
 */
export interface JobSchedule {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
  jobScheduleId?: string;
  schedule?: ScheduleAssociationProperty;
  runbook?: RunbookAssociationProperty;
  runOn?: string;
  parameters?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the JobScheduleCreateParameters class.
 * @constructor
 * The parameters supplied to the create job schedule operation.
 *
 * @member {object} schedule Gets or sets the schedule.
 * @member {string} [schedule.name] Gets or sets the name of the Schedule.
 * @member {object} runbook Gets or sets the runbook.
 * @member {string} [runbook.name] Gets or sets the name of the runbook.
 * @member {string} [runOn] Gets or sets the hybrid worker group that the
 * scheduled job should run on.
 * @member {object} [parameters] Gets or sets a list of job properties.
 */
export interface JobScheduleCreateParameters {
  schedule: ScheduleAssociationProperty;
  runbook: RunbookAssociationProperty;
  runOn?: string;
  parameters?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the LinkedWorkspace class.
 * @constructor
 * Definition of the linked workspace.
 *
 * @member {string} [id] Gets the id of the linked workspace.
 */
export interface LinkedWorkspace {
  readonly id?: string;
}

/**
 * @class
 * Initializes a new instance of the ActivityParameterValidationSet class.
 * @constructor
 * Definition of the activity parameter validation set.
 *
 * @member {string} [memberValue] Gets or sets the name of the activity
 * parameter validation set member.
 */
export interface ActivityParameterValidationSet {
  memberValue?: string;
}

/**
 * @class
 * Initializes a new instance of the ActivityParameter class.
 * @constructor
 * Definition of the activity parameter.
 *
 * @member {string} [name] Gets or sets the name of the activity parameter.
 * @member {string} [type] Gets or sets the type of the activity parameter.
 * @member {boolean} [isMandatory] Gets or sets a Boolean value that indicates
 * true if the parameter is required. If the value is false, the parameter is
 * optional.
 * @member {boolean} [isDynamic] Gets or sets a Boolean value that indicates
 * true if the parameter is dynamic.
 * @member {number} [position] Gets or sets the position of the activity
 * parameter.
 * @member {boolean} [valueFromPipeline] Gets or sets a Boolean value that
 * indicates true if the parameter can take values from the incoming pipeline
 * objects. This setting is used if the cmdlet must access the complete input
 * object. false indicates that the parameter cannot take values from the
 * complete input object.
 * @member {boolean} [valueFromPipelineByPropertyName] Gets or sets a Boolean
 * value that indicates true if the parameter can be filled from a property of
 * the incoming pipeline object that has the same name as this parameter. false
 * indicates that the parameter cannot be filled from the incoming pipeline
 * object property with the same name.
 * @member {boolean} [valueFromRemainingArguments] Gets or sets a Boolean value
 * that indicates true if the cmdlet parameter accepts all the remaining
 * command-line arguments that are associated with this parameter in the form
 * of an array. false if the cmdlet parameter does not accept all the remaining
 * argument values.
 * @member {string} [description] Gets or sets the description of the activity
 * parameter.
 * @member {array} [validationSet] Gets or sets the validation set of activity
 * parameter.
 */
export interface ActivityParameter {
  name?: string;
  type?: string;
  isMandatory?: boolean;
  isDynamic?: boolean;
  position?: number;
  valueFromPipeline?: boolean;
  valueFromPipelineByPropertyName?: boolean;
  valueFromRemainingArguments?: boolean;
  description?: string;
  validationSet?: ActivityParameterValidationSet[];
}

/**
 * @class
 * Initializes a new instance of the ActivityParameterSet class.
 * @constructor
 * Definition of the activity parameter set.
 *
 * @member {string} [name] Gets or sets the name of the activity parameter set.
 * @member {array} [parameters] Gets or sets the parameters of the activity
 * parameter set.
 */
export interface ActivityParameterSet {
  name?: string;
  parameters?: ActivityParameter[];
}

/**
 * @class
 * Initializes a new instance of the ActivityOutputType class.
 * @constructor
 * Definition of the activity output type.
 *
 * @member {string} [name] Gets or sets the name of the activity output type.
 * @member {string} [type] Gets or sets the type of the activity output type.
 */
export interface ActivityOutputType {
  name?: string;
  type?: string;
}

/**
 * @class
 * Initializes a new instance of the Activity class.
 * @constructor
 * Definition of the activity.
 *
 * @member {string} [id] Gets or sets the id of the resource.
 * @member {string} [name] Gets the name of the activity.
 * @member {string} [definition] Gets or sets the user name of the activity.
 * @member {array} [parameterSets] Gets or sets the parameter sets of the
 * activity.
 * @member {array} [outputTypes] Gets or sets the output types of the activity.
 * @member {date} [creationTime] Gets or sets the creation time.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time.
 * @member {string} [description] Gets or sets the description.
 */
export interface Activity {
  id?: string;
  readonly name?: string;
  definition?: string;
  parameterSets?: ActivityParameterSet[];
  outputTypes?: ActivityOutputType[];
  creationTime?: Date;
  lastModifiedTime?: Date;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the ModuleErrorInfo class.
 * @constructor
 * Definition of the module error info type.
 *
 * @member {string} [code] Gets or sets the error code.
 * @member {string} [message] Gets or sets the error message.
 */
export interface ModuleErrorInfo {
  code?: string;
  message?: string;
}

/**
 * @class
 * Initializes a new instance of the ContentLink class.
 * @constructor
 * Definition of the content link.
 *
 * @member {string} [uri] Gets or sets the uri of the runbook content.
 * @member {object} [contentHash] Gets or sets the hash.
 * @member {string} [contentHash.algorithm] Gets or sets the content hash
 * algorithm used to hash the content.
 * @member {string} [contentHash.value] Gets or sets expected hash value of the
 * content.
 * @member {string} [version] Gets or sets the version of the content.
 */
export interface ContentLink {
  uri?: string;
  contentHash?: ContentHash;
  version?: string;
}

/**
 * @class
 * Initializes a new instance of the Module class.
 * @constructor
 * Definition of the module type.
 *
 * @member {boolean} [isGlobal] Gets or sets the isGlobal flag of the module.
 * @member {string} [version] Gets or sets the version of the module.
 * @member {number} [sizeInBytes] Gets or sets the size in bytes of the module.
 * @member {number} [activityCount] Gets or sets the activity count of the
 * module.
 * @member {string} [provisioningState] Gets or sets the provisioning state of
 * the module. Possible values include: 'Created', 'Creating',
 * 'StartingImportModuleRunbook', 'RunningImportModuleRunbook',
 * 'ContentRetrieved', 'ContentDownloaded', 'ContentValidated',
 * 'ConnectionTypeImported', 'ContentStored', 'ModuleDataStored',
 * 'ActivitiesStored', 'ModuleImportRunbookComplete', 'Succeeded', 'Failed',
 * 'Cancelled', 'Updating'
 * @member {object} [contentLink] Gets or sets the contentLink of the module.
 * @member {string} [contentLink.uri] Gets or sets the uri of the runbook
 * content.
 * @member {object} [contentLink.contentHash] Gets or sets the hash.
 * @member {string} [contentLink.contentHash.algorithm] Gets or sets the
 * content hash algorithm used to hash the content.
 * @member {string} [contentLink.contentHash.value] Gets or sets expected hash
 * value of the content.
 * @member {string} [contentLink.version] Gets or sets the version of the
 * content.
 * @member {object} [error] Gets or sets the error info of the module.
 * @member {string} [error.code] Gets or sets the error code.
 * @member {string} [error.message] Gets or sets the error message.
 * @member {date} [creationTime] Gets or sets the creation time.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time.
 * @member {string} [description] Gets or sets the description.
 * @member {boolean} [isComposite] Gets or sets type of module, if its
 * composite or not.
 * @member {string} [etag] Gets or sets the etag of the resource.
 */
export interface Module extends TrackedResource {
  isGlobal?: boolean;
  version?: string;
  sizeInBytes?: number;
  activityCount?: number;
  provisioningState?: string;
  contentLink?: ContentLink;
  error?: ModuleErrorInfo;
  creationTime?: Date;
  lastModifiedTime?: Date;
  description?: string;
  isComposite?: boolean;
  etag?: string;
}

/**
 * @class
 * Initializes a new instance of the ModuleCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update module operation.
 *
 * @member {object} contentLink Gets or sets the module content link.
 * @member {string} [contentLink.uri] Gets or sets the uri of the runbook
 * content.
 * @member {object} [contentLink.contentHash] Gets or sets the hash.
 * @member {string} [contentLink.contentHash.algorithm] Gets or sets the
 * content hash algorithm used to hash the content.
 * @member {string} [contentLink.contentHash.value] Gets or sets expected hash
 * value of the content.
 * @member {string} [contentLink.version] Gets or sets the version of the
 * content.
 * @member {string} [name] Gets or sets name of the resource.
 * @member {string} [location] Gets or sets the location of the resource.
 * @member {object} [tags] Gets or sets the tags attached to the resource.
 */
export interface ModuleCreateOrUpdateParameters {
  contentLink: ContentLink;
  name?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the ModuleUpdateParameters class.
 * @constructor
 * The parameters supplied to the update module operation.
 *
 * @member {object} [contentLink] Gets or sets the module content link.
 * @member {string} [contentLink.uri] Gets or sets the uri of the runbook
 * content.
 * @member {object} [contentLink.contentHash] Gets or sets the hash.
 * @member {string} [contentLink.contentHash.algorithm] Gets or sets the
 * content hash algorithm used to hash the content.
 * @member {string} [contentLink.contentHash.value] Gets or sets expected hash
 * value of the content.
 * @member {string} [contentLink.version] Gets or sets the version of the
 * content.
 * @member {string} [name] Gets or sets name of the resource.
 * @member {string} [location] Gets or sets the location of the resource.
 * @member {object} [tags] Gets or sets the tags attached to the resource.
 */
export interface ModuleUpdateParameters {
  contentLink?: ContentLink;
  name?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the TypeField class.
 * @constructor
 * Information about a field of a type.
 *
 * @member {string} [name] Gets or sets the name of the field.
 * @member {string} [type] Gets or sets the type of the field.
 */
export interface TypeField {
  name?: string;
  type?: string;
}

/**
 * @class
 * Initializes a new instance of the AdvancedScheduleMonthlyOccurrence class.
 * @constructor
 * The properties of the create advanced schedule monthly occurrence.
 *
 * @member {number} [occurrence] Occurrence of the week within the month. Must
 * be between 1 and 5
 * @member {string} [day] Day of the occurrence. Must be one of monday,
 * tuesday, wednesday, thursday, friday, saturday, sunday. Possible values
 * include: 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday',
 * 'Sunday'
 */
export interface AdvancedScheduleMonthlyOccurrence {
  occurrence?: number;
  day?: string;
}

/**
 * @class
 * Initializes a new instance of the AdvancedSchedule class.
 * @constructor
 * The properties of the create Advanced Schedule.
 *
 * @member {array} [weekDays] Days of the week that the job should execute on.
 * @member {array} [monthDays] Days of the month that the job should execute
 * on. Must be between 1 and 31.
 * @member {array} [monthlyOccurrences] Occurrences of days within a month.
 */
export interface AdvancedSchedule {
  weekDays?: string[];
  monthDays?: number[];
  monthlyOccurrences?: AdvancedScheduleMonthlyOccurrence[];
}

/**
 * @class
 * Initializes a new instance of the ScheduleCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update schedule operation.
 *
 * @member {string} name Gets or sets the name of the Schedule.
 * @member {string} [description] Gets or sets the description of the schedule.
 * @member {date} startTime Gets or sets the start time of the schedule.
 * @member {date} [expiryTime] Gets or sets the end time of the schedule.
 * @member {object} [interval] Gets or sets the interval of the schedule.
 * @member {string} frequency Possible values include: 'OneTime', 'Day',
 * 'Hour', 'Week', 'Month'
 * @member {string} [timeZone] Gets or sets the time zone of the schedule.
 * @member {object} [advancedSchedule] Gets or sets the AdvancedSchedule.
 * @member {array} [advancedSchedule.weekDays] Days of the week that the job
 * should execute on.
 * @member {array} [advancedSchedule.monthDays] Days of the month that the job
 * should execute on. Must be between 1 and 31.
 * @member {array} [advancedSchedule.monthlyOccurrences] Occurrences of days
 * within a month.
 */
export interface ScheduleCreateOrUpdateParameters {
  name: string;
  description?: string;
  startTime: Date;
  expiryTime?: Date;
  interval?: any;
  frequency: string;
  timeZone?: string;
  advancedSchedule?: AdvancedSchedule;
}

/**
 * @class
 * Initializes a new instance of the ScheduleProperties class.
 * @constructor
 * Definition of schedule parameters.
 *
 * @member {date} [startTime] Gets or sets the start time of the schedule.
 * @member {number} [startTimeOffsetMinutes] Gets the start time's offset in
 * minutes.
 * @member {date} [expiryTime] Gets or sets the end time of the schedule.
 * @member {number} [expiryTimeOffsetMinutes] Gets or sets the expiry time's
 * offset in minutes.
 * @member {boolean} [isEnabled] Gets or sets a value indicating whether this
 * schedule is enabled. Default value: false .
 * @member {date} [nextRun] Gets or sets the next run time of the schedule.
 * @member {number} [nextRunOffsetMinutes] Gets or sets the next run time's
 * offset in minutes.
 * @member {number} [interval] Gets or sets the interval of the schedule.
 * @member {string} [frequency] Gets or sets the frequency of the schedule.
 * Possible values include: 'OneTime', 'Day', 'Hour', 'Week', 'Month'
 * @member {string} [timeZone] Gets or sets the time zone of the schedule.
 * @member {object} [advancedSchedule] Gets or sets the advanced schedule.
 * @member {array} [advancedSchedule.weekDays] Days of the week that the job
 * should execute on.
 * @member {array} [advancedSchedule.monthDays] Days of the month that the job
 * should execute on. Must be between 1 and 31.
 * @member {array} [advancedSchedule.monthlyOccurrences] Occurrences of days
 * within a month.
 * @member {date} [creationTime] Gets or sets the creation time.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time.
 * @member {string} [description] Gets or sets the description.
 */
export interface ScheduleProperties {
  startTime?: Date;
  readonly startTimeOffsetMinutes?: number;
  expiryTime?: Date;
  expiryTimeOffsetMinutes?: number;
  isEnabled?: boolean;
  nextRun?: Date;
  nextRunOffsetMinutes?: number;
  interval?: number;
  frequency?: string;
  timeZone?: string;
  advancedSchedule?: AdvancedSchedule;
  creationTime?: Date;
  lastModifiedTime?: Date;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the Schedule class.
 * @constructor
 * Definition of the schedule.
 *
 * @member {date} [startTime] Gets or sets the start time of the schedule.
 * @member {number} [startTimeOffsetMinutes] Gets the start time's offset in
 * minutes.
 * @member {date} [expiryTime] Gets or sets the end time of the schedule.
 * @member {number} [expiryTimeOffsetMinutes] Gets or sets the expiry time's
 * offset in minutes.
 * @member {boolean} [isEnabled] Gets or sets a value indicating whether this
 * schedule is enabled. Default value: false .
 * @member {date} [nextRun] Gets or sets the next run time of the schedule.
 * @member {number} [nextRunOffsetMinutes] Gets or sets the next run time's
 * offset in minutes.
 * @member {number} [interval] Gets or sets the interval of the schedule.
 * @member {string} [frequency] Gets or sets the frequency of the schedule.
 * Possible values include: 'OneTime', 'Day', 'Hour', 'Week', 'Month'
 * @member {string} [timeZone] Gets or sets the time zone of the schedule.
 * @member {object} [advancedSchedule] Gets or sets the advanced schedule.
 * @member {array} [advancedSchedule.weekDays] Days of the week that the job
 * should execute on.
 * @member {array} [advancedSchedule.monthDays] Days of the month that the job
 * should execute on. Must be between 1 and 31.
 * @member {array} [advancedSchedule.monthlyOccurrences] Occurrences of days
 * within a month.
 * @member {date} [creationTime] Gets or sets the creation time.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time.
 * @member {string} [description] Gets or sets the description.
 */
export interface Schedule extends ProxyResource {
  startTime?: Date;
  readonly startTimeOffsetMinutes?: number;
  expiryTime?: Date;
  expiryTimeOffsetMinutes?: number;
  isEnabled?: boolean;
  nextRun?: Date;
  nextRunOffsetMinutes?: number;
  interval?: number;
  frequency?: string;
  timeZone?: string;
  advancedSchedule?: AdvancedSchedule;
  creationTime?: Date;
  lastModifiedTime?: Date;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the ScheduleUpdateParameters class.
 * @constructor
 * The parameters supplied to the update schedule operation.
 *
 * @member {string} [name] Gets or sets the name of the Schedule.
 * @member {string} [description] Gets or sets the description of the schedule.
 * @member {boolean} [isEnabled] Gets or sets a value indicating whether this
 * schedule is enabled.
 */
export interface ScheduleUpdateParameters {
  name?: string;
  description?: string;
  isEnabled?: boolean;
}

/**
 * @class
 * Initializes a new instance of the VariableCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update variable operation.
 *
 * @member {string} name Gets or sets the name of the variable.
 * @member {string} [value] Gets or sets the value of the variable.
 * @member {string} [description] Gets or sets the description of the variable.
 * @member {boolean} [isEncrypted] Gets or sets the encrypted flag of the
 * variable.
 */
export interface VariableCreateOrUpdateParameters {
  name: string;
  value?: string;
  description?: string;
  isEncrypted?: boolean;
}

/**
 * @class
 * Initializes a new instance of the Variable class.
 * @constructor
 * Definition of the varible.
 *
 * @member {string} [value] Gets or sets the value of the variable.
 * @member {boolean} [isEncrypted] Gets or sets the encrypted flag of the
 * variable.
 * @member {date} [creationTime] Gets or sets the creation time.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time.
 * @member {string} [description] Gets or sets the description.
 */
export interface Variable extends ProxyResource {
  value?: string;
  isEncrypted?: boolean;
  creationTime?: Date;
  lastModifiedTime?: Date;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the VariableUpdateParameters class.
 * @constructor
 * The parameters supplied to the update variable operation.
 *
 * @member {string} [name] Gets or sets the name of the variable.
 * @member {string} [value] Gets or sets the value of the variable.
 * @member {string} [description] Gets or sets the description of the variable.
 */
export interface VariableUpdateParameters {
  name?: string;
  value?: string;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the Webhook class.
 * @constructor
 * Definition of the webhook type.
 *
 * @member {boolean} [isEnabled] Gets or sets the value of the enabled flag of
 * the webhook. Default value: false .
 * @member {string} [uri] Gets or sets the webhook uri.
 * @member {date} [expiryTime] Gets or sets the expiry time.
 * @member {date} [lastInvokedTime] Gets or sets the last invoked time.
 * @member {object} [parameters] Gets or sets the parameters of the job that is
 * created when the webhook calls the runbook it is associated with.
 * @member {object} [runbook] Gets or sets the runbook the webhook is
 * associated with.
 * @member {string} [runbook.name] Gets or sets the name of the runbook.
 * @member {string} [runOn] Gets or sets the name of the hybrid worker group
 * the webhook job will run on.
 * @member {date} [creationTime] Gets or sets the creation time.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time.
 * @member {string} [lastModifiedBy] Details of the user who last modified the
 * Webhook
 * @member {string} [description] Gets or sets the description.
 */
export interface Webhook extends ProxyResource {
  isEnabled?: boolean;
  uri?: string;
  expiryTime?: Date;
  lastInvokedTime?: Date;
  parameters?: { [propertyName: string]: string };
  runbook?: RunbookAssociationProperty;
  runOn?: string;
  creationTime?: Date;
  lastModifiedTime?: Date;
  lastModifiedBy?: string;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the WebhookUpdateParameters class.
 * @constructor
 * The parameters supplied to the update webhook operation.
 *
 * @member {string} [name] Gets or sets the name of the webhook.
 * @member {boolean} [isEnabled] Gets or sets the value of the enabled flag of
 * webhook.
 * @member {string} [runOn] Gets or sets the name of the hybrid worker group
 * the webhook job will run on.
 * @member {object} [parameters] Gets or sets the parameters of the job.
 * @member {string} [description] Gets or sets the description of the webhook.
 */
export interface WebhookUpdateParameters {
  name?: string;
  isEnabled?: boolean;
  runOn?: string;
  parameters?: { [propertyName: string]: string };
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the WebhookCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update webhook operation.
 *
 * @member {string} name Gets or sets the name of the webhook.
 * @member {boolean} [isEnabled] Gets or sets the value of the enabled flag of
 * webhook.
 * @member {string} [uri] Gets or sets the uri.
 * @member {date} [expiryTime] Gets or sets the expiry time.
 * @member {object} [parameters] Gets or sets the parameters of the job.
 * @member {object} [runbook] Gets or sets the runbook.
 * @member {string} [runbook.name] Gets or sets the name of the runbook.
 * @member {string} [runOn] Gets or sets the name of the hybrid worker group
 * the webhook job will run on.
 */
export interface WebhookCreateOrUpdateParameters {
  name: string;
  isEnabled?: boolean;
  uri?: string;
  expiryTime?: Date;
  parameters?: { [propertyName: string]: string };
  runbook?: RunbookAssociationProperty;
  runOn?: string;
}

/**
 * @class
 * Initializes a new instance of the Watcher class.
 * @constructor
 * Definition of the watcher type.
 *
 * @member {number} [executionFrequencyInSeconds] Gets or sets the frequency at
 * which the watcher is invoked.
 * @member {string} [scriptName] Gets or sets the name of the script the
 * watcher is attached to, i.e. the name of an existing runbook.
 * @member {object} [scriptParameters] Gets or sets the parameters of the
 * script.
 * @member {string} [scriptRunOn] Gets or sets the name of the hybrid worker
 * group the watcher will run on.
 * @member {string} [status] Gets the current status of the watcher.
 * @member {date} [creationTime] Gets or sets the creation time.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time.
 * @member {string} [lastModifiedBy] Details of the user who last modified the
 * watcher.
 * @member {string} [description] Gets or sets the description.
 * @member {string} [etag] Gets or sets the etag of the resource.
 */
export interface Watcher extends TrackedResource {
  executionFrequencyInSeconds?: number;
  scriptName?: string;
  scriptParameters?: { [propertyName: string]: string };
  scriptRunOn?: string;
  readonly status?: string;
  readonly creationTime?: Date;
  readonly lastModifiedTime?: Date;
  readonly lastModifiedBy?: string;
  description?: string;
  etag?: string;
}

/**
 * @class
 * Initializes a new instance of the WatcherUpdateParameters class.
 * @constructor
 * @member {number} [executionFrequencyInSeconds] Gets or sets the frequency at
 * which the watcher is invoked.
 * @member {string} [name] Gets or sets the name of the resource.
 */
export interface WatcherUpdateParameters {
  executionFrequencyInSeconds?: number;
  name?: string;
}

/**
 * @class
 * Initializes a new instance of the WindowsProperties class.
 * @constructor
 * Windows specific update configuration.
 *
 * @member {string} [includedUpdateClassifications] Update classification
 * included in the software update configuration. A comma separated string with
 * required values. Possible values include: 'Unclassified', 'Critical',
 * 'Security', 'UpdateRollup', 'FeaturePack', 'ServicePack', 'Definition',
 * 'Tools', 'Updates'
 * @member {array} [excludedKbNumbers] KB numbers excluded from the software
 * update configuration.
 * @member {array} [includedKbNumbers] KB numbers included from the software
 * update configuration.
 * @member {string} [rebootSetting] Reboot setting for the software update
 * configuration.
 */
export interface WindowsProperties {
  includedUpdateClassifications?: string;
  excludedKbNumbers?: string[];
  includedKbNumbers?: string[];
  rebootSetting?: string;
}

/**
 * @class
 * Initializes a new instance of the LinuxProperties class.
 * @constructor
 * Linux specific update configuration.
 *
 * @member {string} [includedPackageClassifications] Update classifications
 * included in the software update configuration. Possible values include:
 * 'Unclassified', 'Critical', 'Security', 'Other'
 * @member {array} [excludedPackageNameMasks] packages excluded from the
 * software update configuration.
 * @member {array} [includedPackageNameMasks] packages included from the
 * software update configuration.
 * @member {string} [rebootSetting] Reboot setting for the software update
 * configuration.
 */
export interface LinuxProperties {
  includedPackageClassifications?: string;
  excludedPackageNameMasks?: string[];
  includedPackageNameMasks?: string[];
  rebootSetting?: string;
}

/**
 * @class
 * Initializes a new instance of the TagSettingsProperties class.
 * @constructor
 * Tag filter information for the VM.
 *
 * @member {object} [tags] Dictionary of tags with its list of values.
 * @member {string} [filterOperator] Filter VMs by Any or All specified tags.
 * Possible values include: 'All', 'Any'
 */
export interface TagSettingsProperties {
  tags?: { [propertyName: string]: string[] };
  filterOperator?: string;
}

/**
 * @class
 * Initializes a new instance of the AzureQueryProperties class.
 * @constructor
 * Azure query for the update configuration.
 *
 * @member {array} [scope] List of Subscription or Resource Group ARM Ids.
 * @member {array} [locations] List of locations to scope the query to.
 * @member {object} [tagSettings] Tag settings for the VM.
 * @member {object} [tagSettings.tags] Dictionary of tags with its list of
 * values.
 * @member {string} [tagSettings.filterOperator] Filter VMs by Any or All
 * specified tags. Possible values include: 'All', 'Any'
 */
export interface AzureQueryProperties {
  scope?: string[];
  locations?: string[];
  tagSettings?: TagSettingsProperties;
}

/**
 * @class
 * Initializes a new instance of the TargetProperties class.
 * @constructor
 * Group specific to the update configuration.
 *
 * @member {array} [azureQueries] List of Azure queries in the software update
 * configuration.
 */
export interface TargetProperties {
  azureQueries?: AzureQueryProperties[];
}

/**
 * @class
 * Initializes a new instance of the UpdateConfiguration class.
 * @constructor
 * Update specifc properties of the software update configuration.
 *
 * @member {string} operatingSystem operating system of target machines.
 * Possible values include: 'Windows', 'Linux'
 * @member {object} [windows] Windows specific update configuration.
 * @member {string} [windows.includedUpdateClassifications] Update
 * classification included in the software update configuration. A comma
 * separated string with required values. Possible values include:
 * 'Unclassified', 'Critical', 'Security', 'UpdateRollup', 'FeaturePack',
 * 'ServicePack', 'Definition', 'Tools', 'Updates'
 * @member {array} [windows.excludedKbNumbers] KB numbers excluded from the
 * software update configuration.
 * @member {array} [windows.includedKbNumbers] KB numbers included from the
 * software update configuration.
 * @member {string} [windows.rebootSetting] Reboot setting for the software
 * update configuration.
 * @member {object} [linux] Linux specific update configuration.
 * @member {string} [linux.includedPackageClassifications] Update
 * classifications included in the software update configuration. Possible
 * values include: 'Unclassified', 'Critical', 'Security', 'Other'
 * @member {array} [linux.excludedPackageNameMasks] packages excluded from the
 * software update configuration.
 * @member {array} [linux.includedPackageNameMasks] packages included from the
 * software update configuration.
 * @member {string} [linux.rebootSetting] Reboot setting for the software
 * update configuration.
 * @member {moment.duration} [duration] Maximum time allowed for the software
 * update configuration run. Duration needs to be specified using the format
 * PT[n]H[n]M[n]S as per ISO8601
 * @member {array} [azureVirtualMachines] List of azure resource Ids for azure
 * virtual machines targeted by the software update configuration.
 * @member {array} [nonAzureComputerNames] List of names of non-azure machines
 * targeted by the software update configuration.
 * @member {object} [targets] Group targets for the software update
 * configuration.
 * @member {array} [targets.azureQueries] List of Azure queries in the software
 * update configuration.
 */
export interface UpdateConfiguration {
  operatingSystem: string;
  windows?: WindowsProperties;
  linux?: LinuxProperties;
  duration?: moment.Duration;
  azureVirtualMachines?: string[];
  nonAzureComputerNames?: string[];
  targets?: TargetProperties;
}

/**
 * @class
 * Initializes a new instance of the TaskProperties class.
 * @constructor
 * Task properties of the software update configuration.
 *
 * @member {object} [parameters] Gets or sets the parameters of the task.
 * @member {string} [source] Gets or sets the name of the runbook.
 */
export interface TaskProperties {
  parameters?: { [propertyName: string]: string };
  source?: string;
}

/**
 * @class
 * Initializes a new instance of the SoftwareUpdateConfigurationTasks class.
 * @constructor
 * Task properties of the software update configuration.
 *
 * @member {object} [preTask] Pre task properties.
 * @member {object} [preTask.parameters] Gets or sets the parameters of the
 * task.
 * @member {string} [preTask.source] Gets or sets the name of the runbook.
 * @member {object} [postTask] Post task properties.
 * @member {object} [postTask.parameters] Gets or sets the parameters of the
 * task.
 * @member {string} [postTask.source] Gets or sets the name of the runbook.
 */
export interface SoftwareUpdateConfigurationTasks {
  preTask?: TaskProperties;
  postTask?: TaskProperties;
}

/**
 * @class
 * Initializes a new instance of the SoftwareUpdateConfiguration class.
 * @constructor
 * Software update configuration properties.
 *
 * @member {string} [name] Resource name.
 * @member {string} [id] Resource Id.
 * @member {string} [type] Resource type
 * @member {object} updateConfiguration update specific properties for the
 * Software update configuration
 * @member {string} [updateConfiguration.operatingSystem] operating system of
 * target machines. Possible values include: 'Windows', 'Linux'
 * @member {object} [updateConfiguration.windows] Windows specific update
 * configuration.
 * @member {string} [updateConfiguration.windows.includedUpdateClassifications]
 * Update classification included in the software update configuration. A comma
 * separated string with required values. Possible values include:
 * 'Unclassified', 'Critical', 'Security', 'UpdateRollup', 'FeaturePack',
 * 'ServicePack', 'Definition', 'Tools', 'Updates'
 * @member {array} [updateConfiguration.windows.excludedKbNumbers] KB numbers
 * excluded from the software update configuration.
 * @member {array} [updateConfiguration.windows.includedKbNumbers] KB numbers
 * included from the software update configuration.
 * @member {string} [updateConfiguration.windows.rebootSetting] Reboot setting
 * for the software update configuration.
 * @member {object} [updateConfiguration.linux] Linux specific update
 * configuration.
 * @member {string} [updateConfiguration.linux.includedPackageClassifications]
 * Update classifications included in the software update configuration.
 * Possible values include: 'Unclassified', 'Critical', 'Security', 'Other'
 * @member {array} [updateConfiguration.linux.excludedPackageNameMasks]
 * packages excluded from the software update configuration.
 * @member {array} [updateConfiguration.linux.includedPackageNameMasks]
 * packages included from the software update configuration.
 * @member {string} [updateConfiguration.linux.rebootSetting] Reboot setting
 * for the software update configuration.
 * @member {moment.duration} [updateConfiguration.duration] Maximum time
 * allowed for the software update configuration run. Duration needs to be
 * specified using the format PT[n]H[n]M[n]S as per ISO8601
 * @member {array} [updateConfiguration.azureVirtualMachines] List of azure
 * resource Ids for azure virtual machines targeted by the software update
 * configuration.
 * @member {array} [updateConfiguration.nonAzureComputerNames] List of names of
 * non-azure machines targeted by the software update configuration.
 * @member {object} [updateConfiguration.targets] Group targets for the
 * software update configuration.
 * @member {array} [updateConfiguration.targets.azureQueries] List of Azure
 * queries in the software update configuration.
 * @member {object} scheduleInfo Schedule information for the Software update
 * configuration
 * @member {date} [scheduleInfo.startTime] Gets or sets the start time of the
 * schedule.
 * @member {number} [scheduleInfo.startTimeOffsetMinutes] Gets the start time's
 * offset in minutes.
 * @member {date} [scheduleInfo.expiryTime] Gets or sets the end time of the
 * schedule.
 * @member {number} [scheduleInfo.expiryTimeOffsetMinutes] Gets or sets the
 * expiry time's offset in minutes.
 * @member {boolean} [scheduleInfo.isEnabled] Gets or sets a value indicating
 * whether this schedule is enabled.
 * @member {date} [scheduleInfo.nextRun] Gets or sets the next run time of the
 * schedule.
 * @member {number} [scheduleInfo.nextRunOffsetMinutes] Gets or sets the next
 * run time's offset in minutes.
 * @member {number} [scheduleInfo.interval] Gets or sets the interval of the
 * schedule.
 * @member {string} [scheduleInfo.frequency] Gets or sets the frequency of the
 * schedule. Possible values include: 'OneTime', 'Day', 'Hour', 'Week', 'Month'
 * @member {string} [scheduleInfo.timeZone] Gets or sets the time zone of the
 * schedule.
 * @member {object} [scheduleInfo.advancedSchedule] Gets or sets the advanced
 * schedule.
 * @member {array} [scheduleInfo.advancedSchedule.weekDays] Days of the week
 * that the job should execute on.
 * @member {array} [scheduleInfo.advancedSchedule.monthDays] Days of the month
 * that the job should execute on. Must be between 1 and 31.
 * @member {array} [scheduleInfo.advancedSchedule.monthlyOccurrences]
 * Occurrences of days within a month.
 * @member {date} [scheduleInfo.creationTime] Gets or sets the creation time.
 * @member {date} [scheduleInfo.lastModifiedTime] Gets or sets the last
 * modified time.
 * @member {string} [scheduleInfo.description] Gets or sets the description.
 * @member {string} [provisioningState] Provisioning state for the software
 * update configuration, which only appears in the response.
 * @member {object} [error] Details of provisioning error
 * @member {string} [error.code] Error code
 * @member {string} [error.message] Error message indicating why the operation
 * failed.
 * @member {date} [creationTime] Creation time of the resource, which only
 * appears in the response.
 * @member {string} [createdBy] CreatedBy property, which only appears in the
 * response.
 * @member {date} [lastModifiedTime] Last time resource was modified, which
 * only appears in the response.
 * @member {string} [lastModifiedBy] LastModifiedBy property, which only
 * appears in the response.
 * @member {object} [tasks] Tasks information for the Software update
 * configuration.
 * @member {object} [tasks.preTask] Pre task properties.
 * @member {object} [tasks.preTask.parameters] Gets or sets the parameters of
 * the task.
 * @member {string} [tasks.preTask.source] Gets or sets the name of the
 * runbook.
 * @member {object} [tasks.postTask] Post task properties.
 * @member {object} [tasks.postTask.parameters] Gets or sets the parameters of
 * the task.
 * @member {string} [tasks.postTask.source] Gets or sets the name of the
 * runbook.
 */
export interface SoftwareUpdateConfiguration extends BaseResource {
  readonly name?: string;
  readonly id?: string;
  readonly type?: string;
  updateConfiguration: UpdateConfiguration;
  scheduleInfo: ScheduleProperties;
  readonly provisioningState?: string;
  error?: ErrorResponse;
  readonly creationTime?: Date;
  readonly createdBy?: string;
  readonly lastModifiedTime?: Date;
  readonly lastModifiedBy?: string;
  tasks?: SoftwareUpdateConfigurationTasks;
}

/**
 * @class
 * Initializes a new instance of the CollectionItemUpdateConfiguration class.
 * @constructor
 * object returned when requesting a collection of software update
 * configuration
 *
 * @member {array} [azureVirtualMachines] List of azure resource Ids for azure
 * virtual machines targeted by the software update configuration.
 * @member {moment.duration} [duration] Maximum time allowed for the software
 * update configuration run. Duration needs to be specified using the format
 * PT[n]H[n]M[n]S as per ISO8601
 */
export interface CollectionItemUpdateConfiguration {
  azureVirtualMachines?: string[];
  duration?: moment.Duration;
}

/**
 * @class
 * Initializes a new instance of the SoftwareUpdateConfigurationCollectionItem class.
 * @constructor
 * Software update configuration collection item properties.
 *
 * @member {string} [name] Name of the software update configuration.
 * @member {string} [id] Resource Id of the software update configuration
 * @member {object} [updateConfiguration] Update specific properties of the
 * software update configuration.
 * @member {array} [updateConfiguration.azureVirtualMachines] List of azure
 * resource Ids for azure virtual machines targeted by the software update
 * configuration.
 * @member {moment.duration} [updateConfiguration.duration] Maximum time
 * allowed for the software update configuration run. Duration needs to be
 * specified using the format PT[n]H[n]M[n]S as per ISO8601
 * @member {string} [frequency] execution frequency of the schedule associated
 * with the software update configuration. Possible values include: 'OneTime',
 * 'Day', 'Hour', 'Week', 'Month'
 * @member {date} [startTime] the start time of the update.
 * @member {date} [creationTime] Creation time of the software update
 * configuration, which only appears in the response.
 * @member {date} [lastModifiedTime] Last time software update configuration
 * was modified, which only appears in the response.
 * @member {string} [provisioningState] Provisioning state for the software
 * update configuration, which only appears in the response.
 * @member {date} [nextRun] ext run time of the update.
 */
export interface SoftwareUpdateConfigurationCollectionItem {
  readonly name?: string;
  readonly id?: string;
  updateConfiguration?: CollectionItemUpdateConfiguration;
  frequency?: string;
  startTime?: Date;
  readonly creationTime?: Date;
  readonly lastModifiedTime?: Date;
  readonly provisioningState?: string;
  nextRun?: Date;
}

/**
 * @class
 * Initializes a new instance of the SoftwareUpdateConfigurationListResult class.
 * @constructor
 * result of listing all software update configuration
 *
 * @member {array} [value] outer object returned when listing all software
 * update configurations
 */
export interface SoftwareUpdateConfigurationListResult {
  value?: SoftwareUpdateConfigurationCollectionItem[];
}

/**
 * @class
 * Initializes a new instance of the UpdateConfigurationNavigation class.
 * @constructor
 * Software update configuration Run Navigation model.
 *
 * @member {string} [name] Name of the software update configuration triggered
 * the software update configuration run
 */
export interface UpdateConfigurationNavigation {
  readonly name?: string;
}

/**
 * @class
 * Initializes a new instance of the SoftareUpdateConfigurationRunTaskProperties class.
 * @constructor
 * Task properties of the software update configuration.
 *
 * @member {string} [status] The status of the task.
 * @member {string} [source] The name of the source of the task.
 * @member {string} [jobId] The job id of the task.
 */
export interface SoftareUpdateConfigurationRunTaskProperties {
  status?: string;
  source?: string;
  jobId?: string;
}

/**
 * @class
 * Initializes a new instance of the SoftareUpdateConfigurationRunTasks class.
 * @constructor
 * Software update configuration run tasks model.
 *
 * @member {object} [preTask] Pre task properties.
 * @member {string} [preTask.status] The status of the task.
 * @member {string} [preTask.source] The name of the source of the task.
 * @member {string} [preTask.jobId] The job id of the task.
 * @member {object} [postTask] Post task properties.
 * @member {string} [postTask.status] The status of the task.
 * @member {string} [postTask.source] The name of the source of the task.
 * @member {string} [postTask.jobId] The job id of the task.
 */
export interface SoftareUpdateConfigurationRunTasks {
  preTask?: SoftareUpdateConfigurationRunTaskProperties;
  postTask?: SoftareUpdateConfigurationRunTaskProperties;
}

/**
 * @class
 * Initializes a new instance of the SoftwareUpdateConfigurationRun class.
 * @constructor
 * Software update configuration Run properties.
 *
 * @member {string} [name] Name of the software update configuration run.
 * @member {string} [id] Resource Id of the software update configuration run
 * @member {object} [softwareUpdateConfiguration] software update configuration
 * triggered this run
 * @member {string} [softwareUpdateConfiguration.name] Name of the software
 * update configuration triggered the software update configuration run
 * @member {string} [status] Status of the software update configuration run.
 * @member {string} [configuredDuration] Configured duration for the software
 * update configuration run.
 * @member {string} [osType] Operating system target of the software update
 * configuration triggered this run
 * @member {date} [startTime] Start time of the software update configuration
 * run.
 * @member {date} [endTime] End time of the software update configuration run.
 * @member {number} [computerCount] Number of computers in the software update
 * configuration run.
 * @member {number} [failedCount] Number of computers with failed status.
 * @member {date} [creationTime] Creation time of theresource, which only
 * appears in the response.
 * @member {string} [createdBy] CreatedBy property, which only appears in the
 * response.
 * @member {date} [lastModifiedTime] Last time resource was modified, which
 * only appears in the response.
 * @member {string} [lastModifiedBy] LastModifiedBy property, which only
 * appears in the response.
 * @member {object} [tasks] Software update configuration tasks triggered in
 * this run
 * @member {object} [tasks.preTask] Pre task properties.
 * @member {string} [tasks.preTask.status] The status of the task.
 * @member {string} [tasks.preTask.source] The name of the source of the task.
 * @member {string} [tasks.preTask.jobId] The job id of the task.
 * @member {object} [tasks.postTask] Post task properties.
 * @member {string} [tasks.postTask.status] The status of the task.
 * @member {string} [tasks.postTask.source] The name of the source of the task.
 * @member {string} [tasks.postTask.jobId] The job id of the task.
 */
export interface SoftwareUpdateConfigurationRun {
  readonly name?: string;
  readonly id?: string;
  softwareUpdateConfiguration?: UpdateConfigurationNavigation;
  readonly status?: string;
  readonly configuredDuration?: string;
  readonly osType?: string;
  readonly startTime?: Date;
  readonly endTime?: Date;
  readonly computerCount?: number;
  readonly failedCount?: number;
  readonly creationTime?: Date;
  readonly createdBy?: string;
  readonly lastModifiedTime?: Date;
  readonly lastModifiedBy?: string;
  tasks?: SoftareUpdateConfigurationRunTasks;
}

/**
 * @class
 * Initializes a new instance of the SoftwareUpdateConfigurationRunListResult class.
 * @constructor
 * result of listing all software update configuration runs
 *
 * @member {array} [value] outer object returned when listing all software
 * update configuration runs
 * @member {string} [nextLink] link to next page of results.
 */
export interface SoftwareUpdateConfigurationRunListResult {
  value?: SoftwareUpdateConfigurationRun[];
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the JobNavigation class.
 * @constructor
 * Software update configuration machine run job navigation properties.
 *
 * @member {string} [id] Id of the job associated with the software update
 * configuration run
 */
export interface JobNavigation {
  readonly id?: string;
}

/**
 * @class
 * Initializes a new instance of the SoftwareUpdateConfigurationMachineRun class.
 * @constructor
 * Software update configuration machine run model.
 *
 * @member {string} [name] Name of the software update configuration machine
 * run
 * @member {string} [id] Resource Id of the software update configuration
 * machine run
 * @member {string} [targetComputer] name of the updated computer
 * @member {string} [targetComputerType] type of the updated computer.
 * @member {object} [softwareUpdateConfiguration] software update configuration
 * triggered this run
 * @member {string} [softwareUpdateConfiguration.name] Name of the software
 * update configuration triggered the software update configuration run
 * @member {string} [status] Status of the software update configuration
 * machine run.
 * @member {string} [osType] Operating system target of the software update
 * configuration triggered this run
 * @member {uuid} [correlationId] correlation id of the software update
 * configuration machine run
 * @member {uuid} [sourceComputerId] source computer id of the software update
 * configuration machine run
 * @member {date} [startTime] Start time of the software update configuration
 * machine run.
 * @member {date} [endTime] End time of the software update configuration
 * machine run.
 * @member {string} [configuredDuration] configured duration for the software
 * update configuration run.
 * @member {object} [job] Job associated with the software update configuration
 * machine run
 * @member {string} [job.id] Id of the job associated with the software update
 * configuration run
 * @member {date} [creationTime] Creation time of theresource, which only
 * appears in the response.
 * @member {string} [createdBy] createdBy property, which only appears in the
 * response.
 * @member {date} [lastModifiedTime] Last time resource was modified, which
 * only appears in the response.
 * @member {string} [lastModifiedBy] lastModifiedBy property, which only
 * appears in the response.
 * @member {object} [error] detailes of provisioning error
 * @member {string} [error.code] Error code
 * @member {string} [error.message] Error message indicating why the operation
 * failed.
 */
export interface SoftwareUpdateConfigurationMachineRun {
  readonly name?: string;
  readonly id?: string;
  readonly targetComputer?: string;
  readonly targetComputerType?: string;
  softwareUpdateConfiguration?: UpdateConfigurationNavigation;
  readonly status?: string;
  readonly osType?: string;
  readonly correlationId?: string;
  readonly sourceComputerId?: string;
  readonly startTime?: Date;
  readonly endTime?: Date;
  readonly configuredDuration?: string;
  job?: JobNavigation;
  readonly creationTime?: Date;
  readonly createdBy?: string;
  readonly lastModifiedTime?: Date;
  readonly lastModifiedBy?: string;
  error?: ErrorResponse;
}

/**
 * @class
 * Initializes a new instance of the SoftwareUpdateConfigurationMachineRunListResult class.
 * @constructor
 * result of listing all software update configuration machine runs
 *
 * @member {array} [value] outer object returned when listing all software
 * update configuration machine runs
 * @member {string} [nextLink] link to next page of results.
 */
export interface SoftwareUpdateConfigurationMachineRunListResult {
  value?: SoftwareUpdateConfigurationMachineRun[];
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the SourceControl class.
 * @constructor
 * Definition of the source control.
 *
 * @member {string} [repoUrl] The repo url of the source control.
 * @member {string} [branch] The repo branch of the source control. Include
 * branch as empty string for VsoTfvc.
 * @member {string} [folderPath] The folder path of the source control.
 * @member {boolean} [autoSync] The auto sync of the source control. Default is
 * false.
 * @member {boolean} [publishRunbook] The auto publish of the source control.
 * Default is true.
 * @member {string} [sourceType] The source type. Must be one of VsoGit,
 * VsoTfvc, GitHub. Possible values include: 'VsoGit', 'VsoTfvc', 'GitHub'
 * @member {string} [description] The description.
 * @member {date} [creationTime] The creation time.
 * @member {date} [lastModifiedTime] The last modified time.
 */
export interface SourceControl extends ProxyResource {
  repoUrl?: string;
  branch?: string;
  folderPath?: string;
  autoSync?: boolean;
  publishRunbook?: boolean;
  sourceType?: string;
  description?: string;
  creationTime?: Date;
  lastModifiedTime?: Date;
}

/**
 * @class
 * Initializes a new instance of the SourceControlSecurityTokenProperties class.
 * @constructor
 * @member {string} [accessToken] The access token.
 * @member {string} [refreshToken] The refresh token.
 * @member {string} [tokenType] The token type. Must be either
 * PersonalAccessToken or Oauth. Possible values include:
 * 'PersonalAccessToken', 'Oauth'
 */
export interface SourceControlSecurityTokenProperties {
  accessToken?: string;
  refreshToken?: string;
  tokenType?: string;
}

/**
 * @class
 * Initializes a new instance of the SourceControlUpdateParameters class.
 * @constructor
 * The parameters supplied to the update source control operation.
 *
 * @member {string} [branch] The repo branch of the source control.
 * @member {string} [folderPath] The folder path of the source control. Path
 * must be relative.
 * @member {boolean} [autoSync] The auto sync of the source control. Default is
 * false.
 * @member {boolean} [publishRunbook] The auto publish of the source control.
 * Default is true.
 * @member {object} [securityToken] The authorization token for the repo of the
 * source control.
 * @member {string} [securityToken.accessToken] The access token.
 * @member {string} [securityToken.refreshToken] The refresh token.
 * @member {string} [securityToken.tokenType] The token type. Must be either
 * PersonalAccessToken or Oauth. Possible values include:
 * 'PersonalAccessToken', 'Oauth'
 * @member {string} [description] The user description of the source control.
 */
export interface SourceControlUpdateParameters {
  branch?: string;
  folderPath?: string;
  autoSync?: boolean;
  publishRunbook?: boolean;
  securityToken?: SourceControlSecurityTokenProperties;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the SourceControlCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update source control operation.
 *
 * @member {string} [repoUrl] The repo url of the source control.
 * @member {string} [branch] The repo branch of the source control. Include
 * branch as empty string for VsoTfvc.
 * @member {string} [folderPath] The folder path of the source control. Path
 * must be relative.
 * @member {boolean} [autoSync] The auto async of the source control. Default
 * is false.
 * @member {boolean} [publishRunbook] The auto publish of the source control.
 * Default is true.
 * @member {string} [sourceType] The source type. Must be one of VsoGit,
 * VsoTfvc, GitHub, case sensitive. Possible values include: 'VsoGit',
 * 'VsoTfvc', 'GitHub'
 * @member {object} [securityToken] The authorization token for the repo of the
 * source control.
 * @member {string} [securityToken.accessToken] The access token.
 * @member {string} [securityToken.refreshToken] The refresh token.
 * @member {string} [securityToken.tokenType] The token type. Must be either
 * PersonalAccessToken or Oauth. Possible values include:
 * 'PersonalAccessToken', 'Oauth'
 * @member {string} [description] The user description of the source control.
 */
export interface SourceControlCreateOrUpdateParameters {
  repoUrl?: string;
  branch?: string;
  folderPath?: string;
  autoSync?: boolean;
  publishRunbook?: boolean;
  sourceType?: string;
  securityToken?: SourceControlSecurityTokenProperties;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the SourceControlSyncJob class.
 * @constructor
 * Definition of the source control sync job.
 *
 * @member {string} [name] Resource name.
 * @member {string} [type] Resource type.
 * @member {string} [id] Resource id.
 * @member {string} [sourceControlSyncJobId] The source control sync job id.
 * @member {date} [creationTime] The creation time of the job.
 * @member {string} [provisioningState] The provisioning state of the job.
 * Possible values include: 'Completed', 'Failed', 'Running'
 * @member {date} [startTime] The start time of the job.
 * @member {date} [endTime] The end time of the job.
 * @member {string} [syncType] The sync type. Possible values include:
 * 'PartialSync', 'FullSync'
 */
export interface SourceControlSyncJob {
  readonly name?: string;
  readonly type?: string;
  readonly id?: string;
  sourceControlSyncJobId?: string;
  readonly creationTime?: Date;
  provisioningState?: string;
  readonly startTime?: Date;
  readonly endTime?: Date;
  syncType?: string;
}

/**
 * @class
 * Initializes a new instance of the SourceControlSyncJobCreateParameters class.
 * @constructor
 * The parameters supplied to the create source control sync job operation.
 *
 * @member {string} commitId The commit id of the source control sync job. If
 * not syncing to a commitId, enter an empty string.
 */
export interface SourceControlSyncJobCreateParameters {
  commitId: string;
}

/**
 * @class
 * Initializes a new instance of the SourceControlSyncJobById class.
 * @constructor
 * Definition of the source control sync job.
 *
 * @member {string} [id] The id of the job.
 * @member {string} [sourceControlSyncJobId] The source control sync job id.
 * @member {date} [creationTime] The creation time of the job.
 * @member {string} [provisioningState] The provisioning state of the job.
 * Possible values include: 'Completed', 'Failed', 'Running'
 * @member {date} [startTime] The start time of the job.
 * @member {date} [endTime] The end time of the job.
 * @member {string} [syncType] The sync type. Possible values include:
 * 'PartialSync', 'FullSync'
 * @member {string} [exception] The exceptions that occured while running the
 * sync job.
 */
export interface SourceControlSyncJobById {
  id?: string;
  sourceControlSyncJobId?: string;
  readonly creationTime?: Date;
  provisioningState?: string;
  readonly startTime?: Date;
  readonly endTime?: Date;
  syncType?: string;
  exception?: string;
}

/**
 * @class
 * Initializes a new instance of the SourceControlSyncJobStream class.
 * @constructor
 * Definition of the source control sync job stream.
 *
 * @member {string} [id] Resource id.
 * @member {string} [sourceControlSyncJobStreamId] The sync job stream id.
 * @member {string} [summary] The summary of the sync job stream.
 * @member {date} [time] The time of the sync job stream.
 * @member {string} [streamType] The type of the sync job stream. Possible
 * values include: 'Error', 'Output'
 */
export interface SourceControlSyncJobStream {
  readonly id?: string;
  sourceControlSyncJobStreamId?: string;
  summary?: string;
  readonly time?: Date;
  streamType?: string;
}

/**
 * @class
 * Initializes a new instance of the SourceControlSyncJobStreamById class.
 * @constructor
 * Definition of the source control sync job stream by id.
 *
 * @member {string} [id] Resource id.
 * @member {string} [sourceControlSyncJobStreamId] The sync job stream id.
 * @member {string} [summary] The summary of the sync job stream.
 * @member {date} [time] The time of the sync job stream.
 * @member {string} [streamType] The type of the sync job stream. Possible
 * values include: 'Error', 'Output'
 * @member {string} [streamText] The text of the sync job stream.
 * @member {object} [value] The values of the job stream.
 */
export interface SourceControlSyncJobStreamById {
  readonly id?: string;
  sourceControlSyncJobStreamId?: string;
  summary?: string;
  readonly time?: Date;
  streamType?: string;
  streamText?: string;
  value?: { [propertyName: string]: any };
}

/**
 * @class
 * Initializes a new instance of the JobStream class.
 * @constructor
 * Definition of the job stream.
 *
 * @member {string} [id] Gets or sets the id of the resource.
 * @member {string} [jobStreamId] Gets or sets the id of the job stream.
 * @member {date} [time] Gets or sets the creation time of the job.
 * @member {string} [streamType] Gets or sets the stream type. Possible values
 * include: 'Progress', 'Output', 'Warning', 'Error', 'Debug', 'Verbose', 'Any'
 * @member {string} [streamText] Gets or sets the stream text.
 * @member {string} [summary] Gets or sets the summary.
 * @member {object} [value] Gets or sets the values of the job stream.
 */
export interface JobStream {
  id?: string;
  jobStreamId?: string;
  time?: Date;
  streamType?: string;
  streamText?: string;
  summary?: string;
  value?: { [propertyName: string]: any };
}

/**
 * @class
 * Initializes a new instance of the Job class.
 * @constructor
 * Definition of the job.
 *
 * @member {object} [runbook] Gets or sets the runbook.
 * @member {string} [runbook.name] Gets or sets the name of the runbook.
 * @member {string} [startedBy] Gets or sets the job started by.
 * @member {string} [runOn] Gets or sets the runOn which specifies the group
 * name where the job is to be executed.
 * @member {uuid} [jobId] Gets or sets the id of the job.
 * @member {date} [creationTime] Gets or sets the creation time of the job.
 * @member {string} [status] Gets or sets the status of the job. Possible
 * values include: 'New', 'Activating', 'Running', 'Completed', 'Failed',
 * 'Stopped', 'Blocked', 'Suspended', 'Disconnected', 'Suspending', 'Stopping',
 * 'Resuming', 'Removing'
 * @member {string} [statusDetails] Gets or sets the status details of the job.
 * @member {date} [startTime] Gets or sets the start time of the job.
 * @member {date} [endTime] Gets or sets the end time of the job.
 * @member {string} [exception] Gets or sets the exception of the job.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time of the
 * job.
 * @member {date} [lastStatusModifiedTime] Gets or sets the last status
 * modified time of the job.
 * @member {object} [parameters] Gets or sets the parameters of the job.
 * @member {string} [provisioningState] The current provisioning state of the
 * job. Possible values include: 'Failed', 'Succeeded', 'Suspended',
 * 'Processing'
 */
export interface Job extends ProxyResource {
  runbook?: RunbookAssociationProperty;
  startedBy?: string;
  runOn?: string;
  jobId?: string;
  creationTime?: Date;
  status?: string;
  statusDetails?: string;
  startTime?: Date;
  endTime?: Date;
  exception?: string;
  lastModifiedTime?: Date;
  lastStatusModifiedTime?: Date;
  parameters?: { [propertyName: string]: string };
  provisioningState?: string;
}

/**
 * @class
 * Initializes a new instance of the JobCollectionItem class.
 * @constructor
 * Job collection item properties.
 *
 * @member {object} [runbook] The runbook association.
 * @member {string} [runbook.name] Gets or sets the name of the runbook.
 * @member {uuid} [jobId] The id of the job.
 * @member {date} [creationTime] The creation time of the job.
 * @member {string} [status] The status of the job. Possible values include:
 * 'New', 'Activating', 'Running', 'Completed', 'Failed', 'Stopped', 'Blocked',
 * 'Suspended', 'Disconnected', 'Suspending', 'Stopping', 'Resuming',
 * 'Removing'
 * @member {date} [startTime] The start time of the job.
 * @member {date} [endTime] The end time of the job.
 * @member {date} [lastModifiedTime] The last modified time of the job.
 * @member {string} [provisioningState] The provisioning state of a resource.
 * @member {string} [runOn] Specifies the runOn group name where the job was
 * executed.
 */
export interface JobCollectionItem extends ProxyResource {
  readonly runbook?: RunbookAssociationProperty;
  readonly jobId?: string;
  readonly creationTime?: Date;
  readonly status?: string;
  readonly startTime?: Date;
  readonly endTime?: Date;
  readonly lastModifiedTime?: Date;
  readonly provisioningState?: string;
  runOn?: string;
}

/**
 * @class
 * Initializes a new instance of the JobCreateParameters class.
 * @constructor
 * The parameters supplied to the create job operation.
 *
 * @member {object} [runbook] Gets or sets the runbook.
 * @member {string} [runbook.name] Gets or sets the name of the runbook.
 * @member {object} [parameters] Gets or sets the parameters of the job.
 * @member {string} [runOn] Gets or sets the runOn which specifies the group
 * name where the job is to be executed.
 */
export interface JobCreateParameters {
  runbook?: RunbookAssociationProperty;
  parameters?: { [propertyName: string]: string };
  runOn?: string;
}

/**
 * @class
 * Initializes a new instance of the DscReportError class.
 * @constructor
 * Definition of the dsc node report error type.
 *
 * @member {string} [errorSource] Gets or sets the source of the error.
 * @member {string} [resourceId] Gets or sets the resource ID which generated
 * the error.
 * @member {string} [errorCode] Gets or sets the error code.
 * @member {string} [errorMessage] Gets or sets the error message.
 * @member {string} [locale] Gets or sets the locale of the error.
 * @member {string} [errorDetails] Gets or sets the error details.
 */
export interface DscReportError {
  errorSource?: string;
  resourceId?: string;
  errorCode?: string;
  errorMessage?: string;
  locale?: string;
  errorDetails?: string;
}

/**
 * @class
 * Initializes a new instance of the DscReportResourceNavigation class.
 * @constructor
 * Navigation for DSC Report Resource.
 *
 * @member {string} [resourceId] Gets or sets the ID of the resource to
 * navigate to.
 */
export interface DscReportResourceNavigation {
  resourceId?: string;
}

/**
 * @class
 * Initializes a new instance of the DscReportResource class.
 * @constructor
 * Definition of the DSC Report Resource.
 *
 * @member {string} [resourceId] Gets or sets the ID of the resource.
 * @member {string} [sourceInfo] Gets or sets the source info of the resource.
 * @member {array} [dependsOn] Gets or sets the Resource Navigation values for
 * resources the resource depends on.
 * @member {string} [moduleName] Gets or sets the module name of the resource.
 * @member {string} [moduleVersion] Gets or sets the module version of the
 * resource.
 * @member {string} [resourceName] Gets or sets the name of the resource.
 * @member {string} [error] Gets or sets the error of the resource.
 * @member {string} [status] Gets or sets the status of the resource.
 * @member {number} [durationInSeconds] Gets or sets the duration in seconds
 * for the resource.
 * @member {date} [startDate] Gets or sets the start date of the resource.
 */
export interface DscReportResource {
  resourceId?: string;
  sourceInfo?: string;
  dependsOn?: DscReportResourceNavigation[];
  moduleName?: string;
  moduleVersion?: string;
  resourceName?: string;
  error?: string;
  status?: string;
  durationInSeconds?: number;
  startDate?: Date;
}

/**
 * @class
 * Initializes a new instance of the DscMetaConfiguration class.
 * @constructor
 * Definition of the DSC Meta Configuration.
 *
 * @member {number} [configurationModeFrequencyMins] Gets or sets the
 * ConfigurationModeFrequencyMins value of the meta configuration.
 * @member {boolean} [rebootNodeIfNeeded] Gets or sets the RebootNodeIfNeeded
 * value of the meta configuration.
 * @member {string} [configurationMode] Gets or sets the ConfigurationMode
 * value of the meta configuration.
 * @member {string} [actionAfterReboot] Gets or sets the ActionAfterReboot
 * value of the meta configuration.
 * @member {string} [certificateId] Gets or sets the CertificateId value of the
 * meta configuration.
 * @member {number} [refreshFrequencyMins] Gets or sets the
 * RefreshFrequencyMins value of the meta configuration.
 * @member {boolean} [allowModuleOverwrite] Gets or sets the
 * AllowModuleOverwrite value of the meta configuration.
 */
export interface DscMetaConfiguration {
  configurationModeFrequencyMins?: number;
  rebootNodeIfNeeded?: boolean;
  configurationMode?: string;
  actionAfterReboot?: string;
  certificateId?: string;
  refreshFrequencyMins?: number;
  allowModuleOverwrite?: boolean;
}

/**
 * @class
 * Initializes a new instance of the DscNodeReport class.
 * @constructor
 * Definition of the dsc node report type.
 *
 * @member {date} [endTime] Gets or sets the end time of the node report.
 * @member {date} [lastModifiedTime] Gets or sets the lastModifiedTime of the
 * node report.
 * @member {date} [startTime] Gets or sets the start time of the node report.
 * @member {string} [type] Gets or sets the type of the node report.
 * @member {string} [reportId] Gets or sets the id of the node report.
 * @member {string} [status] Gets or sets the status of the node report.
 * @member {string} [refreshMode] Gets or sets the refreshMode of the node
 * report.
 * @member {string} [rebootRequested] Gets or sets the rebootRequested of the
 * node report.
 * @member {string} [reportFormatVersion] Gets or sets the reportFormatVersion
 * of the node report.
 * @member {string} [configurationVersion] Gets or sets the
 * configurationVersion of the node report.
 * @member {string} [id] Gets or sets the id.
 * @member {array} [errors] Gets or sets the errors for the node report.
 * @member {array} [resources] Gets or sets the resource for the node report.
 * @member {object} [metaConfiguration] Gets or sets the metaConfiguration of
 * the node at the time of the report.
 * @member {number} [metaConfiguration.configurationModeFrequencyMins] Gets or
 * sets the ConfigurationModeFrequencyMins value of the meta configuration.
 * @member {boolean} [metaConfiguration.rebootNodeIfNeeded] Gets or sets the
 * RebootNodeIfNeeded value of the meta configuration.
 * @member {string} [metaConfiguration.configurationMode] Gets or sets the
 * ConfigurationMode value of the meta configuration.
 * @member {string} [metaConfiguration.actionAfterReboot] Gets or sets the
 * ActionAfterReboot value of the meta configuration.
 * @member {string} [metaConfiguration.certificateId] Gets or sets the
 * CertificateId value of the meta configuration.
 * @member {number} [metaConfiguration.refreshFrequencyMins] Gets or sets the
 * RefreshFrequencyMins value of the meta configuration.
 * @member {boolean} [metaConfiguration.allowModuleOverwrite] Gets or sets the
 * AllowModuleOverwrite value of the meta configuration.
 * @member {string} [hostName] Gets or sets the hostname of the node that sent
 * the report.
 * @member {array} [iPV4Addresses] Gets or sets the IPv4 address of the node
 * that sent the report.
 * @member {array} [iPV6Addresses] Gets or sets the IPv6 address of the node
 * that sent the report.
 * @member {number} [numberOfResources] Gets or sets the number of resource in
 * the node report.
 * @member {string} [rawErrors] Gets or sets the unparsed errors for the node
 * report.
 */
export interface DscNodeReport {
  endTime?: Date;
  lastModifiedTime?: Date;
  startTime?: Date;
  type?: string;
  reportId?: string;
  status?: string;
  refreshMode?: string;
  rebootRequested?: string;
  reportFormatVersion?: string;
  configurationVersion?: string;
  id?: string;
  errors?: DscReportError[];
  resources?: DscReportResource[];
  metaConfiguration?: DscMetaConfiguration;
  hostName?: string;
  iPV4Addresses?: string[];
  iPV6Addresses?: string[];
  numberOfResources?: number;
  rawErrors?: string;
}

/**
 * @class
 * Initializes a new instance of the AgentRegistrationKeys class.
 * @constructor
 * Definition of the agent registration keys.
 *
 * @member {string} [primary] Gets or sets the primary key.
 * @member {string} [secondary] Gets or sets the secondary key.
 */
export interface AgentRegistrationKeys {
  primary?: string;
  secondary?: string;
}

/**
 * @class
 * Initializes a new instance of the AgentRegistration class.
 * @constructor
 * Definition of the agent registration infomration type.
 *
 * @member {string} [dscMetaConfiguration] Gets or sets the dsc meta
 * configuration.
 * @member {string} [endpoint] Gets or sets the dsc server endpoint.
 * @member {object} [keys] Gets or sets the agent registration keys.
 * @member {string} [keys.primary] Gets or sets the primary key.
 * @member {string} [keys.secondary] Gets or sets the secondary key.
 * @member {string} [id] Gets or sets the id.
 */
export interface AgentRegistration {
  dscMetaConfiguration?: string;
  endpoint?: string;
  keys?: AgentRegistrationKeys;
  id?: string;
}

/**
 * @class
 * Initializes a new instance of the DscNodeExtensionHandlerAssociationProperty class.
 * @constructor
 * The dsc extensionHandler property associated with the node
 *
 * @member {string} [name] Gets or sets the name of the extension handler.
 * @member {string} [version] Gets or sets the version of the extension
 * handler.
 */
export interface DscNodeExtensionHandlerAssociationProperty {
  name?: string;
  version?: string;
}

/**
 * @class
 * Initializes a new instance of the DscNode class.
 * @constructor
 * Definition of a DscNode
 *
 * @member {date} [lastSeen] Gets or sets the last seen time of the node.
 * @member {date} [registrationTime] Gets or sets the registration time of the
 * node.
 * @member {string} [ip] Gets or sets the ip of the node.
 * @member {string} [accountId] Gets or sets the account id of the node.
 * @member {string} [dscNodeName] Gets or sets the name of the dsc
 * nodeconfiguration.
 * @member {string} [status] Gets or sets the status of the node.
 * @member {string} [nodeId] Gets or sets the node id.
 * @member {string} [etag] Gets or sets the etag of the resource.
 * @member {number} [totalCount] Gets the total number of records matching
 * filter criteria.
 * @member {array} [extensionHandler] Gets or sets the list of extensionHandler
 * properties for a Node.
 */
export interface DscNode extends ProxyResource {
  lastSeen?: Date;
  registrationTime?: Date;
  ip?: string;
  accountId?: string;
  dscNodeName?: string;
  status?: string;
  nodeId?: string;
  etag?: string;
  totalCount?: number;
  extensionHandler?: DscNodeExtensionHandlerAssociationProperty[];
}

/**
 * @class
 * Initializes a new instance of the AgentRegistrationRegenerateKeyParameter class.
 * @constructor
 * The parameters supplied to the regenerate keys operation.
 *
 * @member {string} keyName Gets or sets the agent registration key name -
 * primary or secondary. Possible values include: 'primary', 'secondary'
 */
export interface AgentRegistrationRegenerateKeyParameter {
  keyName: string;
}

/**
 * @class
 * Initializes a new instance of the DscNodeUpdateParametersProperties class.
 * @constructor
 * @member {string} [name] Gets or sets the name of the dsc nodeconfiguration.
 */
export interface DscNodeUpdateParametersProperties {
  name?: string;
}

/**
 * @class
 * Initializes a new instance of the DscNodeUpdateParameters class.
 * @constructor
 * The parameters supplied to the update dsc node operation.
 *
 * @member {string} [nodeId] Gets or sets the id of the dsc node.
 * @member {object} [properties]
 * @member {string} [properties.name] Gets or sets the name of the dsc
 * nodeconfiguration.
 */
export interface DscNodeUpdateParameters {
  nodeId?: string;
  properties?: DscNodeUpdateParametersProperties;
}

/**
 * @class
 * Initializes a new instance of the DscConfigurationAssociationProperty class.
 * @constructor
 * The Dsc configuration property associated with the entity.
 *
 * @member {string} [name] Gets or sets the name of the Dsc configuration.
 */
export interface DscConfigurationAssociationProperty {
  name?: string;
}

/**
 * @class
 * Initializes a new instance of the DscCompilationJob class.
 * @constructor
 * Definition of the Dsc Compilation job.
 *
 * @member {object} [configuration] Gets or sets the configuration.
 * @member {string} [configuration.name] Gets or sets the name of the Dsc
 * configuration.
 * @member {string} [startedBy] Gets the compilation job started by.
 * @member {uuid} [jobId] Gets the id of the job.
 * @member {date} [creationTime] Gets the creation time of the job.
 * @member {string} [provisioningState] The current provisioning state of the
 * job. Possible values include: 'Failed', 'Succeeded', 'Suspended',
 * 'Processing'
 * @member {string} [runOn] Gets or sets the runOn which specifies the group
 * name where the job is to be executed.
 * @member {string} [status] Gets or sets the status of the job. Possible
 * values include: 'New', 'Activating', 'Running', 'Completed', 'Failed',
 * 'Stopped', 'Blocked', 'Suspended', 'Disconnected', 'Suspending', 'Stopping',
 * 'Resuming', 'Removing'
 * @member {string} [statusDetails] Gets or sets the status details of the job.
 * @member {date} [startTime] Gets the start time of the job.
 * @member {date} [endTime] Gets the end time of the job.
 * @member {string} [exception] Gets the exception of the job.
 * @member {date} [lastModifiedTime] Gets the last modified time of the job.
 * @member {date} [lastStatusModifiedTime] Gets the last status modified time
 * of the job.
 * @member {object} [parameters] Gets or sets the parameters of the job.
 */
export interface DscCompilationJob extends ProxyResource {
  configuration?: DscConfigurationAssociationProperty;
  readonly startedBy?: string;
  readonly jobId?: string;
  readonly creationTime?: Date;
  provisioningState?: string;
  runOn?: string;
  status?: string;
  statusDetails?: string;
  readonly startTime?: Date;
  readonly endTime?: Date;
  readonly exception?: string;
  readonly lastModifiedTime?: Date;
  readonly lastStatusModifiedTime?: Date;
  parameters?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the DscCompilationJobCreateParameters class.
 * @constructor
 * The parameters supplied to the create compilation job operation.
 *
 * @member {object} configuration Gets or sets the configuration.
 * @member {string} [configuration.name] Gets or sets the name of the Dsc
 * configuration.
 * @member {object} [parameters] Gets or sets the parameters of the job.
 * @member {boolean} [incrementNodeConfigurationBuild] If a new build version
 * of NodeConfiguration is required.
 * @member {string} [name] Gets or sets name of the resource.
 * @member {string} [location] Gets or sets the location of the resource.
 * @member {object} [tags] Gets or sets the tags attached to the resource.
 */
export interface DscCompilationJobCreateParameters {
  configuration: DscConfigurationAssociationProperty;
  parameters?: { [propertyName: string]: string };
  incrementNodeConfigurationBuild?: boolean;
  name?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the DscNodeConfiguration class.
 * @constructor
 * Definition of the dsc node configuration.
 *
 * @member {date} [lastModifiedTime] Gets or sets the last modified time.
 * @member {date} [creationTime] Gets or sets creation time.
 * @member {object} [configuration] Gets or sets the configuration of the node.
 * @member {string} [configuration.name] Gets or sets the name of the Dsc
 * configuration.
 * @member {string} [source] Source of node configuration.
 * @member {number} [nodeCount] Number of nodes with this nodeconfiguration
 * assigned
 * @member {boolean} [incrementNodeConfigurationBuild] If a new build version
 * of NodeConfiguration is required.
 */
export interface DscNodeConfiguration extends ProxyResource {
  lastModifiedTime?: Date;
  creationTime?: Date;
  configuration?: DscConfigurationAssociationProperty;
  source?: string;
  nodeCount?: number;
  incrementNodeConfigurationBuild?: boolean;
}

/**
 * @class
 * Initializes a new instance of the DscNodeConfigurationCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update node configuration
 * operation.
 *
 * @member {object} source Gets or sets the source.
 * @member {object} [source.hash] Gets or sets the hash.
 * @member {string} [source.hash.algorithm] Gets or sets the content hash
 * algorithm used to hash the content.
 * @member {string} [source.hash.value] Gets or sets expected hash value of the
 * content.
 * @member {string} [source.type] Gets or sets the content source type.
 * Possible values include: 'embeddedContent', 'uri'
 * @member {string} [source.value] Gets or sets the value of the content. This
 * is based on the content source type.
 * @member {string} [source.version] Gets or sets the version of the content.
 * @member {object} configuration Gets or sets the configuration of the node.
 * @member {string} [configuration.name] Gets or sets the name of the Dsc
 * configuration.
 * @member {boolean} [incrementNodeConfigurationBuild] If a new build version
 * of NodeConfiguration is required.
 * @member {string} [name] Name of the node configuration.
 * @member {object} [tags] Gets or sets the tags attached to the resource.
 */
export interface DscNodeConfigurationCreateOrUpdateParameters {
  source: ContentSource;
  configuration: DscConfigurationAssociationProperty;
  incrementNodeConfigurationBuild?: boolean;
  name?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the NodeCountProperties class.
 * @constructor
 * @member {number} [count] Gets the count for the name
 */
export interface NodeCountProperties {
  count?: number;
}

/**
 * @class
 * Initializes a new instance of the NodeCount class.
 * @constructor
 * Number of nodes based on the Filter
 *
 * @member {string} [name] Gets the name of a count type
 * @member {object} [properties]
 * @member {number} [properties.count] Gets the count for the name
 */
export interface NodeCount {
  name?: string;
  properties?: NodeCountProperties;
}

/**
 * @class
 * Initializes a new instance of the NodeCounts class.
 * @constructor
 * Gets the count of nodes by count type
 *
 * @member {array} [value] Gets an array of counts
 * @member {number} [totalCount] Gets the total number of records matching
 * countType criteria.
 */
export interface NodeCounts {
  value?: NodeCount[];
  totalCount?: number;
}

/**
 * @class
 * Initializes a new instance of the RunbookParameter class.
 * @constructor
 * Definition of the runbook parameter type.
 *
 * @member {string} [type] Gets or sets the type of the parameter.
 * @member {boolean} [isMandatory] Gets or sets a Boolean value to indicate
 * whether the parameter is madatory or not.
 * @member {number} [position] Get or sets the position of the parameter.
 * @member {string} [defaultValue] Gets or sets the default value of parameter.
 */
export interface RunbookParameter {
  type?: string;
  isMandatory?: boolean;
  position?: number;
  defaultValue?: string;
}

/**
 * @class
 * Initializes a new instance of the RunbookDraft class.
 * @constructor
 * @member {boolean} [inEdit] Gets or sets whether runbook is in edit mode.
 * @member {object} [draftContentLink] Gets or sets the draft runbook content
 * link.
 * @member {string} [draftContentLink.uri] Gets or sets the uri of the runbook
 * content.
 * @member {object} [draftContentLink.contentHash] Gets or sets the hash.
 * @member {string} [draftContentLink.contentHash.algorithm] Gets or sets the
 * content hash algorithm used to hash the content.
 * @member {string} [draftContentLink.contentHash.value] Gets or sets expected
 * hash value of the content.
 * @member {string} [draftContentLink.version] Gets or sets the version of the
 * content.
 * @member {date} [creationTime] Gets or sets the creation time of the runbook
 * draft.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time of the
 * runbook draft.
 * @member {object} [parameters] Gets or sets the runbook draft parameters.
 * @member {array} [outputTypes] Gets or sets the runbook output types.
 */
export interface RunbookDraft {
  inEdit?: boolean;
  draftContentLink?: ContentLink;
  creationTime?: Date;
  lastModifiedTime?: Date;
  parameters?: { [propertyName: string]: RunbookParameter };
  outputTypes?: string[];
}

/**
 * @class
 * Initializes a new instance of the Runbook class.
 * @constructor
 * Definition of the runbook type.
 *
 * @member {string} [runbookType] Gets or sets the type of the runbook.
 * Possible values include: 'Script', 'Graph', 'PowerShellWorkflow',
 * 'PowerShell', 'GraphPowerShellWorkflow', 'GraphPowerShell'
 * @member {object} [publishContentLink] Gets or sets the published runbook
 * content link.
 * @member {string} [publishContentLink.uri] Gets or sets the uri of the
 * runbook content.
 * @member {object} [publishContentLink.contentHash] Gets or sets the hash.
 * @member {string} [publishContentLink.contentHash.algorithm] Gets or sets the
 * content hash algorithm used to hash the content.
 * @member {string} [publishContentLink.contentHash.value] Gets or sets
 * expected hash value of the content.
 * @member {string} [publishContentLink.version] Gets or sets the version of
 * the content.
 * @member {string} [state] Gets or sets the state of the runbook. Possible
 * values include: 'New', 'Edit', 'Published'
 * @member {boolean} [logVerbose] Gets or sets verbose log option.
 * @member {boolean} [logProgress] Gets or sets progress log option.
 * @member {number} [logActivityTrace] Gets or sets the option to log activity
 * trace of the runbook.
 * @member {number} [jobCount] Gets or sets the job count of the runbook.
 * @member {object} [parameters] Gets or sets the runbook parameters.
 * @member {array} [outputTypes] Gets or sets the runbook output types.
 * @member {object} [draft] Gets or sets the draft runbook properties.
 * @member {boolean} [draft.inEdit] Gets or sets whether runbook is in edit
 * mode.
 * @member {object} [draft.draftContentLink] Gets or sets the draft runbook
 * content link.
 * @member {string} [draft.draftContentLink.uri] Gets or sets the uri of the
 * runbook content.
 * @member {object} [draft.draftContentLink.contentHash] Gets or sets the hash.
 * @member {string} [draft.draftContentLink.contentHash.algorithm] Gets or sets
 * the content hash algorithm used to hash the content.
 * @member {string} [draft.draftContentLink.contentHash.value] Gets or sets
 * expected hash value of the content.
 * @member {string} [draft.draftContentLink.version] Gets or sets the version
 * of the content.
 * @member {date} [draft.creationTime] Gets or sets the creation time of the
 * runbook draft.
 * @member {date} [draft.lastModifiedTime] Gets or sets the last modified time
 * of the runbook draft.
 * @member {object} [draft.parameters] Gets or sets the runbook draft
 * parameters.
 * @member {array} [draft.outputTypes] Gets or sets the runbook output types.
 * @member {string} [provisioningState] Gets or sets the provisioning state of
 * the runbook. Possible values include: 'Succeeded'
 * @member {string} [lastModifiedBy] Gets or sets the last modified by.
 * @member {date} [creationTime] Gets or sets the creation time.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time.
 * @member {string} [description] Gets or sets the description.
 * @member {string} [etag] Gets or sets the etag of the resource.
 */
export interface Runbook extends TrackedResource {
  runbookType?: string;
  publishContentLink?: ContentLink;
  state?: string;
  logVerbose?: boolean;
  logProgress?: boolean;
  logActivityTrace?: number;
  jobCount?: number;
  parameters?: { [propertyName: string]: RunbookParameter };
  outputTypes?: string[];
  draft?: RunbookDraft;
  provisioningState?: string;
  lastModifiedBy?: string;
  creationTime?: Date;
  lastModifiedTime?: Date;
  description?: string;
  etag?: string;
}

/**
 * @class
 * Initializes a new instance of the RunbookCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update runbook operation.
 *
 * @member {boolean} [logVerbose] Gets or sets verbose log option.
 * @member {boolean} [logProgress] Gets or sets progress log option.
 * @member {string} runbookType Gets or sets the type of the runbook. Possible
 * values include: 'Script', 'Graph', 'PowerShellWorkflow', 'PowerShell',
 * 'GraphPowerShellWorkflow', 'GraphPowerShell'
 * @member {object} [draft] Gets or sets the draft runbook properties.
 * @member {boolean} [draft.inEdit] Gets or sets whether runbook is in edit
 * mode.
 * @member {object} [draft.draftContentLink] Gets or sets the draft runbook
 * content link.
 * @member {string} [draft.draftContentLink.uri] Gets or sets the uri of the
 * runbook content.
 * @member {object} [draft.draftContentLink.contentHash] Gets or sets the hash.
 * @member {string} [draft.draftContentLink.contentHash.algorithm] Gets or sets
 * the content hash algorithm used to hash the content.
 * @member {string} [draft.draftContentLink.contentHash.value] Gets or sets
 * expected hash value of the content.
 * @member {string} [draft.draftContentLink.version] Gets or sets the version
 * of the content.
 * @member {date} [draft.creationTime] Gets or sets the creation time of the
 * runbook draft.
 * @member {date} [draft.lastModifiedTime] Gets or sets the last modified time
 * of the runbook draft.
 * @member {object} [draft.parameters] Gets or sets the runbook draft
 * parameters.
 * @member {array} [draft.outputTypes] Gets or sets the runbook output types.
 * @member {object} [publishContentLink] Gets or sets the published runbook
 * content link.
 * @member {string} [publishContentLink.uri] Gets or sets the uri of the
 * runbook content.
 * @member {object} [publishContentLink.contentHash] Gets or sets the hash.
 * @member {string} [publishContentLink.contentHash.algorithm] Gets or sets the
 * content hash algorithm used to hash the content.
 * @member {string} [publishContentLink.contentHash.value] Gets or sets
 * expected hash value of the content.
 * @member {string} [publishContentLink.version] Gets or sets the version of
 * the content.
 * @member {string} [description] Gets or sets the description of the runbook.
 * @member {number} [logActivityTrace] Gets or sets the activity-level tracing
 * options of the runbook.
 * @member {string} [name] Gets or sets the name of the resource.
 * @member {string} [location] Gets or sets the location of the resource.
 * @member {object} [tags] Gets or sets the tags attached to the resource.
 */
export interface RunbookCreateOrUpdateParameters {
  logVerbose?: boolean;
  logProgress?: boolean;
  runbookType: string;
  draft?: RunbookDraft;
  publishContentLink?: ContentLink;
  description?: string;
  logActivityTrace?: number;
  name?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the RunbookUpdateParameters class.
 * @constructor
 * The parameters supplied to the update runbook operation.
 *
 * @member {string} [description] Gets or sets the description of the runbook.
 * @member {boolean} [logVerbose] Gets or sets verbose log option.
 * @member {boolean} [logProgress] Gets or sets progress log option.
 * @member {number} [logActivityTrace] Gets or sets the activity-level tracing
 * options of the runbook.
 * @member {string} [name] Gets or sets the name of the resource.
 * @member {string} [location] Gets or sets the location of the resource.
 * @member {object} [tags] Gets or sets the tags attached to the resource.
 */
export interface RunbookUpdateParameters {
  description?: string;
  logVerbose?: boolean;
  logProgress?: boolean;
  logActivityTrace?: number;
  name?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the RunbookDraftUndoEditResult class.
 * @constructor
 * The response model for the undoedit runbook operation.
 *
 * @member {string} [statusCode] Possible values include: 'Continue',
 * 'SwitchingProtocols', 'OK', 'Created', 'Accepted',
 * 'NonAuthoritativeInformation', 'NoContent', 'ResetContent',
 * 'PartialContent', 'MultipleChoices', 'Ambiguous', 'MovedPermanently',
 * 'Moved', 'Found', 'Redirect', 'SeeOther', 'RedirectMethod', 'NotModified',
 * 'UseProxy', 'Unused', 'TemporaryRedirect', 'RedirectKeepVerb', 'BadRequest',
 * 'Unauthorized', 'PaymentRequired', 'Forbidden', 'NotFound',
 * 'MethodNotAllowed', 'NotAcceptable', 'ProxyAuthenticationRequired',
 * 'RequestTimeout', 'Conflict', 'Gone', 'LengthRequired',
 * 'PreconditionFailed', 'RequestEntityTooLarge', 'RequestUriTooLong',
 * 'UnsupportedMediaType', 'RequestedRangeNotSatisfiable', 'ExpectationFailed',
 * 'UpgradeRequired', 'InternalServerError', 'NotImplemented', 'BadGateway',
 * 'ServiceUnavailable', 'GatewayTimeout', 'HttpVersionNotSupported'
 * @member {string} [requestId]
 */
export interface RunbookDraftUndoEditResult {
  statusCode?: string;
  requestId?: string;
}

/**
 * @class
 * Initializes a new instance of the TestJobCreateParameters class.
 * @constructor
 * The parameters supplied to the create test job operation.
 *
 * @member {object} [parameters] Gets or sets the parameters of the test job.
 * @member {string} [runOn] Gets or sets the runOn which specifies the group
 * name where the job is to be executed.
 */
export interface TestJobCreateParameters {
  parameters?: { [propertyName: string]: string };
  runOn?: string;
}

/**
 * @class
 * Initializes a new instance of the TestJob class.
 * @constructor
 * Definition of the test job.
 *
 * @member {date} [creationTime] Gets or sets the creation time of the test
 * job.
 * @member {string} [status] Gets or sets the status of the test job.
 * @member {string} [statusDetails] Gets or sets the status details of the test
 * job.
 * @member {string} [runOn] Gets or sets the runOn which specifies the group
 * name where the job is to be executed.
 * @member {date} [startTime] Gets or sets the start time of the test job.
 * @member {date} [endTime] Gets or sets the end time of the test job.
 * @member {string} [exception] Gets or sets the exception of the test job.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time of the
 * test job.
 * @member {date} [lastStatusModifiedTime] Gets or sets the last status
 * modified time of the test job.
 * @member {object} [parameters] Gets or sets the parameters of the test job.
 * @member {number} [logActivityTrace] The activity-level tracing options of
 * the runbook.
 */
export interface TestJob {
  creationTime?: Date;
  status?: string;
  statusDetails?: string;
  runOn?: string;
  startTime?: Date;
  endTime?: Date;
  exception?: string;
  lastModifiedTime?: Date;
  lastStatusModifiedTime?: Date;
  parameters?: { [propertyName: string]: string };
  logActivityTrace?: number;
}

/**
 * @class
 * Initializes a new instance of the RunbookCreateOrUpdateDraftProperties class.
 * @constructor
 * The parameters supplied to the create or update dratft runbook properties.
 *
 * @member {boolean} [logVerbose] Gets or sets verbose log option.
 * @member {boolean} [logProgress] Gets or sets progress log option.
 * @member {string} runbookType Gets or sets the type of the runbook. Possible
 * values include: 'Script', 'Graph', 'PowerShellWorkflow', 'PowerShell',
 * 'GraphPowerShellWorkflow', 'GraphPowerShell'
 * @member {object} draft Gets or sets the draft runbook properties.
 * @member {boolean} [draft.inEdit] Gets or sets whether runbook is in edit
 * mode.
 * @member {object} [draft.draftContentLink] Gets or sets the draft runbook
 * content link.
 * @member {string} [draft.draftContentLink.uri] Gets or sets the uri of the
 * runbook content.
 * @member {object} [draft.draftContentLink.contentHash] Gets or sets the hash.
 * @member {string} [draft.draftContentLink.contentHash.algorithm] Gets or sets
 * the content hash algorithm used to hash the content.
 * @member {string} [draft.draftContentLink.contentHash.value] Gets or sets
 * expected hash value of the content.
 * @member {string} [draft.draftContentLink.version] Gets or sets the version
 * of the content.
 * @member {date} [draft.creationTime] Gets or sets the creation time of the
 * runbook draft.
 * @member {date} [draft.lastModifiedTime] Gets or sets the last modified time
 * of the runbook draft.
 * @member {object} [draft.parameters] Gets or sets the runbook draft
 * parameters.
 * @member {array} [draft.outputTypes] Gets or sets the runbook output types.
 * @member {string} [description] Gets or sets the description of the runbook.
 * @member {number} [logActivityTrace] Gets or sets the activity-level tracing
 * options of the runbook.
 */
export interface RunbookCreateOrUpdateDraftProperties {
  logVerbose?: boolean;
  logProgress?: boolean;
  runbookType: string;
  draft: RunbookDraft;
  description?: string;
  logActivityTrace?: number;
}

/**
 * @class
 * Initializes a new instance of the RunbookCreateOrUpdateDraftParameters class.
 * @constructor
 * The parameters supplied to the create or update runbook operation.
 *
 * @member {string} runbookContent Content of the Runbook.
 */
export interface RunbookCreateOrUpdateDraftParameters {
  runbookContent: string;
}

/**
 * @class
 * Initializes a new instance of the PythonPackageCreateParameters class.
 * @constructor
 * The parameters supplied to the create or update module operation.
 *
 * @member {object} contentLink Gets or sets the module content link.
 * @member {string} [contentLink.uri] Gets or sets the uri of the runbook
 * content.
 * @member {object} [contentLink.contentHash] Gets or sets the hash.
 * @member {string} [contentLink.contentHash.algorithm] Gets or sets the
 * content hash algorithm used to hash the content.
 * @member {string} [contentLink.contentHash.value] Gets or sets expected hash
 * value of the content.
 * @member {string} [contentLink.version] Gets or sets the version of the
 * content.
 * @member {object} [tags] Gets or sets the tags attached to the resource.
 */
export interface PythonPackageCreateParameters {
  contentLink: ContentLink;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the PythonPackageUpdateParameters class.
 * @constructor
 * The parameters supplied to the update module operation.
 *
 * @member {object} [tags] Gets or sets the tags attached to the resource.
 */
export interface PythonPackageUpdateParameters {
  tags?: { [propertyName: string]: string };
}


/**
 * @class
 * Initializes a new instance of the AutomationAccountListResult class.
 * @constructor
 * The response model for the list account operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface AutomationAccountListResult extends Array<AutomationAccount> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationListResult class.
 * @constructor
 * The response model for the list of Automation operations
 *
 */
export interface OperationListResult extends Array<Operation> {
}

/**
 * @class
 * Initializes a new instance of the StatisticsListResult class.
 * @constructor
 * The response model for the list statistics operation.
 *
 */
export interface StatisticsListResult extends Array<Statistics> {
}

/**
 * @class
 * Initializes a new instance of the UsageListResult class.
 * @constructor
 * The response model for the get usage operation.
 *
 */
export interface UsageListResult extends Array<Usage> {
}

/**
 * @class
 * Initializes a new instance of the CertificateListResult class.
 * @constructor
 * The response model for the list certificate operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface CertificateListResult extends Array<Certificate> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the ConnectionListResult class.
 * @constructor
 * The response model for the list connection operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface ConnectionListResult extends Array<Connection> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the ConnectionTypeListResult class.
 * @constructor
 * The response model for the list connection type operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface ConnectionTypeListResult extends Array<ConnectionType> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the CredentialListResult class.
 * @constructor
 * The response model for the list credential operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface CredentialListResult extends Array<Credential> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the DscConfigurationListResult class.
 * @constructor
 * The response model for the list configuration operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 * @member {number} [totalCount] Gets the total number of configurations
 * matching filter criteria.
 */
export interface DscConfigurationListResult extends Array<DscConfiguration> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the HybridRunbookWorkerGroupsListResult class.
 * @constructor
 * The response model for the list hybrid runbook worker groups.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface HybridRunbookWorkerGroupsListResult extends Array<HybridRunbookWorkerGroup> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the JobScheduleListResult class.
 * @constructor
 * The response model for the list job schedule operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface JobScheduleListResult extends Array<JobSchedule> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the ActivityListResult class.
 * @constructor
 * The response model for the list activity operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface ActivityListResult extends Array<Activity> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the ModuleListResult class.
 * @constructor
 * The response model for the list module operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface ModuleListResult extends Array<Module> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the TypeFieldListResult class.
 * @constructor
 * The response model for the list fields operation.
 *
 */
export interface TypeFieldListResult extends Array<TypeField> {
}

/**
 * @class
 * Initializes a new instance of the ScheduleListResult class.
 * @constructor
 * The response model for the list schedule operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface ScheduleListResult extends Array<Schedule> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the VariableListResult class.
 * @constructor
 * The response model for the list variables operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface VariableListResult extends Array<Variable> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the WebhookListResult class.
 * @constructor
 * The response model for the list webhook operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface WebhookListResult extends Array<Webhook> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the WatcherListResult class.
 * @constructor
 * The response model for the list watcher operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface WatcherListResult extends Array<Watcher> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the SourceControlListResult class.
 * @constructor
 * The response model for the list source controls operation.
 *
 * @member {string} [nextLink] The next link.
 */
export interface SourceControlListResult extends Array<SourceControl> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the SourceControlSyncJobListResult class.
 * @constructor
 * The response model for the list source control sync jobs operation.
 *
 * @member {string} [nextLink] The next link.
 */
export interface SourceControlSyncJobListResult extends Array<SourceControlSyncJob> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the SourceControlSyncJobStreamsListBySyncJob class.
 * @constructor
 * The response model for the list source control sync job streams operation.
 *
 * @member {string} [nextLink] The next link.
 */
export interface SourceControlSyncJobStreamsListBySyncJob extends Array<SourceControlSyncJobStream> {
  readonly nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the JobListResultV2 class.
 * @constructor
 * The response model for the list job operation.
 *
 * @member {string} [nextLink] The  link to the next page.
 */
export interface JobListResultV2 extends Array<JobCollectionItem> {
  readonly nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the JobStreamListResult class.
 * @constructor
 * The response model for the list job stream operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface JobStreamListResult extends Array<JobStream> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the DscNodeListResult class.
 * @constructor
 * The response model for the list dsc nodes operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 * @member {number} [totalCount] Gets the total number of nodes matching filter
 * criteria.
 */
export interface DscNodeListResult extends Array<DscNode> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the DscNodeReportListResult class.
 * @constructor
 * The response model for the list dsc nodes operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface DscNodeReportListResult extends Array<DscNodeReport> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the DscCompilationJobListResult class.
 * @constructor
 * The response model for the list job operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface DscCompilationJobListResult extends Array<DscCompilationJob> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the DscNodeConfigurationListResult class.
 * @constructor
 * The response model for the list job operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 * @member {number} [totalCount] Gets or sets the total rows in query.
 */
export interface DscNodeConfigurationListResult extends Array<DscNodeConfiguration> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the RunbookListResult class.
 * @constructor
 * The response model for the list runbook operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface RunbookListResult extends Array<Runbook> {
  nextLink?: string;
}
