/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * @summary Properties used to create a user used to execute Tasks on an Azure
 * Batch Compute Node.
  *
 */
class UserAccount {
  /**
   * Create a UserAccount.
   * @property {string} name The name of the user Account.
   * @property {string} password The password for the user Account.
   * @property {string} [elevationLevel] The elevation level of the user
   * Account. The default value is nonAdmin. Possible values include:
   * 'nonAdmin', 'admin'
   * @property {object} [linuxUserConfiguration] The Linux-specific user
   * configuration for the user Account. This property is ignored if specified
   * on a Windows Pool. If not specified, the user is created with the default
   * options.
   * @property {number} [linuxUserConfiguration.uid] The uid and gid properties
   * must be specified together or not at all. If not specified the underlying
   * operating system picks the uid.
   * @property {number} [linuxUserConfiguration.gid] The uid and gid properties
   * must be specified together or not at all. If not specified the underlying
   * operating system picks the gid.
   * @property {string} [linuxUserConfiguration.sshPrivateKey] The private key
   * must not be password protected. The private key is used to automatically
   * configure asymmetric-key based authentication for SSH between Compute
   * Nodes in a Linux Pool when the Pool's enableInterNodeCommunication
   * property is true (it is ignored if enableInterNodeCommunication is false).
   * It does this by placing the key pair into the user's .ssh directory. If
   * not specified, password-less SSH is not configured between Compute Nodes
   * (no modification of the user's .ssh directory is done).
   * @property {object} [windowsUserConfiguration] The Windows-specific user
   * configuration for the user Account. This property can only be specified if
   * the user is on a Windows Pool. If not specified and on a Windows Pool, the
   * user is created with the default options.
   * @property {string} [windowsUserConfiguration.loginMode] The default value
   * for VirtualMachineConfiguration Pools is 'batch' and for
   * CloudServiceConfiguration Pools is 'interactive'. Possible values include:
   * 'batch', 'interactive'
   */
  constructor() {
  }

  /**
   * Defines the metadata of UserAccount
   *
   * @returns {object} metadata of UserAccount
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'UserAccount',
      type: {
        name: 'Composite',
        className: 'UserAccount',
        modelProperties: {
          name: {
            required: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          password: {
            required: true,
            serializedName: 'password',
            type: {
              name: 'String'
            }
          },
          elevationLevel: {
            required: false,
            serializedName: 'elevationLevel',
            type: {
              name: 'Enum',
              allowedValues: [ 'nonAdmin', 'admin' ]
            }
          },
          linuxUserConfiguration: {
            required: false,
            serializedName: 'linuxUserConfiguration',
            type: {
              name: 'Composite',
              className: 'LinuxUserConfiguration'
            }
          },
          windowsUserConfiguration: {
            required: false,
            serializedName: 'windowsUserConfiguration',
            type: {
              name: 'Composite',
              className: 'WindowsUserConfiguration'
            }
          }
        }
      }
    };
  }
}

module.exports = UserAccount;
