/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is regenerated.
 */

import * as moment from "moment";

/**
 * Defines the query context that Bing used for the request.
 */
export interface QueryContext {
  /**
   * The query string as specified in the request.
   */
  originalQuery: string;
  /**
   * The query string used by Bing to perform the query. Bing uses the altered query string if the
   * original query string contained spelling mistakes. For example, if the query string is "saling
   * downwind", the altered query string will be "sailing downwind". This field is included only if
   * the original query string contains a spelling mistake.
   */
  readonly alteredQuery?: string;
  /**
   * The query string to use to force Bing to use the original string. For example, if the query
   * string is "saling downwind", the override query string will be "+saling downwind". Remember to
   * encode the query string which results in "%2Bsaling+downwind". This field is included only if
   * the original query string contains a spelling mistake.
   */
  readonly alterationOverrideQuery?: string;
  /**
   * A Boolean value that indicates whether the specified query has adult intent. The value is true
   * if the query has adult intent; otherwise, false.
   */
  readonly adultIntent?: boolean;
}

/**
 * Defines a webpage's metadata.
 */
export interface WebMetaTag {
  /**
   * The metadata.
   */
  readonly name?: string;
  /**
   * The name of the metadata.
   */
  readonly content?: string;
}

export interface ResponseBase {
  /**
   * Polymorphic Discriminator
  */
  _type: string;
}

/**
 * Defines the identity of a resource.
*/
export interface Identifiable extends ResponseBase {
  /**
   * A String identifier.
  */
  readonly id?: string;
}

/**
 * Defines a response. All schemas that could be returned at the root of a response should inherit
 * from this
*/
export interface Response extends Identifiable {
  /**
   * The URL To Bing's search result for this item.
  */
  readonly webSearchUrl?: string;
}

export interface Thing extends Response {
  /**
   * The name of the thing represented by this object.
  */
  readonly name?: string;
  /**
   * The URL to get more information about the thing represented by this object.
  */
  readonly url?: string;
  /**
   * A short description of the item.
  */
  readonly description?: string;
  /**
   * An ID that uniquely identifies this item.
  */
  readonly bingId?: string;
}

export interface CreativeWork extends Thing {
  /**
   * The URL to a thumbnail of the item.
  */
  readonly thumbnailUrl?: string;
  /**
   * The source of the creative work.
  */
  readonly provider?: Thing[];
  readonly text?: string;
}

/**
 * Defines a webpage that is relevant to the query.
*/
export interface WebPage extends CreativeWork {
  /**
   * The display URL of the webpage. The URL is meant for display purposes only and is not well
   * formed.
  */
  readonly displayUrl?: string;
  /**
   * A snippet of text from the webpage that describes its contents.
  */
  readonly snippet?: string;
  /**
   * A list of links to related content that Bing found in the website that contains this webpage.
   * The Webpage object in this context includes only the name, url, urlPingSuffix, and snippet
   * fields.
  */
  readonly deepLinks?: WebPage[];
  /**
   * The last time that Bing crawled the webpage. The date is in the form, YYYY-MM-DDTHH:MM:SS. For
   * example, 2015-04-13T05:23:39.
  */
  readonly dateLastCrawled?: string;
  /**
   * A list of search tags that the webpage owner specified on the webpage. The API returns only
   * indexed search tags. The name field of the MetaTag object contains the indexed search tag.
   * Search tags begin with search.* (for example, search.assetId). The content field contains the
   * tag's value.
  */
  readonly searchTags?: WebMetaTag[];
}

export interface Answer extends Response {
  readonly followUpQueries?: Query[];
}

export interface SearchResultsAnswer extends Answer {
  readonly queryContext?: QueryContext;
  /**
   * The estimated number of webpages that are relevant to the query. Use this number along with
   * the count and offset query parameters to page the results.
  */
  readonly totalEstimatedMatches?: number;
  readonly isFamilyFriendly?: boolean;
}

/**
 * Defines a list of relevant webpage links.
*/
export interface WebWebAnswer extends SearchResultsAnswer {
  /**
   * A list of webpages that are relevant to the query.
  */
  value: WebPage[];
  /**
   * A Boolean value that indicates whether the response excluded some results from the answer. If
   * Bing excluded some results, the value is true.
  */
  readonly someResultsRemoved?: boolean;
}

/**
 * Defines the top-level object that the response includes when the request succeeds.
*/
export interface SearchResponse extends Response {
  /**
   * An object that contains the query string that Bing used for the request. This object contains
   * the query string as entered by the user. It may also contain an altered query string that Bing
   * used for the query if the query string contained a spelling mistake.
  */
  readonly queryContext?: QueryContext;
  /**
   * A list of webpages that are relevant to the search query.
  */
  readonly webPages?: WebWebAnswer;
}

/**
 * Defines the error that occurred.
*/
export interface ErrorModel {
  /**
   * The error code that identifies the category of error. Possible values include: 'None',
   * 'ServerError', 'InvalidRequest', 'RateLimitExceeded', 'InvalidAuthorization',
   * 'InsufficientAuthorization'
  */
  code: string;
  /**
   * The error code that further helps to identify the error. Possible values include:
   * 'UnexpectedError', 'ResourceError', 'NotImplemented', 'ParameterMissing',
   * 'ParameterInvalidValue', 'HttpNotAllowed', 'Blocked', 'AuthorizationMissing',
   * 'AuthorizationRedundancy', 'AuthorizationDisabled', 'AuthorizationExpired'
  */
  readonly subCode?: string;
  /**
   * A description of the error.
  */
  message: string;
  /**
   * A description that provides additional information about the error.
  */
  readonly moreDetails?: string;
  /**
   * The parameter in the request that caused the error.
  */
  readonly parameter?: string;
  /**
   * The parameter's value in the request that was not valid.
  */
  readonly value?: string;
}

/**
 * The top-level response that represents a failed request.
*/
export interface ErrorResponse extends Response {
  /**
   * A list of errors that describe the reasons why the request failed.
  */
  errors: ErrorModel[];
}

/**
 * Defines a search query.
*/
export interface Query {
  /**
   * The query string. Use this string as the query term in a new search request.
  */
  text: string;
  /**
   * The display version of the query term. This version of the query term may contain special
   * characters that highlight the search term found in the query string. The string contains the
   * highlighting characters only if the query enabled hit highlighting
  */
  readonly displayText?: string;
  /**
   * The URL that takes the user to the Bing search results page for the query.Only related search
   * results include this field.
  */
  readonly webSearchUrl?: string;
  readonly searchLink?: string;
}
