/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Defines the top-level object that the response includes when the request
 * succeeds.
 *
 * @extends models['Response']
 */
class SearchResponse extends models['Response'] {
  /**
   * Create a SearchResponse.
   * @property {object} [queryContext] An object that contains the query string
   * that Bing used for the request. This object contains the query string as
   * entered by the user. It may also contain an altered query string that Bing
   * used for the query if the query string contained a spelling mistake.
   * @property {string} [queryContext.originalQuery] The query string as
   * specified in the request.
   * @property {string} [queryContext.alteredQuery] The query string used by
   * Bing to perform the query. Bing uses the altered query string if the
   * original query string contained spelling mistakes. For example, if the
   * query string is "saling downwind", the altered query string will be
   * "sailing downwind". This field is included only if the original query
   * string contains a spelling mistake.
   * @property {string} [queryContext.alterationDisplayQuery] AlteredQuery that
   * is formatted for display purpose. The query string in the
   * AlterationDisplayQuery can be html-escaped and can contain
   * hit-highlighting characters
   * @property {string} [queryContext.alterationOverrideQuery] The query string
   * to use to force Bing to use the original string. For example, if the query
   * string is "saling downwind", the override query string will be "+saling
   * downwind". Remember to encode the query string which results in
   * "%2Bsaling+downwind". This field is included only if the original query
   * string contains a spelling mistake.
   * @property {boolean} [queryContext.adultIntent] A Boolean value that
   * indicates whether the specified query has adult intent. The value is true
   * if the query has adult intent; otherwise, false.
   * @property {boolean} [queryContext.askUserForLocation] A Boolean value that
   * indicates whether Bing requires the user's location to provide accurate
   * results. If you specified the user's location by using the
   * X-MSEdge-ClientIP and X-Search-Location headers, you can ignore this
   * field. For location aware queries, such as "today's weather" or
   * "restaurants near me" that need the user's location to provide accurate
   * results, this field is set to true. For location aware queries that
   * include the location (for example, "Seattle weather"), this field is set
   * to false. This field is also set to false for queries that are not
   * location aware, such as "best sellers".
   * @property {boolean} [queryContext.isTransactional]
   * @property {string} [queryContext._type] Polymorphic Discriminator
   * @property {object} [places] A list of local entities such as restaurants
   * or hotels that are relevant to the query.
   * @property {array} [places.value] A list of local entities, such as
   * restaurants or hotels.
   * @property {object} [lottery]
   * @property {object} [lottery.queryContext]
   * @property {string} [lottery.queryContext.originalQuery] The query string
   * as specified in the request.
   * @property {string} [lottery.queryContext.alteredQuery] The query string
   * used by Bing to perform the query. Bing uses the altered query string if
   * the original query string contained spelling mistakes. For example, if the
   * query string is "saling downwind", the altered query string will be
   * "sailing downwind". This field is included only if the original query
   * string contains a spelling mistake.
   * @property {string} [lottery.queryContext.alterationDisplayQuery]
   * AlteredQuery that is formatted for display purpose. The query string in
   * the AlterationDisplayQuery can be html-escaped and can contain
   * hit-highlighting characters
   * @property {string} [lottery.queryContext.alterationOverrideQuery] The
   * query string to use to force Bing to use the original string. For example,
   * if the query string is "saling downwind", the override query string will
   * be "+saling downwind". Remember to encode the query string which results
   * in "%2Bsaling+downwind". This field is included only if the original query
   * string contains a spelling mistake.
   * @property {boolean} [lottery.queryContext.adultIntent] A Boolean value
   * that indicates whether the specified query has adult intent. The value is
   * true if the query has adult intent; otherwise, false.
   * @property {boolean} [lottery.queryContext.askUserForLocation] A Boolean
   * value that indicates whether Bing requires the user's location to provide
   * accurate results. If you specified the user's location by using the
   * X-MSEdge-ClientIP and X-Search-Location headers, you can ignore this
   * field. For location aware queries, such as "today's weather" or
   * "restaurants near me" that need the user's location to provide accurate
   * results, this field is set to true. For location aware queries that
   * include the location (for example, "Seattle weather"), this field is set
   * to false. This field is also set to false for queries that are not
   * location aware, such as "best sellers".
   * @property {boolean} [lottery.queryContext.isTransactional]
   * @property {string} [lottery.queryContext._type] Polymorphic Discriminator
   * @property {number} [lottery.totalEstimatedMatches] The estimated number of
   * webpages that are relevant to the query. Use this number along with the
   * count and offset query parameters to page the results.
   * @property {boolean} [lottery.isFamilyFriendly]
   * @property {number} [searchResultsConfidenceScore]
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of SearchResponse
   *
   * @returns {object} metadata of SearchResponse
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'SearchResponse',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: '_type',
          clientName: '_type'
        },
        uberParent: 'ResponseBase',
        className: 'SearchResponse',
        modelProperties: {
          _type: {
            required: true,
            serializedName: '_type',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          readLink: {
            required: false,
            readOnly: true,
            serializedName: 'readLink',
            type: {
              name: 'String'
            }
          },
          webSearchUrl: {
            required: false,
            readOnly: true,
            serializedName: 'webSearchUrl',
            type: {
              name: 'String'
            }
          },
          potentialAction: {
            required: false,
            readOnly: true,
            serializedName: 'potentialAction',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ActionElementType',
                  type: {
                    name: 'Composite',
                    polymorphicDiscriminator: {
                      serializedName: '_type',
                      clientName: '_type'
                    },
                    uberParent: 'ResponseBase',
                    className: 'Action'
                  }
              }
            }
          },
          immediateAction: {
            required: false,
            readOnly: true,
            serializedName: 'immediateAction',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ActionElementType',
                  type: {
                    name: 'Composite',
                    polymorphicDiscriminator: {
                      serializedName: '_type',
                      clientName: '_type'
                    },
                    uberParent: 'ResponseBase',
                    className: 'Action'
                  }
              }
            }
          },
          preferredClickthroughUrl: {
            required: false,
            readOnly: true,
            serializedName: 'preferredClickthroughUrl',
            type: {
              name: 'String'
            }
          },
          adaptiveCard: {
            required: false,
            readOnly: true,
            serializedName: 'adaptiveCard',
            type: {
              name: 'String'
            }
          },
          queryContext: {
            required: false,
            readOnly: true,
            serializedName: 'queryContext',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: '_type',
                clientName: '_type'
              },
              uberParent: 'QueryContext',
              className: 'QueryContext'
            }
          },
          places: {
            required: false,
            readOnly: true,
            serializedName: 'places',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: '_type',
                clientName: '_type'
              },
              uberParent: 'ResponseBase',
              className: 'Places'
            }
          },
          lottery: {
            required: false,
            readOnly: true,
            serializedName: 'lottery',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: '_type',
                clientName: '_type'
              },
              uberParent: 'ResponseBase',
              className: 'SearchResultsAnswer'
            }
          },
          searchResultsConfidenceScore: {
            required: false,
            readOnly: true,
            serializedName: 'searchResultsConfidenceScore',
            type: {
              name: 'Number'
            }
          }
        }
      }
    };
  }
}

module.exports = SearchResponse;
