/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Dynamics linked service.
 *
 * @extends models['LinkedService']
 */
class DynamicsLinkedService extends models['LinkedService'] {
  /**
   * Create a DynamicsLinkedService.
   * @member {object} deploymentType The deployment type of the Dynamics
   * instance. 'Online' for Dynamics Online and 'OnPremisesWithIfd' for
   * Dynamics on-premises with Ifd. Type: string (or Expression with resultType
   * string).
   * @member {object} [hostName] The host name of the on-premises Dynamics
   * server. The property is required for on-prem and not allowed for online.
   * Type: string (or Expression with resultType string).
   * @member {object} [port] The port of on-premises Dynamics server. The
   * property is required for on-prem and not allowed for online. Default is
   * 443. Type: integer (or Expression with resultType integer), minimum: 0.
   * @member {object} [serviceUri] The URL to the Microsoft Dynamics server.
   * The property is required for on-line and not allowed for on-prem. Type:
   * string (or Expression with resultType string).
   * @member {object} [organizationName] The organization name of the Dynamics
   * instance. The property is required for on-prem and required for online
   * when there are more than one Dynamics instances associated with the user.
   * Type: string (or Expression with resultType string).
   * @member {object} authenticationType The authentication type to connect to
   * Dynamics server. 'Office365' for online scenario, 'Ifd' for on-premises
   * with Ifd scenario. Type: string (or Expression with resultType string).
   * @member {object} username User name to access the Dynamics instance. Type:
   * string (or Expression with resultType string).
   * @member {object} [password] Password to access the Dynamics instance.
   * @member {string} [password.type] Polymorphic Discriminator
   * @member {object} [encryptedCredential] The encrypted credential used for
   * authentication. Credentials are encrypted using the integration runtime
   * credential manager. Type: string (or Expression with resultType string).
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of DynamicsLinkedService
   *
   * @returns {object} metadata of DynamicsLinkedService
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Dynamics',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: 'type',
          clientName: 'type'
        },
        uberParent: 'LinkedService',
        className: 'DynamicsLinkedService',
        modelProperties: {
          connectVia: {
            required: false,
            serializedName: 'connectVia',
            type: {
              name: 'Composite',
              className: 'IntegrationRuntimeReference'
            }
          },
          description: {
            required: false,
            serializedName: 'description',
            type: {
              name: 'String'
            }
          },
          parameters: {
            required: false,
            serializedName: 'parameters',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'ParameterSpecificationElementType',
                  type: {
                    name: 'Composite',
                    className: 'ParameterSpecification'
                  }
              }
            }
          },
          annotations: {
            required: false,
            serializedName: 'annotations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ObjectElementType',
                  type: {
                    name: 'Object'
                  }
              }
            }
          },
          type: {
            required: true,
            serializedName: 'type',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          deploymentType: {
            required: true,
            serializedName: 'typeProperties.deploymentType',
            type: {
              name: 'Object'
            }
          },
          hostName: {
            required: false,
            serializedName: 'typeProperties.hostName',
            type: {
              name: 'Object'
            }
          },
          port: {
            required: false,
            serializedName: 'typeProperties.port',
            type: {
              name: 'Object'
            }
          },
          serviceUri: {
            required: false,
            serializedName: 'typeProperties.serviceUri',
            type: {
              name: 'Object'
            }
          },
          organizationName: {
            required: false,
            serializedName: 'typeProperties.organizationName',
            type: {
              name: 'Object'
            }
          },
          authenticationType: {
            required: true,
            serializedName: 'typeProperties.authenticationType',
            type: {
              name: 'Object'
            }
          },
          username: {
            required: true,
            serializedName: 'typeProperties.username',
            type: {
              name: 'Object'
            }
          },
          password: {
            required: false,
            serializedName: 'typeProperties.password',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'type',
                clientName: 'type'
              },
              uberParent: 'SecretBase',
              className: 'SecretBase'
            }
          },
          encryptedCredential: {
            required: false,
            serializedName: 'typeProperties.encryptedCredential',
            type: {
              name: 'Object'
            }
          }
        }
      }
    };
  }
}

module.exports = DynamicsLinkedService;
