/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Magento server linked service.
 *
 * @extends models['LinkedService']
 */
class MagentoLinkedService extends models['LinkedService'] {
  /**
   * Create a MagentoLinkedService.
   * @member {object} host The URL of the Magento instance. (i.e.
   * 192.168.222.110/magento3)
   * @member {object} [accessToken] The access token from Magento.
   * @member {string} [accessToken.type] Polymorphic Discriminator
   * @member {object} [useEncryptedEndpoints] Specifies whether the data source
   * endpoints are encrypted using HTTPS. The default value is true.
   * @member {object} [useHostVerification] Specifies whether to require the
   * host name in the server's certificate to match the host name of the server
   * when connecting over SSL. The default value is true.
   * @member {object} [usePeerVerification] Specifies whether to verify the
   * identity of the server when connecting over SSL. The default value is
   * true.
   * @member {object} [encryptedCredential] The encrypted credential used for
   * authentication. Credentials are encrypted using the integration runtime
   * credential manager. Type: string (or Expression with resultType string).
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of MagentoLinkedService
   *
   * @returns {object} metadata of MagentoLinkedService
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Magento',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: 'type',
          clientName: 'type'
        },
        uberParent: 'LinkedService',
        className: 'MagentoLinkedService',
        modelProperties: {
          connectVia: {
            required: false,
            serializedName: 'connectVia',
            type: {
              name: 'Composite',
              className: 'IntegrationRuntimeReference'
            }
          },
          description: {
            required: false,
            serializedName: 'description',
            type: {
              name: 'String'
            }
          },
          parameters: {
            required: false,
            serializedName: 'parameters',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'ParameterSpecificationElementType',
                  type: {
                    name: 'Composite',
                    className: 'ParameterSpecification'
                  }
              }
            }
          },
          annotations: {
            required: false,
            serializedName: 'annotations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ObjectElementType',
                  type: {
                    name: 'Object'
                  }
              }
            }
          },
          type: {
            required: true,
            serializedName: 'type',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          host: {
            required: true,
            serializedName: 'typeProperties.host',
            type: {
              name: 'Object'
            }
          },
          accessToken: {
            required: false,
            serializedName: 'typeProperties.accessToken',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'type',
                clientName: 'type'
              },
              uberParent: 'SecretBase',
              className: 'SecretBase'
            }
          },
          useEncryptedEndpoints: {
            required: false,
            serializedName: 'typeProperties.useEncryptedEndpoints',
            type: {
              name: 'Object'
            }
          },
          useHostVerification: {
            required: false,
            serializedName: 'typeProperties.useHostVerification',
            type: {
              name: 'Object'
            }
          },
          usePeerVerification: {
            required: false,
            serializedName: 'typeProperties.usePeerVerification',
            type: {
              name: 'Object'
            }
          },
          encryptedCredential: {
            required: false,
            serializedName: 'typeProperties.encryptedCredential',
            type: {
              name: 'Object'
            }
          }
        }
      }
    };
  }
}

module.exports = MagentoLinkedService;
