/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Rest Service linked service.
 *
 * @extends models['LinkedService']
 */
class RestServiceLinkedService extends models['LinkedService'] {
  /**
   * Create a RestServiceLinkedService.
   * @member {object} url The base URL of the REST service.
   * @member {object} [enableServerCertificateValidation] Whether to validate
   * server side SSL certificate when connecting to the endpoint.The default
   * value is true. Type: boolean (or Expression with resultType boolean).
   * @member {string} authenticationType Type of authentication used to connect
   * to the REST service. Possible values include: 'Anonymous', 'Basic',
   * 'AadServicePrincial', 'ManagedServiceIdentity'
   * @member {object} [userName] The user name used in Basic authentication
   * type.
   * @member {object} [password] The password used in Basic authentication
   * type.
   * @member {string} [password.type] Polymorphic Discriminator
   * @member {object} [servicePrincipalId] The application’s client ID used in
   * AadServicePrincipal authentication type.
   * @member {object} [servicePrincipalKey] The application’s key used in
   * AadServicePrincipal authentication type.
   * @member {string} [servicePrincipalKey.type] Polymorphic Discriminator
   * @member {object} [tenant] The tenant information (domain name or tenant
   * ID) used in AadServicePrincipal authentication type under which your
   * application resides.
   * @member {object} [aadResourceId] The resource you are requesting
   * authorization to use.
   * @member {object} [encryptedCredential] The encrypted credential used for
   * authentication. Credentials are encrypted using the integration runtime
   * credential manager. Type: string (or Expression with resultType string).
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of RestServiceLinkedService
   *
   * @returns {object} metadata of RestServiceLinkedService
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'RestService',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: 'type',
          clientName: 'type'
        },
        uberParent: 'LinkedService',
        className: 'RestServiceLinkedService',
        modelProperties: {
          connectVia: {
            required: false,
            serializedName: 'connectVia',
            type: {
              name: 'Composite',
              className: 'IntegrationRuntimeReference'
            }
          },
          description: {
            required: false,
            serializedName: 'description',
            type: {
              name: 'String'
            }
          },
          parameters: {
            required: false,
            serializedName: 'parameters',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'ParameterSpecificationElementType',
                  type: {
                    name: 'Composite',
                    className: 'ParameterSpecification'
                  }
              }
            }
          },
          annotations: {
            required: false,
            serializedName: 'annotations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ObjectElementType',
                  type: {
                    name: 'Object'
                  }
              }
            }
          },
          type: {
            required: true,
            serializedName: 'type',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          url: {
            required: true,
            serializedName: 'typeProperties.url',
            type: {
              name: 'Object'
            }
          },
          enableServerCertificateValidation: {
            required: false,
            serializedName: 'typeProperties.enableServerCertificateValidation',
            type: {
              name: 'Object'
            }
          },
          authenticationType: {
            required: true,
            serializedName: 'typeProperties.authenticationType',
            type: {
              name: 'String'
            }
          },
          userName: {
            required: false,
            serializedName: 'typeProperties.userName',
            type: {
              name: 'Object'
            }
          },
          password: {
            required: false,
            serializedName: 'typeProperties.password',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'type',
                clientName: 'type'
              },
              uberParent: 'SecretBase',
              className: 'SecretBase'
            }
          },
          servicePrincipalId: {
            required: false,
            serializedName: 'typeProperties.servicePrincipalId',
            type: {
              name: 'Object'
            }
          },
          servicePrincipalKey: {
            required: false,
            serializedName: 'typeProperties.servicePrincipalKey',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'type',
                clientName: 'type'
              },
              uberParent: 'SecretBase',
              className: 'SecretBase'
            }
          },
          tenant: {
            required: false,
            serializedName: 'typeProperties.tenant',
            type: {
              name: 'Object'
            }
          },
          aadResourceId: {
            required: false,
            serializedName: 'typeProperties.aadResourceId',
            type: {
              name: 'Object'
            }
          },
          encryptedCredential: {
            required: false,
            serializedName: 'typeProperties.encryptedCredential',
            type: {
              name: 'Object'
            }
          }
        }
      }
    };
  }
}

module.exports = RestServiceLinkedService;
