/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * Properties for creating a virtual machine.
 *
 */
class LabVirtualMachineCreationParameter {
  /**
   * Create a LabVirtualMachineCreationParameter.
   * @property {object} [bulkCreationParameters] The number of virtual machine
   * instances to create.
   * @property {number} [bulkCreationParameters.instanceCount] The number of
   * virtual machine instances to create.
   * @property {string} [notes] The notes of the virtual machine.
   * @property {string} [ownerObjectId] The object identifier of the owner of
   * the virtual machine.
   * @property {string} [ownerUserPrincipalName] The user principal name of the
   * virtual machine owner.
   * @property {string} [createdByUserId] The object identifier of the creator
   * of the virtual machine.
   * @property {string} [createdByUser] The email address of creator of the
   * virtual machine.
   * @property {date} [createdDate] The creation date of the virtual machine.
   * @property {string} [computeId] The resource identifier (Microsoft.Compute)
   * of the virtual machine.
   * @property {string} [customImageId] The custom image identifier of the
   * virtual machine.
   * @property {string} [osType] The OS type of the virtual machine.
   * @property {string} [size] The size of the virtual machine.
   * @property {string} [userName] The user name of the virtual machine.
   * @property {string} [password] The password of the virtual machine
   * administrator.
   * @property {string} [sshKey] The SSH key of the virtual machine
   * administrator.
   * @property {boolean} [isAuthenticationWithSshKey] Indicates whether this
   * virtual machine uses an SSH key for authentication.
   * @property {string} [fqdn] The fully-qualified domain name of the virtual
   * machine.
   * @property {string} [labSubnetName] The lab subnet name of the virtual
   * machine.
   * @property {string} [labVirtualNetworkId] The lab virtual network
   * identifier of the virtual machine.
   * @property {boolean} [disallowPublicIpAddress] Indicates whether the
   * virtual machine is to be created without a public IP address.
   * @property {array} [artifacts] The artifacts to be installed on the virtual
   * machine.
   * @property {object} [artifactDeploymentStatus] The artifact deployment
   * status for the virtual machine.
   * @property {string} [artifactDeploymentStatus.deploymentStatus] The
   * deployment status of the artifact.
   * @property {number} [artifactDeploymentStatus.artifactsApplied] The total
   * count of the artifacts that were successfully applied.
   * @property {number} [artifactDeploymentStatus.totalArtifacts] The total
   * count of the artifacts that were tentatively applied.
   * @property {object} [galleryImageReference] The Microsoft Azure Marketplace
   * image reference of the virtual machine.
   * @property {string} [galleryImageReference.offer] The offer of the gallery
   * image.
   * @property {string} [galleryImageReference.publisher] The publisher of the
   * gallery image.
   * @property {string} [galleryImageReference.sku] The SKU of the gallery
   * image.
   * @property {string} [galleryImageReference.osType] The OS type of the
   * gallery image.
   * @property {string} [galleryImageReference.version] The version of the
   * gallery image.
   * @property {string} [planId] The id of the plan associated with the virtual
   * machine image
   * @property {object} [networkInterface] The network interface properties.
   * @property {string} [networkInterface.virtualNetworkId] The resource ID of
   * the virtual network.
   * @property {string} [networkInterface.subnetId] The resource ID of the sub
   * net.
   * @property {string} [networkInterface.publicIpAddressId] The resource ID of
   * the public IP address.
   * @property {string} [networkInterface.publicIpAddress] The public IP
   * address.
   * @property {string} [networkInterface.privateIpAddress] The private IP
   * address.
   * @property {string} [networkInterface.dnsName] The DNS name.
   * @property {string} [networkInterface.rdpAuthority] The RdpAuthority
   * property is a server DNS host name or IP address followed by the service
   * port number for RDP (Remote Desktop Protocol).
   * @property {string} [networkInterface.sshAuthority] The SshAuthority
   * property is a server DNS host name or IP address followed by the service
   * port number for SSH.
   * @property {object} [networkInterface.sharedPublicIpAddressConfiguration]
   * The configuration for sharing a public IP address across multiple virtual
   * machines.
   * @property {array}
   * [networkInterface.sharedPublicIpAddressConfiguration.inboundNatRules] The
   * incoming NAT rules
   * @property {date} [expirationDate] The expiration date for VM.
   * @property {boolean} [allowClaim] Indicates whether another user can take
   * ownership of the virtual machine
   * @property {string} [storageType] Storage type to use for virtual machine
   * (i.e. Standard, Premium).
   * @property {string} [virtualMachineCreationSource] Tells source of creation
   * of lab virtual machine. Output property only. Possible values include:
   * 'FromCustomImage', 'FromGalleryImage'
   * @property {string} [environmentId] The resource ID of the environment that
   * contains this virtual machine, if any.
   * @property {array} [dataDiskParameters] New or existing data disks to
   * attach to the virtual machine after creation
   * @property {array} [scheduleParameters] Virtual Machine schedules to be
   * created
   * @property {string} [lastKnownPowerState] Last known compute power state
   * captured in DTL
   * @property {string} [name] The name of the virtual machine or environment
   * @property {string} [location] The location of the new virtual machine or
   * environment
   * @property {object} [tags] The tags of the resource.
   */
  constructor() {
  }

  /**
   * Defines the metadata of LabVirtualMachineCreationParameter
   *
   * @returns {object} metadata of LabVirtualMachineCreationParameter
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'LabVirtualMachineCreationParameter',
      type: {
        name: 'Composite',
        className: 'LabVirtualMachineCreationParameter',
        modelProperties: {
          bulkCreationParameters: {
            required: false,
            serializedName: 'properties.bulkCreationParameters',
            type: {
              name: 'Composite',
              className: 'BulkCreationParameters'
            }
          },
          notes: {
            required: false,
            serializedName: 'properties.notes',
            type: {
              name: 'String'
            }
          },
          ownerObjectId: {
            required: false,
            serializedName: 'properties.ownerObjectId',
            type: {
              name: 'String'
            }
          },
          ownerUserPrincipalName: {
            required: false,
            serializedName: 'properties.ownerUserPrincipalName',
            type: {
              name: 'String'
            }
          },
          createdByUserId: {
            required: false,
            serializedName: 'properties.createdByUserId',
            type: {
              name: 'String'
            }
          },
          createdByUser: {
            required: false,
            serializedName: 'properties.createdByUser',
            type: {
              name: 'String'
            }
          },
          createdDate: {
            required: false,
            serializedName: 'properties.createdDate',
            type: {
              name: 'DateTime'
            }
          },
          computeId: {
            required: false,
            serializedName: 'properties.computeId',
            type: {
              name: 'String'
            }
          },
          customImageId: {
            required: false,
            serializedName: 'properties.customImageId',
            type: {
              name: 'String'
            }
          },
          osType: {
            required: false,
            serializedName: 'properties.osType',
            type: {
              name: 'String'
            }
          },
          size: {
            required: false,
            serializedName: 'properties.size',
            type: {
              name: 'String'
            }
          },
          userName: {
            required: false,
            serializedName: 'properties.userName',
            type: {
              name: 'String'
            }
          },
          password: {
            required: false,
            serializedName: 'properties.password',
            type: {
              name: 'String'
            }
          },
          sshKey: {
            required: false,
            serializedName: 'properties.sshKey',
            type: {
              name: 'String'
            }
          },
          isAuthenticationWithSshKey: {
            required: false,
            serializedName: 'properties.isAuthenticationWithSshKey',
            type: {
              name: 'Boolean'
            }
          },
          fqdn: {
            required: false,
            serializedName: 'properties.fqdn',
            type: {
              name: 'String'
            }
          },
          labSubnetName: {
            required: false,
            serializedName: 'properties.labSubnetName',
            type: {
              name: 'String'
            }
          },
          labVirtualNetworkId: {
            required: false,
            serializedName: 'properties.labVirtualNetworkId',
            type: {
              name: 'String'
            }
          },
          disallowPublicIpAddress: {
            required: false,
            serializedName: 'properties.disallowPublicIpAddress',
            type: {
              name: 'Boolean'
            }
          },
          artifacts: {
            required: false,
            serializedName: 'properties.artifacts',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ArtifactInstallPropertiesElementType',
                  type: {
                    name: 'Composite',
                    className: 'ArtifactInstallProperties'
                  }
              }
            }
          },
          artifactDeploymentStatus: {
            required: false,
            serializedName: 'properties.artifactDeploymentStatus',
            type: {
              name: 'Composite',
              className: 'ArtifactDeploymentStatusProperties'
            }
          },
          galleryImageReference: {
            required: false,
            serializedName: 'properties.galleryImageReference',
            type: {
              name: 'Composite',
              className: 'GalleryImageReference'
            }
          },
          planId: {
            required: false,
            serializedName: 'properties.planId',
            type: {
              name: 'String'
            }
          },
          networkInterface: {
            required: false,
            serializedName: 'properties.networkInterface',
            type: {
              name: 'Composite',
              className: 'NetworkInterfaceProperties'
            }
          },
          expirationDate: {
            required: false,
            serializedName: 'properties.expirationDate',
            type: {
              name: 'DateTime'
            }
          },
          allowClaim: {
            required: false,
            serializedName: 'properties.allowClaim',
            type: {
              name: 'Boolean'
            }
          },
          storageType: {
            required: false,
            serializedName: 'properties.storageType',
            type: {
              name: 'String'
            }
          },
          virtualMachineCreationSource: {
            required: false,
            serializedName: 'properties.virtualMachineCreationSource',
            type: {
              name: 'String'
            }
          },
          environmentId: {
            required: false,
            serializedName: 'properties.environmentId',
            type: {
              name: 'String'
            }
          },
          dataDiskParameters: {
            required: false,
            serializedName: 'properties.dataDiskParameters',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'DataDiskPropertiesElementType',
                  type: {
                    name: 'Composite',
                    className: 'DataDiskProperties'
                  }
              }
            }
          },
          scheduleParameters: {
            required: false,
            serializedName: 'properties.scheduleParameters',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ScheduleCreationParameterElementType',
                  type: {
                    name: 'Composite',
                    className: 'ScheduleCreationParameter'
                  }
              }
            }
          },
          lastKnownPowerState: {
            required: false,
            serializedName: 'properties.lastKnownPowerState',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          location: {
            required: false,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = LabVirtualMachineCreationParameter;
