/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Defines an image
 *
 * @extends models['MediaObject']
 */
class ImageObject extends models['MediaObject'] {
  /**
   * Create a ImageObject.
   * @property {object} [thumbnail] The URL to a thumbnail of the image
   * @property {string} [imageInsightsToken] The token that you use in a
   * subsequent call to the Image Search API to get additional information
   * about the image. For information about using this token, see the
   * insightsToken query parameter.
   * @property {object} [insightsMetadata] A count of the number of websites
   * where you can shop or perform other actions related to the image. For
   * example, if the image is of an apple pie, this object includes a count of
   * the number of websites where you can buy an apple pie. To indicate the
   * number of offers in your UX, include badging such as a shopping cart icon
   * that contains the count. When the user clicks on the icon, use
   * imageInsightsToken to get the list of websites.
   * @property {number} [insightsMetadata.shoppingSourcesCount] The number of
   * websites that offer goods of the products seen in the image.
   * @property {number} [insightsMetadata.recipeSourcesCount] The number of
   * websites that offer recipes of the food seen in the image.
   * @property {object} [insightsMetadata.aggregateOffer] A summary of the
   * online offers of products found in the image. For example, if the image is
   * of a dress, the offer might identify the lowest price and the number of
   * offers found. Only visually similar products insights include this field.
   * The offer includes the following fields: Name, AggregateRating,
   * OfferCount, and LowPrice.
   * @property {array} [insightsMetadata.aggregateOffer.offers] A list of
   * offers from merchants that have offerings related to the image.
   * @property {string} [imageId] Unique Id for the image
   * @property {string} [accentColor] A three-byte hexadecimal number that
   * represents the color that dominates the image. Use the color as the
   * temporary background in your client until the image is loaded.
   * @property {string} [visualWords] Visual representation of the image. Used
   * for getting more sizes
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of ImageObject
   *
   * @returns {object} metadata of ImageObject
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ImageObject',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: '_type',
          clientName: '_type'
        },
        uberParent: 'ResponseBase',
        className: 'ImageObject',
        modelProperties: {
          _type: {
            required: true,
            serializedName: '_type',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          readLink: {
            required: false,
            readOnly: true,
            serializedName: 'readLink',
            type: {
              name: 'String'
            }
          },
          webSearchUrl: {
            required: false,
            readOnly: true,
            serializedName: 'webSearchUrl',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          url: {
            required: false,
            readOnly: true,
            serializedName: 'url',
            type: {
              name: 'String'
            }
          },
          image: {
            required: false,
            readOnly: true,
            serializedName: 'image',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: '_type',
                clientName: '_type'
              },
              uberParent: 'ResponseBase',
              className: 'ImageObject'
            }
          },
          description: {
            required: false,
            readOnly: true,
            serializedName: 'description',
            type: {
              name: 'String'
            }
          },
          alternateName: {
            required: false,
            readOnly: true,
            serializedName: 'alternateName',
            type: {
              name: 'String'
            }
          },
          bingId: {
            required: false,
            readOnly: true,
            serializedName: 'bingId',
            type: {
              name: 'String'
            }
          },
          thumbnailUrl: {
            required: false,
            readOnly: true,
            serializedName: 'thumbnailUrl',
            type: {
              name: 'String'
            }
          },
          provider: {
            required: false,
            readOnly: true,
            serializedName: 'provider',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ThingElementType',
                  type: {
                    name: 'Composite',
                    polymorphicDiscriminator: {
                      serializedName: '_type',
                      clientName: '_type'
                    },
                    uberParent: 'ResponseBase',
                    className: 'Thing'
                  }
              }
            }
          },
          datePublished: {
            required: false,
            readOnly: true,
            serializedName: 'datePublished',
            type: {
              name: 'String'
            }
          },
          text: {
            required: false,
            readOnly: true,
            serializedName: 'text',
            type: {
              name: 'String'
            }
          },
          contentUrl: {
            required: false,
            readOnly: true,
            serializedName: 'contentUrl',
            type: {
              name: 'String'
            }
          },
          hostPageUrl: {
            required: false,
            readOnly: true,
            serializedName: 'hostPageUrl',
            type: {
              name: 'String'
            }
          },
          contentSize: {
            required: false,
            readOnly: true,
            serializedName: 'contentSize',
            type: {
              name: 'String'
            }
          },
          encodingFormat: {
            required: false,
            readOnly: true,
            serializedName: 'encodingFormat',
            type: {
              name: 'String'
            }
          },
          hostPageDisplayUrl: {
            required: false,
            readOnly: true,
            serializedName: 'hostPageDisplayUrl',
            type: {
              name: 'String'
            }
          },
          width: {
            required: false,
            readOnly: true,
            serializedName: 'width',
            type: {
              name: 'Number'
            }
          },
          height: {
            required: false,
            readOnly: true,
            serializedName: 'height',
            type: {
              name: 'Number'
            }
          },
          thumbnail: {
            required: false,
            readOnly: true,
            serializedName: 'thumbnail',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: '_type',
                clientName: '_type'
              },
              uberParent: 'ResponseBase',
              className: 'ImageObject'
            }
          },
          imageInsightsToken: {
            required: false,
            readOnly: true,
            serializedName: 'imageInsightsToken',
            type: {
              name: 'String'
            }
          },
          insightsMetadata: {
            required: false,
            readOnly: true,
            serializedName: 'insightsMetadata',
            type: {
              name: 'Composite',
              className: 'ImagesImageMetadata'
            }
          },
          imageId: {
            required: false,
            readOnly: true,
            serializedName: 'imageId',
            type: {
              name: 'String'
            }
          },
          accentColor: {
            required: false,
            readOnly: true,
            serializedName: 'accentColor',
            type: {
              name: 'String'
            }
          },
          visualWords: {
            required: false,
            readOnly: true,
            serializedName: 'visualWords',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = ImageObject;
