/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * The IoT Central application.
 *
 * @extends models['Resource']
 */
class App extends models['Resource'] {
  /**
   * Create a App.
   * @member {string} [applicationId] The ID of the application.
   * @member {string} [displayName] The display name of the application.
   * @member {string} [subdomain] The subdomain of the application.
   * @member {string} [template] The ID of the application template, which is a
   * blueprint that defines the characteristics and behaviors of an
   * application. Optional; if not specified, defaults to a blank blueprint and
   * allows the application to be defined from scratch.
   * @member {object} sku A valid instance SKU.
   * @member {string} [sku.name] The name of the SKU. Possible values include:
   * 'F1', 'S1'
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of App
   *
   * @returns {object} metadata of App
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'App',
      type: {
        name: 'Composite',
        className: 'App',
        modelProperties: {
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            constraints: {
              Pattern: '^(?![0-9]+$)(?!-)[a-zA-Z0-9-]{2,99}[a-zA-Z0-9]$'
            },
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          location: {
            required: true,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          applicationId: {
            required: false,
            readOnly: true,
            serializedName: 'properties.applicationId',
            type: {
              name: 'String'
            }
          },
          displayName: {
            required: false,
            serializedName: 'properties.displayName',
            constraints: {
              Pattern: '^.{1,200}$'
            },
            type: {
              name: 'String'
            }
          },
          subdomain: {
            required: false,
            serializedName: 'properties.subdomain',
            constraints: {
              Pattern: '^[a-z0-9-]{1,63}$'
            },
            type: {
              name: 'String'
            }
          },
          template: {
            required: false,
            serializedName: 'properties.template',
            type: {
              name: 'String'
            }
          },
          sku: {
            required: true,
            serializedName: 'sku',
            type: {
              name: 'Composite',
              className: 'AppSkuInfo'
            }
          }
        }
      }
    };
  }
}

module.exports = App;
