/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the AppSkuInfo class.
 * @constructor
 * Information about the SKU of the IoT Central application.
 *
 * @member {string} name The name of the SKU. Possible values include: 'F1',
 * 'S1'
 */
export interface AppSkuInfo {
  name: string;
}

/**
 * @class
 * Initializes a new instance of the Resource class.
 * @constructor
 * The common properties of an ARM resource.
 *
 * @member {string} [id] The ARM resource identifier.
 * @member {string} [name] The ARM resource name.
 * @member {string} [type] The resource type.
 * @member {string} location The resource location.
 * @member {object} [tags] The resource tags.
 */
export interface Resource extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
  location: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the App class.
 * @constructor
 * The IoT Central application.
 *
 * @member {string} [applicationId] The ID of the application.
 * @member {string} [displayName] The display name of the application.
 * @member {string} [subdomain] The subdomain of the application.
 * @member {string} [template] The ID of the application template, which is a
 * blueprint that defines the characteristics and behaviors of an application.
 * Optional; if not specified, defaults to a blank blueprint and allows the
 * application to be defined from scratch.
 * @member {object} sku A valid instance SKU.
 * @member {string} [sku.name] The name of the SKU. Possible values include:
 * 'F1', 'S1'
 */
export interface App extends Resource {
  readonly applicationId?: string;
  displayName?: string;
  subdomain?: string;
  template?: string;
  sku: AppSkuInfo;
}

/**
 * @class
 * Initializes a new instance of the AppPatch class.
 * @constructor
 * The description of the IoT Central application.
 *
 * @member {object} [tags] Instance tags
 * @member {string} [applicationId] The ID of the application.
 * @member {string} [displayName] The display name of the application.
 * @member {string} [subdomain] The subdomain of the application.
 * @member {string} [template] The ID of the application template, which is a
 * blueprint that defines the characteristics and behaviors of an application.
 * Optional; if not specified, defaults to a blank blueprint and allows the
 * application to be defined from scratch.
 */
export interface AppPatch {
  tags?: { [propertyName: string]: string };
  readonly applicationId?: string;
  displayName?: string;
  subdomain?: string;
  template?: string;
}

/**
 * @class
 * Initializes a new instance of the ErrorResponseBody class.
 * @constructor
 * Details of error response.
 *
 * @member {string} [code] The error code.
 * @member {string} [message] The error message.
 * @member {string} [target] The target of the particular error.
 * @member {array} [details] A list of additional details about the error.
 */
export interface ErrorResponseBody {
  readonly code?: string;
  readonly message?: string;
  readonly target?: string;
  details?: ErrorResponseBody[];
}

/**
 * @class
 * Initializes a new instance of the ErrorDetails class.
 * @constructor
 * Error details.
 *
 * @member {string} [code] The error code.
 * @member {string} [message] The error message.
 * @member {string} [target] The target of the particular error.
 * @member {array} [details] A list of additional details about the error.
 */
export interface ErrorDetails {
  readonly code?: string;
  readonly message?: string;
  readonly target?: string;
  details?: ErrorResponseBody[];
}

/**
 * @class
 * Initializes a new instance of the OperationDisplay class.
 * @constructor
 * The object that represents the operation.
 *
 * @member {string} [provider] Service provider: Microsoft IoT Central
 * @member {string} [resource] Resource Type: IoT Central
 * @member {string} [operation] Name of the operation
 * @member {string} [description] Friendly description for the operation,
 */
export interface OperationDisplay {
  readonly provider?: string;
  readonly resource?: string;
  readonly operation?: string;
  readonly description?: string;
}

/**
 * @class
 * Initializes a new instance of the Operation class.
 * @constructor
 * IoT Central REST API operation
 *
 * @member {string} [name] Operation name: {provider}/{resource}/{read | write
 * | action | delete}
 * @member {object} [display] The object that represents the operation.
 * @member {string} [display.provider] Service provider: Microsoft IoT Central
 * @member {string} [display.resource] Resource Type: IoT Central
 * @member {string} [display.operation] Name of the operation
 * @member {string} [display.description] Friendly description for the
 * operation,
 */
export interface Operation {
  readonly name?: string;
  display?: OperationDisplay;
}

/**
 * @class
 * Initializes a new instance of the OperationInputs class.
 * @constructor
 * Input values.
 *
 * @member {string} name The name of the IoT Central application instance to
 * check.
 * @member {string} [type] The type of the IoT Central resource to query.
 * Default value: 'IoTApps' .
 */
export interface OperationInputs {
  name: string;
  type?: string;
}

/**
 * @class
 * Initializes a new instance of the AppAvailabilityInfo class.
 * @constructor
 * The properties indicating whether a given IoT Central application name or
 * subdomain is available.
 *
 * @member {boolean} [nameAvailable] The value which indicates whether the
 * provided name is available.
 * @member {string} [reason] The reason for unavailability.
 * @member {string} [message] The detailed reason message.
 */
export interface AppAvailabilityInfo {
  readonly nameAvailable?: boolean;
  readonly reason?: string;
  readonly message?: string;
}


/**
 * @class
 * Initializes a new instance of the AppListResult class.
 * @constructor
 * A list of IoT Central Applications with a next link.
 *
 * @member {string} [nextLink] The link used to get the next page of IoT
 * Central Applications.
 */
export interface AppListResult extends Array<App> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationListResult class.
 * @constructor
 * A list of IoT Central operations. It contains a list of operations and a URL
 * link to get the next set of results.
 *
 * @member {string} [nextLink] The link used to get the next page of IoT
 * Central description objects.
 */
export interface OperationListResult extends Array<Operation> {
  nextLink?: string;
}
