/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
*/

import { ServiceClientOptions, RequestOptions, ServiceCallback, HttpOperationResponse } from 'ms-rest';
import * as models from '../models';


/**
 * @class
 * Operations
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the IotHubClient.
 */
export interface Operations {


    /**
     * Lists all of the available IoT Hub REST API operations.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<OperationListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.OperationListResult>>;

    /**
     * Lists all of the available IoT Hub REST API operations.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {OperationListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {OperationListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link OperationListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.OperationListResult>;
    list(callback: ServiceCallback<models.OperationListResult>): void;
    list(options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.OperationListResult>): void;


    /**
     * Lists all of the available IoT Hub REST API operations.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<OperationListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.OperationListResult>>;

    /**
     * Lists all of the available IoT Hub REST API operations.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {OperationListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {OperationListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link OperationListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.OperationListResult>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.OperationListResult>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.OperationListResult>): void;
}

/**
 * @class
 * IotHubResource
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the IotHubClient.
 */
export interface IotHubResource {


    /**
     * @summary Get the non-security related metadata of an IoT hub
     *
     * Get the non-security related metadata of an IoT hub.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<IotHubDescription>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, resourceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.IotHubDescription>>;

    /**
     * @summary Get the non-security related metadata of an IoT hub
     *
     * Get the non-security related metadata of an IoT hub.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {IotHubDescription} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {IotHubDescription} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link IotHubDescription} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, resourceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.IotHubDescription>;
    get(resourceGroupName: string, resourceName: string, callback: ServiceCallback<models.IotHubDescription>): void;
    get(resourceGroupName: string, resourceName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.IotHubDescription>): void;


    /**
     * @summary Create or update the metadata of an IoT hub.
     *
     * Create or update the metadata of an Iot hub. The usual pattern to modify a
     * property is to retrieve the IoT hub metadata and security metadata, and then
     * combine them with the modified values in a new body to update the IoT hub.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} iotHubDescription The IoT hub metadata and security
     * metadata.
     *
     * @param {string} [iotHubDescription.etag] The Etag field is *not* required.
     * If it is provided in the response body, it must also be provided as a header
     * per the normal ETag convention.
     *
     * @param {object} [iotHubDescription.properties] IotHub properties
     *
     * @param {array} [iotHubDescription.properties.authorizationPolicies] The
     * shared access policies you can use to secure a connection to the IoT hub.
     *
     * @param {array} [iotHubDescription.properties.ipFilterRules] The IP filter
     * rules.
     *
     * @param {object} [iotHubDescription.properties.eventHubEndpoints] The Event
     * Hub-compatible endpoint properties. The possible keys to this dictionary are
     * events and operationsMonitoringEvents. Both of these keys have to be present
     * in the dictionary while making create or update calls for the IoT hub.
     *
     * @param {object} [iotHubDescription.properties.routing]
     *
     * @param {object} [iotHubDescription.properties.routing.endpoints]
     *
     * @param {array}
     * [iotHubDescription.properties.routing.endpoints.serviceBusQueues] The list
     * of Service Bus queue endpoints that IoT hub routes the messages to, based on
     * the routing rules.
     *
     * @param {array}
     * [iotHubDescription.properties.routing.endpoints.serviceBusTopics] The list
     * of Service Bus topic endpoints that the IoT hub routes the messages to,
     * based on the routing rules.
     *
     * @param {array} [iotHubDescription.properties.routing.endpoints.eventHubs]
     * The list of Event Hubs endpoints that IoT hub routes messages to, based on
     * the routing rules. This list does not include the built-in Event Hubs
     * endpoint.
     *
     * @param {array}
     * [iotHubDescription.properties.routing.endpoints.storageContainers] The list
     * of storage container endpoints that IoT hub routes messages to, based on the
     * routing rules.
     *
     * @param {array} [iotHubDescription.properties.routing.routes] The list of
     * user-provided routing rules that the IoT hub uses to route messages to
     * built-in and custom endpoints. A maximum of 100 routing rules are allowed
     * for paid hubs and a maximum of 5 routing rules are allowed for free hubs.
     *
     * @param {object} [iotHubDescription.properties.routing.fallbackRoute] The
     * properties of the route that is used as a fall-back route when none of the
     * conditions specified in the 'routes' section are met. This is an optional
     * parameter. When this property is not set, the messages which do not meet any
     * of the conditions specified in the 'routes' section get routed to the
     * built-in eventhub endpoint.
     *
     * @param {string} [iotHubDescription.properties.routing.fallbackRoute.name]
     * The name of the route. The name can only include alphanumeric characters,
     * periods, underscores, hyphens, has a maximum length of 64 characters, and
     * must be unique.
     *
     * @param {string}
     * [iotHubDescription.properties.routing.fallbackRoute.condition] The condition
     * which is evaluated in order to apply the fallback route. If the condition is
     * not provided it will evaluate to true by default. For grammar, See:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-query-language
     *
     * @param {array}
     * iotHubDescription.properties.routing.fallbackRoute.endpointNames The list of
     * endpoints to which the messages that satisfy the condition are routed to.
     * Currently only 1 endpoint is allowed.
     *
     * @param {boolean}
     * iotHubDescription.properties.routing.fallbackRoute.isEnabled Used to specify
     * whether the fallback route is enabled.
     *
     * @param {object} [iotHubDescription.properties.storageEndpoints] The list of
     * Azure Storage endpoints where you can upload files. Currently you can
     * configure only one Azure Storage account and that MUST have its key as
     * $default. Specifying more than one storage account causes an error to be
     * thrown. Not specifying a value for this property when the
     * enableFileUploadNotifications property is set to True, causes an error to be
     * thrown.
     *
     * @param {object} [iotHubDescription.properties.messagingEndpoints] The
     * messaging endpoint properties for the file upload notification queue.
     *
     * @param {boolean}
     * [iotHubDescription.properties.enableFileUploadNotifications] If True, file
     * upload notifications are enabled.
     *
     * @param {object} [iotHubDescription.properties.cloudToDevice]
     *
     * @param {number}
     * [iotHubDescription.properties.cloudToDevice.maxDeliveryCount] The max
     * delivery count for cloud-to-device messages in the device queue. See:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
     *
     * @param {moment.duration}
     * [iotHubDescription.properties.cloudToDevice.defaultTtlAsIso8601] The default
     * time to live for cloud-to-device messages in the device queue. See:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
     *
     * @param {object} [iotHubDescription.properties.cloudToDevice.feedback]
     *
     * @param {moment.duration}
     * [iotHubDescription.properties.cloudToDevice.feedback.lockDurationAsIso8601]
     * The lock duration for the feedback queue. See:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
     *
     * @param {moment.duration}
     * [iotHubDescription.properties.cloudToDevice.feedback.ttlAsIso8601] The
     * period of time for which a message is available to consume before it is
     * expired by the IoT hub. See:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
     *
     * @param {number}
     * [iotHubDescription.properties.cloudToDevice.feedback.maxDeliveryCount] The
     * number of times the IoT hub attempts to deliver a message on the feedback
     * queue. See:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
     *
     * @param {string} [iotHubDescription.properties.comments] IoT hub comments.
     *
     * @param {object}
     * [iotHubDescription.properties.operationsMonitoringProperties]
     *
     * @param {object}
     * [iotHubDescription.properties.operationsMonitoringProperties.events]
     *
     * @param {string} [iotHubDescription.properties.features] The capabilities and
     * features enabled for the IoT hub. Possible values include: 'None',
     * 'DeviceManagement'
     *
     * @param {object} iotHubDescription.sku IotHub SKU info
     *
     * @param {string} iotHubDescription.sku.name The name of the SKU. Possible
     * values include: 'F1', 'S1', 'S2', 'S3', 'B1', 'B2', 'B3'
     *
     * @param {number} [iotHubDescription.sku.capacity] The number of provisioned
     * IoT Hub units. See:
     * https://docs.microsoft.com/azure/azure-subscription-service-limits#iot-hub-limits.
     *
     * @param {string} iotHubDescription.location The resource location.
     *
     * @param {object} [iotHubDescription.tags] The resource tags.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.ifMatch] ETag of the IoT Hub. Do not specify for
     * creating a brand new IoT Hub. Required to update an existing IoT Hub.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<IotHubDescription>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, resourceName: string, iotHubDescription: models.IotHubDescription, options?: { ifMatch? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.IotHubDescription>>;

    /**
     * @summary Create or update the metadata of an IoT hub.
     *
     * Create or update the metadata of an Iot hub. The usual pattern to modify a
     * property is to retrieve the IoT hub metadata and security metadata, and then
     * combine them with the modified values in a new body to update the IoT hub.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} iotHubDescription The IoT hub metadata and security
     * metadata.
     *
     * @param {string} [iotHubDescription.etag] The Etag field is *not* required.
     * If it is provided in the response body, it must also be provided as a header
     * per the normal ETag convention.
     *
     * @param {object} [iotHubDescription.properties] IotHub properties
     *
     * @param {array} [iotHubDescription.properties.authorizationPolicies] The
     * shared access policies you can use to secure a connection to the IoT hub.
     *
     * @param {array} [iotHubDescription.properties.ipFilterRules] The IP filter
     * rules.
     *
     * @param {object} [iotHubDescription.properties.eventHubEndpoints] The Event
     * Hub-compatible endpoint properties. The possible keys to this dictionary are
     * events and operationsMonitoringEvents. Both of these keys have to be present
     * in the dictionary while making create or update calls for the IoT hub.
     *
     * @param {object} [iotHubDescription.properties.routing]
     *
     * @param {object} [iotHubDescription.properties.routing.endpoints]
     *
     * @param {array}
     * [iotHubDescription.properties.routing.endpoints.serviceBusQueues] The list
     * of Service Bus queue endpoints that IoT hub routes the messages to, based on
     * the routing rules.
     *
     * @param {array}
     * [iotHubDescription.properties.routing.endpoints.serviceBusTopics] The list
     * of Service Bus topic endpoints that the IoT hub routes the messages to,
     * based on the routing rules.
     *
     * @param {array} [iotHubDescription.properties.routing.endpoints.eventHubs]
     * The list of Event Hubs endpoints that IoT hub routes messages to, based on
     * the routing rules. This list does not include the built-in Event Hubs
     * endpoint.
     *
     * @param {array}
     * [iotHubDescription.properties.routing.endpoints.storageContainers] The list
     * of storage container endpoints that IoT hub routes messages to, based on the
     * routing rules.
     *
     * @param {array} [iotHubDescription.properties.routing.routes] The list of
     * user-provided routing rules that the IoT hub uses to route messages to
     * built-in and custom endpoints. A maximum of 100 routing rules are allowed
     * for paid hubs and a maximum of 5 routing rules are allowed for free hubs.
     *
     * @param {object} [iotHubDescription.properties.routing.fallbackRoute] The
     * properties of the route that is used as a fall-back route when none of the
     * conditions specified in the 'routes' section are met. This is an optional
     * parameter. When this property is not set, the messages which do not meet any
     * of the conditions specified in the 'routes' section get routed to the
     * built-in eventhub endpoint.
     *
     * @param {string} [iotHubDescription.properties.routing.fallbackRoute.name]
     * The name of the route. The name can only include alphanumeric characters,
     * periods, underscores, hyphens, has a maximum length of 64 characters, and
     * must be unique.
     *
     * @param {string}
     * [iotHubDescription.properties.routing.fallbackRoute.condition] The condition
     * which is evaluated in order to apply the fallback route. If the condition is
     * not provided it will evaluate to true by default. For grammar, See:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-query-language
     *
     * @param {array}
     * iotHubDescription.properties.routing.fallbackRoute.endpointNames The list of
     * endpoints to which the messages that satisfy the condition are routed to.
     * Currently only 1 endpoint is allowed.
     *
     * @param {boolean}
     * iotHubDescription.properties.routing.fallbackRoute.isEnabled Used to specify
     * whether the fallback route is enabled.
     *
     * @param {object} [iotHubDescription.properties.storageEndpoints] The list of
     * Azure Storage endpoints where you can upload files. Currently you can
     * configure only one Azure Storage account and that MUST have its key as
     * $default. Specifying more than one storage account causes an error to be
     * thrown. Not specifying a value for this property when the
     * enableFileUploadNotifications property is set to True, causes an error to be
     * thrown.
     *
     * @param {object} [iotHubDescription.properties.messagingEndpoints] The
     * messaging endpoint properties for the file upload notification queue.
     *
     * @param {boolean}
     * [iotHubDescription.properties.enableFileUploadNotifications] If True, file
     * upload notifications are enabled.
     *
     * @param {object} [iotHubDescription.properties.cloudToDevice]
     *
     * @param {number}
     * [iotHubDescription.properties.cloudToDevice.maxDeliveryCount] The max
     * delivery count for cloud-to-device messages in the device queue. See:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
     *
     * @param {moment.duration}
     * [iotHubDescription.properties.cloudToDevice.defaultTtlAsIso8601] The default
     * time to live for cloud-to-device messages in the device queue. See:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
     *
     * @param {object} [iotHubDescription.properties.cloudToDevice.feedback]
     *
     * @param {moment.duration}
     * [iotHubDescription.properties.cloudToDevice.feedback.lockDurationAsIso8601]
     * The lock duration for the feedback queue. See:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
     *
     * @param {moment.duration}
     * [iotHubDescription.properties.cloudToDevice.feedback.ttlAsIso8601] The
     * period of time for which a message is available to consume before it is
     * expired by the IoT hub. See:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
     *
     * @param {number}
     * [iotHubDescription.properties.cloudToDevice.feedback.maxDeliveryCount] The
     * number of times the IoT hub attempts to deliver a message on the feedback
     * queue. See:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
     *
     * @param {string} [iotHubDescription.properties.comments] IoT hub comments.
     *
     * @param {object}
     * [iotHubDescription.properties.operationsMonitoringProperties]
     *
     * @param {object}
     * [iotHubDescription.properties.operationsMonitoringProperties.events]
     *
     * @param {string} [iotHubDescription.properties.features] The capabilities and
     * features enabled for the IoT hub. Possible values include: 'None',
     * 'DeviceManagement'
     *
     * @param {object} iotHubDescription.sku IotHub SKU info
     *
     * @param {string} iotHubDescription.sku.name The name of the SKU. Possible
     * values include: 'F1', 'S1', 'S2', 'S3', 'B1', 'B2', 'B3'
     *
     * @param {number} [iotHubDescription.sku.capacity] The number of provisioned
     * IoT Hub units. See:
     * https://docs.microsoft.com/azure/azure-subscription-service-limits#iot-hub-limits.
     *
     * @param {string} iotHubDescription.location The resource location.
     *
     * @param {object} [iotHubDescription.tags] The resource tags.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.ifMatch] ETag of the IoT Hub. Do not specify for
     * creating a brand new IoT Hub. Required to update an existing IoT Hub.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {IotHubDescription} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {IotHubDescription} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link IotHubDescription} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, resourceName: string, iotHubDescription: models.IotHubDescription, options?: { ifMatch? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.IotHubDescription>;
    createOrUpdate(resourceGroupName: string, resourceName: string, iotHubDescription: models.IotHubDescription, callback: ServiceCallback<models.IotHubDescription>): void;
    createOrUpdate(resourceGroupName: string, resourceName: string, iotHubDescription: models.IotHubDescription, options: { ifMatch? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.IotHubDescription>): void;


    /**
     * @summary Update an existing IoT Hubs tags.
     *
     * Update an existing IoT Hub tags. to update other fields use the
     * CreateOrUpdate method
     *
     * @param {string} resourceGroupName Resource group identifier.
     *
     * @param {string} resourceName Name of iot hub to update.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.tags] Resource tags
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<IotHubDescription>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, resourceName: string, options?: { tags? : { [propertyName: string]: string }, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.IotHubDescription>>;

    /**
     * @summary Update an existing IoT Hubs tags.
     *
     * Update an existing IoT Hub tags. to update other fields use the
     * CreateOrUpdate method
     *
     * @param {string} resourceGroupName Resource group identifier.
     *
     * @param {string} resourceName Name of iot hub to update.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.tags] Resource tags
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {IotHubDescription} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {IotHubDescription} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link IotHubDescription} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, resourceName: string, options?: { tags? : { [propertyName: string]: string }, customHeaders? : { [headerName: string]: string; } }): Promise<models.IotHubDescription>;
    update(resourceGroupName: string, resourceName: string, callback: ServiceCallback<models.IotHubDescription>): void;
    update(resourceGroupName: string, resourceName: string, options: { tags? : { [propertyName: string]: string }, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.IotHubDescription>): void;


    /**
     * @summary Delete an IoT hub
     *
     * Delete an IoT hub.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Object>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, resourceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<any>>;

    /**
     * @summary Delete an IoT hub
     *
     * Delete an IoT hub.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Object} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Object} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, resourceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<any>;
    deleteMethod(resourceGroupName: string, resourceName: string, callback: ServiceCallback<any>): void;
    deleteMethod(resourceGroupName: string, resourceName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<any>): void;


    /**
     * @summary Get all the IoT hubs in a subscription
     *
     * Get all the IoT hubs in a subscription.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<IotHubDescriptionListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listBySubscriptionWithHttpOperationResponse(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.IotHubDescriptionListResult>>;

    /**
     * @summary Get all the IoT hubs in a subscription
     *
     * Get all the IoT hubs in a subscription.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {IotHubDescriptionListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {IotHubDescriptionListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link IotHubDescriptionListResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listBySubscription(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.IotHubDescriptionListResult>;
    listBySubscription(callback: ServiceCallback<models.IotHubDescriptionListResult>): void;
    listBySubscription(options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.IotHubDescriptionListResult>): void;


    /**
     * @summary Get all the IoT hubs in a resource group
     *
     * Get all the IoT hubs in a resource group.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<IotHubDescriptionListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByResourceGroupWithHttpOperationResponse(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.IotHubDescriptionListResult>>;

    /**
     * @summary Get all the IoT hubs in a resource group
     *
     * Get all the IoT hubs in a resource group.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {IotHubDescriptionListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {IotHubDescriptionListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link IotHubDescriptionListResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByResourceGroup(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.IotHubDescriptionListResult>;
    listByResourceGroup(resourceGroupName: string, callback: ServiceCallback<models.IotHubDescriptionListResult>): void;
    listByResourceGroup(resourceGroupName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.IotHubDescriptionListResult>): void;


    /**
     * @summary Get the statistics from an IoT hub
     *
     * Get the statistics from an IoT hub.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RegistryStatistics>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getStatsWithHttpOperationResponse(resourceGroupName: string, resourceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RegistryStatistics>>;

    /**
     * @summary Get the statistics from an IoT hub
     *
     * Get the statistics from an IoT hub.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RegistryStatistics} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RegistryStatistics} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RegistryStatistics} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getStats(resourceGroupName: string, resourceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.RegistryStatistics>;
    getStats(resourceGroupName: string, resourceName: string, callback: ServiceCallback<models.RegistryStatistics>): void;
    getStats(resourceGroupName: string, resourceName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RegistryStatistics>): void;


    /**
     * @summary Get the list of valid SKUs for an IoT hub
     *
     * Get the list of valid SKUs for an IoT hub.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<IotHubSkuDescriptionListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getValidSkusWithHttpOperationResponse(resourceGroupName: string, resourceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.IotHubSkuDescriptionListResult>>;

    /**
     * @summary Get the list of valid SKUs for an IoT hub
     *
     * Get the list of valid SKUs for an IoT hub.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {IotHubSkuDescriptionListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {IotHubSkuDescriptionListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link IotHubSkuDescriptionListResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getValidSkus(resourceGroupName: string, resourceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.IotHubSkuDescriptionListResult>;
    getValidSkus(resourceGroupName: string, resourceName: string, callback: ServiceCallback<models.IotHubSkuDescriptionListResult>): void;
    getValidSkus(resourceGroupName: string, resourceName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.IotHubSkuDescriptionListResult>): void;


    /**
     * @summary Get a list of the consumer groups in the Event Hub-compatible
     * device-to-cloud endpoint in an IoT hub
     *
     * Get a list of the consumer groups in the Event Hub-compatible
     * device-to-cloud endpoint in an IoT hub.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {string} eventHubEndpointName The name of the Event Hub-compatible
     * endpoint.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<EventHubConsumerGroupsListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listEventHubConsumerGroupsWithHttpOperationResponse(resourceGroupName: string, resourceName: string, eventHubEndpointName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.EventHubConsumerGroupsListResult>>;

    /**
     * @summary Get a list of the consumer groups in the Event Hub-compatible
     * device-to-cloud endpoint in an IoT hub
     *
     * Get a list of the consumer groups in the Event Hub-compatible
     * device-to-cloud endpoint in an IoT hub.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {string} eventHubEndpointName The name of the Event Hub-compatible
     * endpoint.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {EventHubConsumerGroupsListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {EventHubConsumerGroupsListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link EventHubConsumerGroupsListResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listEventHubConsumerGroups(resourceGroupName: string, resourceName: string, eventHubEndpointName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.EventHubConsumerGroupsListResult>;
    listEventHubConsumerGroups(resourceGroupName: string, resourceName: string, eventHubEndpointName: string, callback: ServiceCallback<models.EventHubConsumerGroupsListResult>): void;
    listEventHubConsumerGroups(resourceGroupName: string, resourceName: string, eventHubEndpointName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.EventHubConsumerGroupsListResult>): void;


    /**
     * @summary Get a consumer group from the Event Hub-compatible device-to-cloud
     * endpoint for an IoT hub
     *
     * Get a consumer group from the Event Hub-compatible device-to-cloud endpoint
     * for an IoT hub.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {string} eventHubEndpointName The name of the Event Hub-compatible
     * endpoint in the IoT hub.
     *
     * @param {string} name The name of the consumer group to retrieve.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<EventHubConsumerGroupInfo>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getEventHubConsumerGroupWithHttpOperationResponse(resourceGroupName: string, resourceName: string, eventHubEndpointName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.EventHubConsumerGroupInfo>>;

    /**
     * @summary Get a consumer group from the Event Hub-compatible device-to-cloud
     * endpoint for an IoT hub
     *
     * Get a consumer group from the Event Hub-compatible device-to-cloud endpoint
     * for an IoT hub.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {string} eventHubEndpointName The name of the Event Hub-compatible
     * endpoint in the IoT hub.
     *
     * @param {string} name The name of the consumer group to retrieve.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {EventHubConsumerGroupInfo} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {EventHubConsumerGroupInfo} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link EventHubConsumerGroupInfo} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getEventHubConsumerGroup(resourceGroupName: string, resourceName: string, eventHubEndpointName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.EventHubConsumerGroupInfo>;
    getEventHubConsumerGroup(resourceGroupName: string, resourceName: string, eventHubEndpointName: string, name: string, callback: ServiceCallback<models.EventHubConsumerGroupInfo>): void;
    getEventHubConsumerGroup(resourceGroupName: string, resourceName: string, eventHubEndpointName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.EventHubConsumerGroupInfo>): void;


    /**
     * @summary Add a consumer group to an Event Hub-compatible endpoint in an IoT
     * hub
     *
     * Add a consumer group to an Event Hub-compatible endpoint in an IoT hub.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {string} eventHubEndpointName The name of the Event Hub-compatible
     * endpoint in the IoT hub.
     *
     * @param {string} name The name of the consumer group to add.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<EventHubConsumerGroupInfo>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createEventHubConsumerGroupWithHttpOperationResponse(resourceGroupName: string, resourceName: string, eventHubEndpointName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.EventHubConsumerGroupInfo>>;

    /**
     * @summary Add a consumer group to an Event Hub-compatible endpoint in an IoT
     * hub
     *
     * Add a consumer group to an Event Hub-compatible endpoint in an IoT hub.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {string} eventHubEndpointName The name of the Event Hub-compatible
     * endpoint in the IoT hub.
     *
     * @param {string} name The name of the consumer group to add.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {EventHubConsumerGroupInfo} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {EventHubConsumerGroupInfo} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link EventHubConsumerGroupInfo} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createEventHubConsumerGroup(resourceGroupName: string, resourceName: string, eventHubEndpointName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.EventHubConsumerGroupInfo>;
    createEventHubConsumerGroup(resourceGroupName: string, resourceName: string, eventHubEndpointName: string, name: string, callback: ServiceCallback<models.EventHubConsumerGroupInfo>): void;
    createEventHubConsumerGroup(resourceGroupName: string, resourceName: string, eventHubEndpointName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.EventHubConsumerGroupInfo>): void;


    /**
     * @summary Delete a consumer group from an Event Hub-compatible endpoint in an
     * IoT hub
     *
     * Delete a consumer group from an Event Hub-compatible endpoint in an IoT hub.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {string} eventHubEndpointName The name of the Event Hub-compatible
     * endpoint in the IoT hub.
     *
     * @param {string} name The name of the consumer group to delete.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteEventHubConsumerGroupWithHttpOperationResponse(resourceGroupName: string, resourceName: string, eventHubEndpointName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * @summary Delete a consumer group from an Event Hub-compatible endpoint in an
     * IoT hub
     *
     * Delete a consumer group from an Event Hub-compatible endpoint in an IoT hub.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {string} eventHubEndpointName The name of the Event Hub-compatible
     * endpoint in the IoT hub.
     *
     * @param {string} name The name of the consumer group to delete.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteEventHubConsumerGroup(resourceGroupName: string, resourceName: string, eventHubEndpointName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteEventHubConsumerGroup(resourceGroupName: string, resourceName: string, eventHubEndpointName: string, name: string, callback: ServiceCallback<void>): void;
    deleteEventHubConsumerGroup(resourceGroupName: string, resourceName: string, eventHubEndpointName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * @summary Get a list of all the jobs in an IoT hub. For more information,
     * see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-identity-registry
     *
     * Get a list of all the jobs in an IoT hub. For more information, see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-identity-registry.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<JobResponseListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listJobsWithHttpOperationResponse(resourceGroupName: string, resourceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.JobResponseListResult>>;

    /**
     * @summary Get a list of all the jobs in an IoT hub. For more information,
     * see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-identity-registry
     *
     * Get a list of all the jobs in an IoT hub. For more information, see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-identity-registry.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {JobResponseListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {JobResponseListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link JobResponseListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listJobs(resourceGroupName: string, resourceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.JobResponseListResult>;
    listJobs(resourceGroupName: string, resourceName: string, callback: ServiceCallback<models.JobResponseListResult>): void;
    listJobs(resourceGroupName: string, resourceName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.JobResponseListResult>): void;


    /**
     * @summary Get the details of a job from an IoT hub. For more information,
     * see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-identity-registry
     *
     * Get the details of a job from an IoT hub. For more information, see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-identity-registry.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {string} jobId The job identifier.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<JobResponse>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getJobWithHttpOperationResponse(resourceGroupName: string, resourceName: string, jobId: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.JobResponse>>;

    /**
     * @summary Get the details of a job from an IoT hub. For more information,
     * see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-identity-registry
     *
     * Get the details of a job from an IoT hub. For more information, see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-identity-registry.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {string} jobId The job identifier.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {JobResponse} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {JobResponse} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link JobResponse} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getJob(resourceGroupName: string, resourceName: string, jobId: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.JobResponse>;
    getJob(resourceGroupName: string, resourceName: string, jobId: string, callback: ServiceCallback<models.JobResponse>): void;
    getJob(resourceGroupName: string, resourceName: string, jobId: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.JobResponse>): void;


    /**
     * @summary Get the quota metrics for an IoT hub
     *
     * Get the quota metrics for an IoT hub.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<IotHubQuotaMetricInfoListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getQuotaMetricsWithHttpOperationResponse(resourceGroupName: string, resourceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.IotHubQuotaMetricInfoListResult>>;

    /**
     * @summary Get the quota metrics for an IoT hub
     *
     * Get the quota metrics for an IoT hub.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {IotHubQuotaMetricInfoListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {IotHubQuotaMetricInfoListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link IotHubQuotaMetricInfoListResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getQuotaMetrics(resourceGroupName: string, resourceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.IotHubQuotaMetricInfoListResult>;
    getQuotaMetrics(resourceGroupName: string, resourceName: string, callback: ServiceCallback<models.IotHubQuotaMetricInfoListResult>): void;
    getQuotaMetrics(resourceGroupName: string, resourceName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.IotHubQuotaMetricInfoListResult>): void;


    /**
     * @summary Get the health for routing endpoints
     *
     * Get the health for routing endpoints.
     *
     * @param {string} resourceGroupName
     *
     * @param {string} iotHubName
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<EndpointHealthDataListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getEndpointHealthWithHttpOperationResponse(resourceGroupName: string, iotHubName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.EndpointHealthDataListResult>>;

    /**
     * @summary Get the health for routing endpoints
     *
     * Get the health for routing endpoints.
     *
     * @param {string} resourceGroupName
     *
     * @param {string} iotHubName
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {EndpointHealthDataListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {EndpointHealthDataListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link EndpointHealthDataListResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getEndpointHealth(resourceGroupName: string, iotHubName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.EndpointHealthDataListResult>;
    getEndpointHealth(resourceGroupName: string, iotHubName: string, callback: ServiceCallback<models.EndpointHealthDataListResult>): void;
    getEndpointHealth(resourceGroupName: string, iotHubName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.EndpointHealthDataListResult>): void;


    /**
     * @summary Check if an IoT hub name is available
     *
     * Check if an IoT hub name is available.
     *
     * @param {string} name The name of the IoT hub to check.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<IotHubNameAvailabilityInfo>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    checkNameAvailabilityWithHttpOperationResponse(name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.IotHubNameAvailabilityInfo>>;

    /**
     * @summary Check if an IoT hub name is available
     *
     * Check if an IoT hub name is available.
     *
     * @param {string} name The name of the IoT hub to check.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {IotHubNameAvailabilityInfo} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {IotHubNameAvailabilityInfo} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link IotHubNameAvailabilityInfo} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    checkNameAvailability(name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.IotHubNameAvailabilityInfo>;
    checkNameAvailability(name: string, callback: ServiceCallback<models.IotHubNameAvailabilityInfo>): void;
    checkNameAvailability(name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.IotHubNameAvailabilityInfo>): void;


    /**
     * @summary Test all routes
     *
     * Test all routes configured in this Iot Hub
     *
     * @param {object} input Input for testing all routes
     *
     * @param {string} [input.routingSource] Routing source. Possible values
     * include: 'Invalid', 'DeviceMessages', 'TwinChangeEvents',
     * 'DeviceLifecycleEvents', 'DeviceJobLifecycleEvents'
     *
     * @param {object} [input.message] Routing message
     *
     * @param {string} [input.message.body] Body of routing message
     *
     * @param {object} [input.message.appProperties] App properties
     *
     * @param {object} [input.message.systemProperties] System properties
     *
     * @param {string} iotHubName IotHub to be tested
     *
     * @param {string} resourceGroupName resource group which Iot Hub belongs to
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<TestAllRoutesResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    testAllRoutesWithHttpOperationResponse(input: models.TestAllRoutesInput, iotHubName: string, resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.TestAllRoutesResult>>;

    /**
     * @summary Test all routes
     *
     * Test all routes configured in this Iot Hub
     *
     * @param {object} input Input for testing all routes
     *
     * @param {string} [input.routingSource] Routing source. Possible values
     * include: 'Invalid', 'DeviceMessages', 'TwinChangeEvents',
     * 'DeviceLifecycleEvents', 'DeviceJobLifecycleEvents'
     *
     * @param {object} [input.message] Routing message
     *
     * @param {string} [input.message.body] Body of routing message
     *
     * @param {object} [input.message.appProperties] App properties
     *
     * @param {object} [input.message.systemProperties] System properties
     *
     * @param {string} iotHubName IotHub to be tested
     *
     * @param {string} resourceGroupName resource group which Iot Hub belongs to
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {TestAllRoutesResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {TestAllRoutesResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link TestAllRoutesResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    testAllRoutes(input: models.TestAllRoutesInput, iotHubName: string, resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.TestAllRoutesResult>;
    testAllRoutes(input: models.TestAllRoutesInput, iotHubName: string, resourceGroupName: string, callback: ServiceCallback<models.TestAllRoutesResult>): void;
    testAllRoutes(input: models.TestAllRoutesInput, iotHubName: string, resourceGroupName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.TestAllRoutesResult>): void;


    /**
     * @summary Test the new route
     *
     * Test the new route for this Iot Hub
     *
     * @param {object} input Route that needs to be tested
     *
     * @param {object} [input.message] Routing message
     *
     * @param {string} [input.message.body] Body of routing message
     *
     * @param {object} [input.message.appProperties] App properties
     *
     * @param {object} [input.message.systemProperties] System properties
     *
     * @param {object} input.route Route properties
     *
     * @param {string} input.route.name The name of the route. The name can only
     * include alphanumeric characters, periods, underscores, hyphens, has a
     * maximum length of 64 characters, and must be unique.
     *
     * @param {string} input.route.source The source that the routing rule is to be
     * applied to, such as DeviceMessages. Possible values include: 'Invalid',
     * 'DeviceMessages', 'TwinChangeEvents', 'DeviceLifecycleEvents',
     * 'DeviceJobLifecycleEvents'
     *
     * @param {string} [input.route.condition] The condition that is evaluated to
     * apply the routing rule. If no condition is provided, it evaluates to true by
     * default. For grammar, see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-query-language
     *
     * @param {array} input.route.endpointNames The list of endpoints to which
     * messages that satisfy the condition are routed. Currently only one endpoint
     * is allowed.
     *
     * @param {boolean} input.route.isEnabled Used to specify whether a route is
     * enabled.
     *
     * @param {string} iotHubName IotHub to be tested
     *
     * @param {string} resourceGroupName resource group which Iot Hub belongs to
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<TestRouteResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    testRouteWithHttpOperationResponse(input: models.TestRouteInput, iotHubName: string, resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.TestRouteResult>>;

    /**
     * @summary Test the new route
     *
     * Test the new route for this Iot Hub
     *
     * @param {object} input Route that needs to be tested
     *
     * @param {object} [input.message] Routing message
     *
     * @param {string} [input.message.body] Body of routing message
     *
     * @param {object} [input.message.appProperties] App properties
     *
     * @param {object} [input.message.systemProperties] System properties
     *
     * @param {object} input.route Route properties
     *
     * @param {string} input.route.name The name of the route. The name can only
     * include alphanumeric characters, periods, underscores, hyphens, has a
     * maximum length of 64 characters, and must be unique.
     *
     * @param {string} input.route.source The source that the routing rule is to be
     * applied to, such as DeviceMessages. Possible values include: 'Invalid',
     * 'DeviceMessages', 'TwinChangeEvents', 'DeviceLifecycleEvents',
     * 'DeviceJobLifecycleEvents'
     *
     * @param {string} [input.route.condition] The condition that is evaluated to
     * apply the routing rule. If no condition is provided, it evaluates to true by
     * default. For grammar, see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-query-language
     *
     * @param {array} input.route.endpointNames The list of endpoints to which
     * messages that satisfy the condition are routed. Currently only one endpoint
     * is allowed.
     *
     * @param {boolean} input.route.isEnabled Used to specify whether a route is
     * enabled.
     *
     * @param {string} iotHubName IotHub to be tested
     *
     * @param {string} resourceGroupName resource group which Iot Hub belongs to
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {TestRouteResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {TestRouteResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link TestRouteResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    testRoute(input: models.TestRouteInput, iotHubName: string, resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.TestRouteResult>;
    testRoute(input: models.TestRouteInput, iotHubName: string, resourceGroupName: string, callback: ServiceCallback<models.TestRouteResult>): void;
    testRoute(input: models.TestRouteInput, iotHubName: string, resourceGroupName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.TestRouteResult>): void;


    /**
     * @summary Get the security metadata for an IoT hub. For more information,
     * see: https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-security
     *
     * Get the security metadata for an IoT hub. For more information, see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-security.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<SharedAccessSignatureAuthorizationRuleListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listKeysWithHttpOperationResponse(resourceGroupName: string, resourceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.SharedAccessSignatureAuthorizationRuleListResult>>;

    /**
     * @summary Get the security metadata for an IoT hub. For more information,
     * see: https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-security
     *
     * Get the security metadata for an IoT hub. For more information, see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-security.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {SharedAccessSignatureAuthorizationRuleListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {SharedAccessSignatureAuthorizationRuleListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link
     *                      SharedAccessSignatureAuthorizationRuleListResult} for
     *                      more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listKeys(resourceGroupName: string, resourceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.SharedAccessSignatureAuthorizationRuleListResult>;
    listKeys(resourceGroupName: string, resourceName: string, callback: ServiceCallback<models.SharedAccessSignatureAuthorizationRuleListResult>): void;
    listKeys(resourceGroupName: string, resourceName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.SharedAccessSignatureAuthorizationRuleListResult>): void;


    /**
     * @summary Get a shared access policy by name from an IoT hub. For more
     * information, see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-security
     *
     * Get a shared access policy by name from an IoT hub. For more information,
     * see: https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-security.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {string} keyName The name of the shared access policy.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<SharedAccessSignatureAuthorizationRule>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getKeysForKeyNameWithHttpOperationResponse(resourceGroupName: string, resourceName: string, keyName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.SharedAccessSignatureAuthorizationRule>>;

    /**
     * @summary Get a shared access policy by name from an IoT hub. For more
     * information, see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-security
     *
     * Get a shared access policy by name from an IoT hub. For more information,
     * see: https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-security.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {string} keyName The name of the shared access policy.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {SharedAccessSignatureAuthorizationRule} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {SharedAccessSignatureAuthorizationRule} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link SharedAccessSignatureAuthorizationRule} for
     *                      more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getKeysForKeyName(resourceGroupName: string, resourceName: string, keyName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.SharedAccessSignatureAuthorizationRule>;
    getKeysForKeyName(resourceGroupName: string, resourceName: string, keyName: string, callback: ServiceCallback<models.SharedAccessSignatureAuthorizationRule>): void;
    getKeysForKeyName(resourceGroupName: string, resourceName: string, keyName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.SharedAccessSignatureAuthorizationRule>): void;


    /**
     * @summary Exports all the device identities in the IoT hub identity registry
     * to an Azure Storage blob container. For more information, see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-identity-registry#import-and-export-device-identities
     *
     * Exports all the device identities in the IoT hub identity registry to an
     * Azure Storage blob container. For more information, see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-identity-registry#import-and-export-device-identities.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} exportDevicesParameters The parameters that specify the
     * export devices operation.
     *
     * @param {string} exportDevicesParameters.exportBlobContainerUri The export
     * blob container URI.
     *
     * @param {boolean} exportDevicesParameters.excludeKeys The value indicating
     * whether keys should be excluded during export.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<JobResponse>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    exportDevicesWithHttpOperationResponse(resourceGroupName: string, resourceName: string, exportDevicesParameters: models.ExportDevicesRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.JobResponse>>;

    /**
     * @summary Exports all the device identities in the IoT hub identity registry
     * to an Azure Storage blob container. For more information, see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-identity-registry#import-and-export-device-identities
     *
     * Exports all the device identities in the IoT hub identity registry to an
     * Azure Storage blob container. For more information, see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-identity-registry#import-and-export-device-identities.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} exportDevicesParameters The parameters that specify the
     * export devices operation.
     *
     * @param {string} exportDevicesParameters.exportBlobContainerUri The export
     * blob container URI.
     *
     * @param {boolean} exportDevicesParameters.excludeKeys The value indicating
     * whether keys should be excluded during export.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {JobResponse} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {JobResponse} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link JobResponse} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    exportDevices(resourceGroupName: string, resourceName: string, exportDevicesParameters: models.ExportDevicesRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.JobResponse>;
    exportDevices(resourceGroupName: string, resourceName: string, exportDevicesParameters: models.ExportDevicesRequest, callback: ServiceCallback<models.JobResponse>): void;
    exportDevices(resourceGroupName: string, resourceName: string, exportDevicesParameters: models.ExportDevicesRequest, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.JobResponse>): void;


    /**
     * @summary Import, update, or delete device identities in the IoT hub identity
     * registry from a blob. For more information, see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-identity-registry#import-and-export-device-identities
     *
     * Import, update, or delete device identities in the IoT hub identity registry
     * from a blob. For more information, see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-identity-registry#import-and-export-device-identities.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} importDevicesParameters The parameters that specify the
     * import devices operation.
     *
     * @param {string} importDevicesParameters.inputBlobContainerUri The input blob
     * container URI.
     *
     * @param {string} importDevicesParameters.outputBlobContainerUri The output
     * blob container URI.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<JobResponse>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    importDevicesWithHttpOperationResponse(resourceGroupName: string, resourceName: string, importDevicesParameters: models.ImportDevicesRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.JobResponse>>;

    /**
     * @summary Import, update, or delete device identities in the IoT hub identity
     * registry from a blob. For more information, see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-identity-registry#import-and-export-device-identities
     *
     * Import, update, or delete device identities in the IoT hub identity registry
     * from a blob. For more information, see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-identity-registry#import-and-export-device-identities.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} importDevicesParameters The parameters that specify the
     * import devices operation.
     *
     * @param {string} importDevicesParameters.inputBlobContainerUri The input blob
     * container URI.
     *
     * @param {string} importDevicesParameters.outputBlobContainerUri The output
     * blob container URI.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {JobResponse} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {JobResponse} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link JobResponse} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    importDevices(resourceGroupName: string, resourceName: string, importDevicesParameters: models.ImportDevicesRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.JobResponse>;
    importDevices(resourceGroupName: string, resourceName: string, importDevicesParameters: models.ImportDevicesRequest, callback: ServiceCallback<models.JobResponse>): void;
    importDevices(resourceGroupName: string, resourceName: string, importDevicesParameters: models.ImportDevicesRequest, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.JobResponse>): void;


    /**
     * @summary Create or update the metadata of an IoT hub.
     *
     * Create or update the metadata of an Iot hub. The usual pattern to modify a
     * property is to retrieve the IoT hub metadata and security metadata, and then
     * combine them with the modified values in a new body to update the IoT hub.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} iotHubDescription The IoT hub metadata and security
     * metadata.
     *
     * @param {string} [iotHubDescription.etag] The Etag field is *not* required.
     * If it is provided in the response body, it must also be provided as a header
     * per the normal ETag convention.
     *
     * @param {object} [iotHubDescription.properties] IotHub properties
     *
     * @param {array} [iotHubDescription.properties.authorizationPolicies] The
     * shared access policies you can use to secure a connection to the IoT hub.
     *
     * @param {array} [iotHubDescription.properties.ipFilterRules] The IP filter
     * rules.
     *
     * @param {object} [iotHubDescription.properties.eventHubEndpoints] The Event
     * Hub-compatible endpoint properties. The possible keys to this dictionary are
     * events and operationsMonitoringEvents. Both of these keys have to be present
     * in the dictionary while making create or update calls for the IoT hub.
     *
     * @param {object} [iotHubDescription.properties.routing]
     *
     * @param {object} [iotHubDescription.properties.routing.endpoints]
     *
     * @param {array}
     * [iotHubDescription.properties.routing.endpoints.serviceBusQueues] The list
     * of Service Bus queue endpoints that IoT hub routes the messages to, based on
     * the routing rules.
     *
     * @param {array}
     * [iotHubDescription.properties.routing.endpoints.serviceBusTopics] The list
     * of Service Bus topic endpoints that the IoT hub routes the messages to,
     * based on the routing rules.
     *
     * @param {array} [iotHubDescription.properties.routing.endpoints.eventHubs]
     * The list of Event Hubs endpoints that IoT hub routes messages to, based on
     * the routing rules. This list does not include the built-in Event Hubs
     * endpoint.
     *
     * @param {array}
     * [iotHubDescription.properties.routing.endpoints.storageContainers] The list
     * of storage container endpoints that IoT hub routes messages to, based on the
     * routing rules.
     *
     * @param {array} [iotHubDescription.properties.routing.routes] The list of
     * user-provided routing rules that the IoT hub uses to route messages to
     * built-in and custom endpoints. A maximum of 100 routing rules are allowed
     * for paid hubs and a maximum of 5 routing rules are allowed for free hubs.
     *
     * @param {object} [iotHubDescription.properties.routing.fallbackRoute] The
     * properties of the route that is used as a fall-back route when none of the
     * conditions specified in the 'routes' section are met. This is an optional
     * parameter. When this property is not set, the messages which do not meet any
     * of the conditions specified in the 'routes' section get routed to the
     * built-in eventhub endpoint.
     *
     * @param {string} [iotHubDescription.properties.routing.fallbackRoute.name]
     * The name of the route. The name can only include alphanumeric characters,
     * periods, underscores, hyphens, has a maximum length of 64 characters, and
     * must be unique.
     *
     * @param {string}
     * [iotHubDescription.properties.routing.fallbackRoute.condition] The condition
     * which is evaluated in order to apply the fallback route. If the condition is
     * not provided it will evaluate to true by default. For grammar, See:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-query-language
     *
     * @param {array}
     * iotHubDescription.properties.routing.fallbackRoute.endpointNames The list of
     * endpoints to which the messages that satisfy the condition are routed to.
     * Currently only 1 endpoint is allowed.
     *
     * @param {boolean}
     * iotHubDescription.properties.routing.fallbackRoute.isEnabled Used to specify
     * whether the fallback route is enabled.
     *
     * @param {object} [iotHubDescription.properties.storageEndpoints] The list of
     * Azure Storage endpoints where you can upload files. Currently you can
     * configure only one Azure Storage account and that MUST have its key as
     * $default. Specifying more than one storage account causes an error to be
     * thrown. Not specifying a value for this property when the
     * enableFileUploadNotifications property is set to True, causes an error to be
     * thrown.
     *
     * @param {object} [iotHubDescription.properties.messagingEndpoints] The
     * messaging endpoint properties for the file upload notification queue.
     *
     * @param {boolean}
     * [iotHubDescription.properties.enableFileUploadNotifications] If True, file
     * upload notifications are enabled.
     *
     * @param {object} [iotHubDescription.properties.cloudToDevice]
     *
     * @param {number}
     * [iotHubDescription.properties.cloudToDevice.maxDeliveryCount] The max
     * delivery count for cloud-to-device messages in the device queue. See:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
     *
     * @param {moment.duration}
     * [iotHubDescription.properties.cloudToDevice.defaultTtlAsIso8601] The default
     * time to live for cloud-to-device messages in the device queue. See:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
     *
     * @param {object} [iotHubDescription.properties.cloudToDevice.feedback]
     *
     * @param {moment.duration}
     * [iotHubDescription.properties.cloudToDevice.feedback.lockDurationAsIso8601]
     * The lock duration for the feedback queue. See:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
     *
     * @param {moment.duration}
     * [iotHubDescription.properties.cloudToDevice.feedback.ttlAsIso8601] The
     * period of time for which a message is available to consume before it is
     * expired by the IoT hub. See:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
     *
     * @param {number}
     * [iotHubDescription.properties.cloudToDevice.feedback.maxDeliveryCount] The
     * number of times the IoT hub attempts to deliver a message on the feedback
     * queue. See:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
     *
     * @param {string} [iotHubDescription.properties.comments] IoT hub comments.
     *
     * @param {object}
     * [iotHubDescription.properties.operationsMonitoringProperties]
     *
     * @param {object}
     * [iotHubDescription.properties.operationsMonitoringProperties.events]
     *
     * @param {string} [iotHubDescription.properties.features] The capabilities and
     * features enabled for the IoT hub. Possible values include: 'None',
     * 'DeviceManagement'
     *
     * @param {object} iotHubDescription.sku IotHub SKU info
     *
     * @param {string} iotHubDescription.sku.name The name of the SKU. Possible
     * values include: 'F1', 'S1', 'S2', 'S3', 'B1', 'B2', 'B3'
     *
     * @param {number} [iotHubDescription.sku.capacity] The number of provisioned
     * IoT Hub units. See:
     * https://docs.microsoft.com/azure/azure-subscription-service-limits#iot-hub-limits.
     *
     * @param {string} iotHubDescription.location The resource location.
     *
     * @param {object} [iotHubDescription.tags] The resource tags.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.ifMatch] ETag of the IoT Hub. Do not specify for
     * creating a brand new IoT Hub. Required to update an existing IoT Hub.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<IotHubDescription>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginCreateOrUpdateWithHttpOperationResponse(resourceGroupName: string, resourceName: string, iotHubDescription: models.IotHubDescription, options?: { ifMatch? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.IotHubDescription>>;

    /**
     * @summary Create or update the metadata of an IoT hub.
     *
     * Create or update the metadata of an Iot hub. The usual pattern to modify a
     * property is to retrieve the IoT hub metadata and security metadata, and then
     * combine them with the modified values in a new body to update the IoT hub.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} iotHubDescription The IoT hub metadata and security
     * metadata.
     *
     * @param {string} [iotHubDescription.etag] The Etag field is *not* required.
     * If it is provided in the response body, it must also be provided as a header
     * per the normal ETag convention.
     *
     * @param {object} [iotHubDescription.properties] IotHub properties
     *
     * @param {array} [iotHubDescription.properties.authorizationPolicies] The
     * shared access policies you can use to secure a connection to the IoT hub.
     *
     * @param {array} [iotHubDescription.properties.ipFilterRules] The IP filter
     * rules.
     *
     * @param {object} [iotHubDescription.properties.eventHubEndpoints] The Event
     * Hub-compatible endpoint properties. The possible keys to this dictionary are
     * events and operationsMonitoringEvents. Both of these keys have to be present
     * in the dictionary while making create or update calls for the IoT hub.
     *
     * @param {object} [iotHubDescription.properties.routing]
     *
     * @param {object} [iotHubDescription.properties.routing.endpoints]
     *
     * @param {array}
     * [iotHubDescription.properties.routing.endpoints.serviceBusQueues] The list
     * of Service Bus queue endpoints that IoT hub routes the messages to, based on
     * the routing rules.
     *
     * @param {array}
     * [iotHubDescription.properties.routing.endpoints.serviceBusTopics] The list
     * of Service Bus topic endpoints that the IoT hub routes the messages to,
     * based on the routing rules.
     *
     * @param {array} [iotHubDescription.properties.routing.endpoints.eventHubs]
     * The list of Event Hubs endpoints that IoT hub routes messages to, based on
     * the routing rules. This list does not include the built-in Event Hubs
     * endpoint.
     *
     * @param {array}
     * [iotHubDescription.properties.routing.endpoints.storageContainers] The list
     * of storage container endpoints that IoT hub routes messages to, based on the
     * routing rules.
     *
     * @param {array} [iotHubDescription.properties.routing.routes] The list of
     * user-provided routing rules that the IoT hub uses to route messages to
     * built-in and custom endpoints. A maximum of 100 routing rules are allowed
     * for paid hubs and a maximum of 5 routing rules are allowed for free hubs.
     *
     * @param {object} [iotHubDescription.properties.routing.fallbackRoute] The
     * properties of the route that is used as a fall-back route when none of the
     * conditions specified in the 'routes' section are met. This is an optional
     * parameter. When this property is not set, the messages which do not meet any
     * of the conditions specified in the 'routes' section get routed to the
     * built-in eventhub endpoint.
     *
     * @param {string} [iotHubDescription.properties.routing.fallbackRoute.name]
     * The name of the route. The name can only include alphanumeric characters,
     * periods, underscores, hyphens, has a maximum length of 64 characters, and
     * must be unique.
     *
     * @param {string}
     * [iotHubDescription.properties.routing.fallbackRoute.condition] The condition
     * which is evaluated in order to apply the fallback route. If the condition is
     * not provided it will evaluate to true by default. For grammar, See:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-query-language
     *
     * @param {array}
     * iotHubDescription.properties.routing.fallbackRoute.endpointNames The list of
     * endpoints to which the messages that satisfy the condition are routed to.
     * Currently only 1 endpoint is allowed.
     *
     * @param {boolean}
     * iotHubDescription.properties.routing.fallbackRoute.isEnabled Used to specify
     * whether the fallback route is enabled.
     *
     * @param {object} [iotHubDescription.properties.storageEndpoints] The list of
     * Azure Storage endpoints where you can upload files. Currently you can
     * configure only one Azure Storage account and that MUST have its key as
     * $default. Specifying more than one storage account causes an error to be
     * thrown. Not specifying a value for this property when the
     * enableFileUploadNotifications property is set to True, causes an error to be
     * thrown.
     *
     * @param {object} [iotHubDescription.properties.messagingEndpoints] The
     * messaging endpoint properties for the file upload notification queue.
     *
     * @param {boolean}
     * [iotHubDescription.properties.enableFileUploadNotifications] If True, file
     * upload notifications are enabled.
     *
     * @param {object} [iotHubDescription.properties.cloudToDevice]
     *
     * @param {number}
     * [iotHubDescription.properties.cloudToDevice.maxDeliveryCount] The max
     * delivery count for cloud-to-device messages in the device queue. See:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
     *
     * @param {moment.duration}
     * [iotHubDescription.properties.cloudToDevice.defaultTtlAsIso8601] The default
     * time to live for cloud-to-device messages in the device queue. See:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
     *
     * @param {object} [iotHubDescription.properties.cloudToDevice.feedback]
     *
     * @param {moment.duration}
     * [iotHubDescription.properties.cloudToDevice.feedback.lockDurationAsIso8601]
     * The lock duration for the feedback queue. See:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
     *
     * @param {moment.duration}
     * [iotHubDescription.properties.cloudToDevice.feedback.ttlAsIso8601] The
     * period of time for which a message is available to consume before it is
     * expired by the IoT hub. See:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
     *
     * @param {number}
     * [iotHubDescription.properties.cloudToDevice.feedback.maxDeliveryCount] The
     * number of times the IoT hub attempts to deliver a message on the feedback
     * queue. See:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
     *
     * @param {string} [iotHubDescription.properties.comments] IoT hub comments.
     *
     * @param {object}
     * [iotHubDescription.properties.operationsMonitoringProperties]
     *
     * @param {object}
     * [iotHubDescription.properties.operationsMonitoringProperties.events]
     *
     * @param {string} [iotHubDescription.properties.features] The capabilities and
     * features enabled for the IoT hub. Possible values include: 'None',
     * 'DeviceManagement'
     *
     * @param {object} iotHubDescription.sku IotHub SKU info
     *
     * @param {string} iotHubDescription.sku.name The name of the SKU. Possible
     * values include: 'F1', 'S1', 'S2', 'S3', 'B1', 'B2', 'B3'
     *
     * @param {number} [iotHubDescription.sku.capacity] The number of provisioned
     * IoT Hub units. See:
     * https://docs.microsoft.com/azure/azure-subscription-service-limits#iot-hub-limits.
     *
     * @param {string} iotHubDescription.location The resource location.
     *
     * @param {object} [iotHubDescription.tags] The resource tags.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.ifMatch] ETag of the IoT Hub. Do not specify for
     * creating a brand new IoT Hub. Required to update an existing IoT Hub.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {IotHubDescription} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {IotHubDescription} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link IotHubDescription} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginCreateOrUpdate(resourceGroupName: string, resourceName: string, iotHubDescription: models.IotHubDescription, options?: { ifMatch? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.IotHubDescription>;
    beginCreateOrUpdate(resourceGroupName: string, resourceName: string, iotHubDescription: models.IotHubDescription, callback: ServiceCallback<models.IotHubDescription>): void;
    beginCreateOrUpdate(resourceGroupName: string, resourceName: string, iotHubDescription: models.IotHubDescription, options: { ifMatch? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.IotHubDescription>): void;


    /**
     * @summary Update an existing IoT Hubs tags.
     *
     * Update an existing IoT Hub tags. to update other fields use the
     * CreateOrUpdate method
     *
     * @param {string} resourceGroupName Resource group identifier.
     *
     * @param {string} resourceName Name of iot hub to update.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.tags] Resource tags
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<IotHubDescription>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginUpdateWithHttpOperationResponse(resourceGroupName: string, resourceName: string, options?: { tags? : { [propertyName: string]: string }, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.IotHubDescription>>;

    /**
     * @summary Update an existing IoT Hubs tags.
     *
     * Update an existing IoT Hub tags. to update other fields use the
     * CreateOrUpdate method
     *
     * @param {string} resourceGroupName Resource group identifier.
     *
     * @param {string} resourceName Name of iot hub to update.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.tags] Resource tags
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {IotHubDescription} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {IotHubDescription} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link IotHubDescription} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginUpdate(resourceGroupName: string, resourceName: string, options?: { tags? : { [propertyName: string]: string }, customHeaders? : { [headerName: string]: string; } }): Promise<models.IotHubDescription>;
    beginUpdate(resourceGroupName: string, resourceName: string, callback: ServiceCallback<models.IotHubDescription>): void;
    beginUpdate(resourceGroupName: string, resourceName: string, options: { tags? : { [propertyName: string]: string }, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.IotHubDescription>): void;


    /**
     * @summary Delete an IoT hub
     *
     * Delete an IoT hub.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Object>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDeleteMethodWithHttpOperationResponse(resourceGroupName: string, resourceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<any>>;

    /**
     * @summary Delete an IoT hub
     *
     * Delete an IoT hub.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Object} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Object} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDeleteMethod(resourceGroupName: string, resourceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<any>;
    beginDeleteMethod(resourceGroupName: string, resourceName: string, callback: ServiceCallback<any>): void;
    beginDeleteMethod(resourceGroupName: string, resourceName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<any>): void;


    /**
     * @summary Get all the IoT hubs in a subscription
     *
     * Get all the IoT hubs in a subscription.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<IotHubDescriptionListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listBySubscriptionNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.IotHubDescriptionListResult>>;

    /**
     * @summary Get all the IoT hubs in a subscription
     *
     * Get all the IoT hubs in a subscription.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {IotHubDescriptionListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {IotHubDescriptionListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link IotHubDescriptionListResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listBySubscriptionNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.IotHubDescriptionListResult>;
    listBySubscriptionNext(nextPageLink: string, callback: ServiceCallback<models.IotHubDescriptionListResult>): void;
    listBySubscriptionNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.IotHubDescriptionListResult>): void;


    /**
     * @summary Get all the IoT hubs in a resource group
     *
     * Get all the IoT hubs in a resource group.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<IotHubDescriptionListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByResourceGroupNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.IotHubDescriptionListResult>>;

    /**
     * @summary Get all the IoT hubs in a resource group
     *
     * Get all the IoT hubs in a resource group.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {IotHubDescriptionListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {IotHubDescriptionListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link IotHubDescriptionListResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByResourceGroupNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.IotHubDescriptionListResult>;
    listByResourceGroupNext(nextPageLink: string, callback: ServiceCallback<models.IotHubDescriptionListResult>): void;
    listByResourceGroupNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.IotHubDescriptionListResult>): void;


    /**
     * @summary Get the list of valid SKUs for an IoT hub
     *
     * Get the list of valid SKUs for an IoT hub.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<IotHubSkuDescriptionListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getValidSkusNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.IotHubSkuDescriptionListResult>>;

    /**
     * @summary Get the list of valid SKUs for an IoT hub
     *
     * Get the list of valid SKUs for an IoT hub.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {IotHubSkuDescriptionListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {IotHubSkuDescriptionListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link IotHubSkuDescriptionListResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getValidSkusNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.IotHubSkuDescriptionListResult>;
    getValidSkusNext(nextPageLink: string, callback: ServiceCallback<models.IotHubSkuDescriptionListResult>): void;
    getValidSkusNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.IotHubSkuDescriptionListResult>): void;


    /**
     * @summary Get a list of the consumer groups in the Event Hub-compatible
     * device-to-cloud endpoint in an IoT hub
     *
     * Get a list of the consumer groups in the Event Hub-compatible
     * device-to-cloud endpoint in an IoT hub.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<EventHubConsumerGroupsListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listEventHubConsumerGroupsNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.EventHubConsumerGroupsListResult>>;

    /**
     * @summary Get a list of the consumer groups in the Event Hub-compatible
     * device-to-cloud endpoint in an IoT hub
     *
     * Get a list of the consumer groups in the Event Hub-compatible
     * device-to-cloud endpoint in an IoT hub.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {EventHubConsumerGroupsListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {EventHubConsumerGroupsListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link EventHubConsumerGroupsListResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listEventHubConsumerGroupsNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.EventHubConsumerGroupsListResult>;
    listEventHubConsumerGroupsNext(nextPageLink: string, callback: ServiceCallback<models.EventHubConsumerGroupsListResult>): void;
    listEventHubConsumerGroupsNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.EventHubConsumerGroupsListResult>): void;


    /**
     * @summary Get a list of all the jobs in an IoT hub. For more information,
     * see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-identity-registry
     *
     * Get a list of all the jobs in an IoT hub. For more information, see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-identity-registry.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<JobResponseListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listJobsNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.JobResponseListResult>>;

    /**
     * @summary Get a list of all the jobs in an IoT hub. For more information,
     * see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-identity-registry
     *
     * Get a list of all the jobs in an IoT hub. For more information, see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-identity-registry.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {JobResponseListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {JobResponseListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link JobResponseListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listJobsNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.JobResponseListResult>;
    listJobsNext(nextPageLink: string, callback: ServiceCallback<models.JobResponseListResult>): void;
    listJobsNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.JobResponseListResult>): void;


    /**
     * @summary Get the quota metrics for an IoT hub
     *
     * Get the quota metrics for an IoT hub.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<IotHubQuotaMetricInfoListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getQuotaMetricsNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.IotHubQuotaMetricInfoListResult>>;

    /**
     * @summary Get the quota metrics for an IoT hub
     *
     * Get the quota metrics for an IoT hub.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {IotHubQuotaMetricInfoListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {IotHubQuotaMetricInfoListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link IotHubQuotaMetricInfoListResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getQuotaMetricsNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.IotHubQuotaMetricInfoListResult>;
    getQuotaMetricsNext(nextPageLink: string, callback: ServiceCallback<models.IotHubQuotaMetricInfoListResult>): void;
    getQuotaMetricsNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.IotHubQuotaMetricInfoListResult>): void;


    /**
     * @summary Get the health for routing endpoints
     *
     * Get the health for routing endpoints.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<EndpointHealthDataListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getEndpointHealthNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.EndpointHealthDataListResult>>;

    /**
     * @summary Get the health for routing endpoints
     *
     * Get the health for routing endpoints.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {EndpointHealthDataListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {EndpointHealthDataListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link EndpointHealthDataListResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getEndpointHealthNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.EndpointHealthDataListResult>;
    getEndpointHealthNext(nextPageLink: string, callback: ServiceCallback<models.EndpointHealthDataListResult>): void;
    getEndpointHealthNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.EndpointHealthDataListResult>): void;


    /**
     * @summary Get the security metadata for an IoT hub. For more information,
     * see: https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-security
     *
     * Get the security metadata for an IoT hub. For more information, see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-security.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<SharedAccessSignatureAuthorizationRuleListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listKeysNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.SharedAccessSignatureAuthorizationRuleListResult>>;

    /**
     * @summary Get the security metadata for an IoT hub. For more information,
     * see: https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-security
     *
     * Get the security metadata for an IoT hub. For more information, see:
     * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-security.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {SharedAccessSignatureAuthorizationRuleListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {SharedAccessSignatureAuthorizationRuleListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link
     *                      SharedAccessSignatureAuthorizationRuleListResult} for
     *                      more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listKeysNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.SharedAccessSignatureAuthorizationRuleListResult>;
    listKeysNext(nextPageLink: string, callback: ServiceCallback<models.SharedAccessSignatureAuthorizationRuleListResult>): void;
    listKeysNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.SharedAccessSignatureAuthorizationRuleListResult>): void;
}

/**
 * @class
 * ResourceProviderCommon
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the IotHubClient.
 */
export interface ResourceProviderCommon {


    /**
     * @summary Get the number of iot hubs in the subscription
     *
     * Get the number of free and paid iot hubs in the subscription
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<UserSubscriptionQuotaListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getSubscriptionQuotaWithHttpOperationResponse(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.UserSubscriptionQuotaListResult>>;

    /**
     * @summary Get the number of iot hubs in the subscription
     *
     * Get the number of free and paid iot hubs in the subscription
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {UserSubscriptionQuotaListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {UserSubscriptionQuotaListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link UserSubscriptionQuotaListResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getSubscriptionQuota(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.UserSubscriptionQuotaListResult>;
    getSubscriptionQuota(callback: ServiceCallback<models.UserSubscriptionQuotaListResult>): void;
    getSubscriptionQuota(options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.UserSubscriptionQuotaListResult>): void;
}

/**
 * @class
 * Certificates
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the IotHubClient.
 */
export interface Certificates {


    /**
     * @summary Get the certificate list.
     *
     * Returns the list of certificates.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CertificateListDescription>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByIotHubWithHttpOperationResponse(resourceGroupName: string, resourceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CertificateListDescription>>;

    /**
     * @summary Get the certificate list.
     *
     * Returns the list of certificates.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CertificateListDescription} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CertificateListDescription} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CertificateListDescription} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByIotHub(resourceGroupName: string, resourceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.CertificateListDescription>;
    listByIotHub(resourceGroupName: string, resourceName: string, callback: ServiceCallback<models.CertificateListDescription>): void;
    listByIotHub(resourceGroupName: string, resourceName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CertificateListDescription>): void;


    /**
     * @summary Get the certificate.
     *
     * Returns the certificate.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {string} certificateName The name of the certificate
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CertificateDescription>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, resourceName: string, certificateName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CertificateDescription>>;

    /**
     * @summary Get the certificate.
     *
     * Returns the certificate.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {string} certificateName The name of the certificate
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CertificateDescription} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CertificateDescription} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CertificateDescription} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, resourceName: string, certificateName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.CertificateDescription>;
    get(resourceGroupName: string, resourceName: string, certificateName: string, callback: ServiceCallback<models.CertificateDescription>): void;
    get(resourceGroupName: string, resourceName: string, certificateName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CertificateDescription>): void;


    /**
     * @summary Upload the certificate to the IoT hub.
     *
     * Adds new or replaces existing certificate.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {string} certificateName The name of the certificate
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.ifMatch] ETag of the Certificate. Do not specify
     * for creating a brand new certificate. Required to update an existing
     * certificate.
     *
     * @param {string} [options.certificate] base-64 representation of the X509
     * leaf certificate .cer file or just .pem file content.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CertificateDescription>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, resourceName: string, certificateName: string, options?: { ifMatch? : string, certificate? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CertificateDescription>>;

    /**
     * @summary Upload the certificate to the IoT hub.
     *
     * Adds new or replaces existing certificate.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {string} certificateName The name of the certificate
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.ifMatch] ETag of the Certificate. Do not specify
     * for creating a brand new certificate. Required to update an existing
     * certificate.
     *
     * @param {string} [options.certificate] base-64 representation of the X509
     * leaf certificate .cer file or just .pem file content.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CertificateDescription} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CertificateDescription} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CertificateDescription} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, resourceName: string, certificateName: string, options?: { ifMatch? : string, certificate? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.CertificateDescription>;
    createOrUpdate(resourceGroupName: string, resourceName: string, certificateName: string, callback: ServiceCallback<models.CertificateDescription>): void;
    createOrUpdate(resourceGroupName: string, resourceName: string, certificateName: string, options: { ifMatch? : string, certificate? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CertificateDescription>): void;


    /**
     * @summary Delete an X509 certificate.
     *
     * Deletes an existing X509 certificate or does nothing if it does not exist.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {string} certificateName The name of the certificate
     *
     * @param {string} ifMatch ETag of the Certificate.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, resourceName: string, certificateName: string, ifMatch: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * @summary Delete an X509 certificate.
     *
     * Deletes an existing X509 certificate or does nothing if it does not exist.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {string} certificateName The name of the certificate
     *
     * @param {string} ifMatch ETag of the Certificate.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, resourceName: string, certificateName: string, ifMatch: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, resourceName: string, certificateName: string, ifMatch: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, resourceName: string, certificateName: string, ifMatch: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * @summary Generate verification code for proof of possession flow.
     *
     * Generates verification code for proof of possession flow. The verification
     * code will be used to generate a leaf certificate.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {string} certificateName The name of the certificate
     *
     * @param {string} ifMatch ETag of the Certificate.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CertificateWithNonceDescription>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    generateVerificationCodeWithHttpOperationResponse(resourceGroupName: string, resourceName: string, certificateName: string, ifMatch: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CertificateWithNonceDescription>>;

    /**
     * @summary Generate verification code for proof of possession flow.
     *
     * Generates verification code for proof of possession flow. The verification
     * code will be used to generate a leaf certificate.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {string} certificateName The name of the certificate
     *
     * @param {string} ifMatch ETag of the Certificate.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CertificateWithNonceDescription} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CertificateWithNonceDescription} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CertificateWithNonceDescription} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    generateVerificationCode(resourceGroupName: string, resourceName: string, certificateName: string, ifMatch: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.CertificateWithNonceDescription>;
    generateVerificationCode(resourceGroupName: string, resourceName: string, certificateName: string, ifMatch: string, callback: ServiceCallback<models.CertificateWithNonceDescription>): void;
    generateVerificationCode(resourceGroupName: string, resourceName: string, certificateName: string, ifMatch: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CertificateWithNonceDescription>): void;


    /**
     * @summary Verify certificate's private key possession.
     *
     * Verifies the certificate's private key possession by providing the leaf cert
     * issued by the verifying pre uploaded certificate.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {string} certificateName The name of the certificate
     *
     * @param {string} ifMatch ETag of the Certificate.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.certificate] base-64 representation of X509
     * certificate .cer file or just .pem file content.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CertificateDescription>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    verifyWithHttpOperationResponse(resourceGroupName: string, resourceName: string, certificateName: string, ifMatch: string, options?: { certificate? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CertificateDescription>>;

    /**
     * @summary Verify certificate's private key possession.
     *
     * Verifies the certificate's private key possession by providing the leaf cert
     * issued by the verifying pre uploaded certificate.
     *
     * @param {string} resourceGroupName The name of the resource group that
     * contains the IoT hub.
     *
     * @param {string} resourceName The name of the IoT hub.
     *
     * @param {string} certificateName The name of the certificate
     *
     * @param {string} ifMatch ETag of the Certificate.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.certificate] base-64 representation of X509
     * certificate .cer file or just .pem file content.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CertificateDescription} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CertificateDescription} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CertificateDescription} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    verify(resourceGroupName: string, resourceName: string, certificateName: string, ifMatch: string, options?: { certificate? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.CertificateDescription>;
    verify(resourceGroupName: string, resourceName: string, certificateName: string, ifMatch: string, callback: ServiceCallback<models.CertificateDescription>): void;
    verify(resourceGroupName: string, resourceName: string, certificateName: string, ifMatch: string, options: { certificate? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CertificateDescription>): void;
}
