/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * The Live Event.
 *
 * @extends models['TrackedResource']
 */
class LiveEvent extends models['TrackedResource'] {
  /**
   * Create a LiveEvent.
   * @property {string} [description] The Live Event description.
   * @property {object} input The Live Event input.
   * @property {string} [input.streamingProtocol] The streaming protocol for
   * the Live Event.  This is specified at creation time and cannot be updated.
   * Possible values include: 'FragmentedMP4', 'RTMP'
   * @property {object} [input.accessControl] The access control for LiveEvent
   * Input.
   * @property {object} [input.accessControl.ip] The IP access control
   * properties.
   * @property {array} [input.accessControl.ip.allow] The IP allow list.
   * @property {string} [input.keyFrameIntervalDuration] ISO 8601 timespan
   * duration of the key frame interval duration.
   * @property {string} [input.accessToken] A unique identifier for a stream.
   * This can be specified at creation time but cannot be updated.  If omitted,
   * the service will generate a unique value.
   * @property {array} [input.endpoints] The input endpoints for the Live
   * Event.
   * @property {object} [preview] The Live Event preview.
   * @property {array} [preview.endpoints] The endpoints for preview.
   * @property {object} [preview.accessControl] The access control for
   * LiveEvent preview.
   * @property {object} [preview.accessControl.ip] The IP access control
   * properties.
   * @property {array} [preview.accessControl.ip.allow] The IP allow list.
   * @property {string} [preview.previewLocator] The identifier of the preview
   * locator in Guid format.  Specifying this at creation time allows the
   * caller to know the preview locator url before the event is created.  If
   * omitted, the service will generate a random identifier.  This value cannot
   * be updated once the live event is created.
   * @property {string} [preview.streamingPolicyName] The name of streaming
   * policy used for the LiveEvent preview.  This value is specified at
   * creation time and cannot be updated.
   * @property {string} [preview.alternativeMediaId] An Alternative Media
   * Identifier associated with the StreamingLocator created for the preview.
   * This value is specified at creation time and cannot be updated.  The
   * identifier can be used in the CustomLicenseAcquisitionUrlTemplate or the
   * CustomKeyAcquisitionUrlTemplate of the StreamingPolicy specified in the
   * StreamingPolicyName field.
   * @property {object} [encoding] The Live Event encoding.
   * @property {string} [encoding.encodingType] The encoding type for Live
   * Event.  This value is specified at creation time and cannot be updated.
   * Possible values include: 'None', 'Basic', 'Standard'
   * @property {string} [encoding.presetName] The encoding preset name.  This
   * value is specified at creation time and cannot be updated.
   * @property {string} [provisioningState] The provisioning state of the Live
   * Event.
   * @property {string} [resourceState] The resource state of the Live Event.
   * Possible values include: 'Stopped', 'Starting', 'Running', 'Stopping',
   * 'Deleting'
   * @property {object} [crossSiteAccessPolicies] The Live Event access
   * policies.
   * @property {string} [crossSiteAccessPolicies.clientAccessPolicy] The
   * content of clientaccesspolicy.xml used by Silverlight.
   * @property {string} [crossSiteAccessPolicies.crossDomainPolicy] The content
   * of crossdomain.xml used by Silverlight.
   * @property {boolean} [vanityUrl] Specifies whether to use a vanity url with
   * the Live Event.  This value is specified at creation time and cannot be
   * updated.
   * @property {array} [streamOptions] The options to use for the LiveEvent.
   * This value is specified at creation time and cannot be updated.
   * @property {date} [created] The exact time the Live Event was created.
   * @property {date} [lastModified] The exact time the Live Event was last
   * modified.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of LiveEvent
   *
   * @returns {object} metadata of LiveEvent
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'LiveEvent',
      type: {
        name: 'Composite',
        className: 'LiveEvent',
        modelProperties: {
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          location: {
            required: false,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          description: {
            required: false,
            serializedName: 'properties.description',
            type: {
              name: 'String'
            }
          },
          input: {
            required: true,
            serializedName: 'properties.input',
            type: {
              name: 'Composite',
              className: 'LiveEventInput'
            }
          },
          preview: {
            required: false,
            serializedName: 'properties.preview',
            type: {
              name: 'Composite',
              className: 'LiveEventPreview'
            }
          },
          encoding: {
            required: false,
            serializedName: 'properties.encoding',
            type: {
              name: 'Composite',
              className: 'LiveEventEncoding'
            }
          },
          provisioningState: {
            required: false,
            readOnly: true,
            serializedName: 'properties.provisioningState',
            type: {
              name: 'String'
            }
          },
          resourceState: {
            required: false,
            readOnly: true,
            serializedName: 'properties.resourceState',
            type: {
              name: 'Enum',
              allowedValues: [ 'Stopped', 'Starting', 'Running', 'Stopping', 'Deleting' ]
            }
          },
          crossSiteAccessPolicies: {
            required: false,
            serializedName: 'properties.crossSiteAccessPolicies',
            type: {
              name: 'Composite',
              className: 'CrossSiteAccessPolicies'
            }
          },
          vanityUrl: {
            required: false,
            serializedName: 'properties.vanityUrl',
            type: {
              name: 'Boolean'
            }
          },
          streamOptions: {
            required: false,
            serializedName: 'properties.streamOptions',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'StreamOptionsFlagElementType',
                  type: {
                    name: 'Enum',
                    allowedValues: [ 'Default', 'LowLatency' ]
                  }
              }
            }
          },
          created: {
            required: false,
            readOnly: true,
            serializedName: 'properties.created',
            type: {
              name: 'DateTime'
            }
          },
          lastModified: {
            required: false,
            readOnly: true,
            serializedName: 'properties.lastModified',
            type: {
              name: 'DateTime'
            }
          }
        }
      }
    };
  }
}

module.exports = LiveEvent;
