/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
*/

import { ServiceClientOptions, RequestOptions, ServiceCallback, HttpOperationResponse } from 'ms-rest';
import * as models from '../models';


/**
 * @class
 * VaultCertificates
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the RecoveryServicesClient.
 */
export interface VaultCertificates {


    /**
     * Uploads a certificate for a resource.
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {string} vaultName The name of the recovery services vault.
     *
     * @param {string} certificateName Certificate friendly name.
     *
     * @param {object} certificateRequest Input parameters for uploading the vault
     * certificate.
     *
     * @param {object} [certificateRequest.properties]
     *
     * @param {string} [certificateRequest.properties.authType] Specifies the
     * authentication type. Possible values include: 'Invalid', 'ACS', 'AAD',
     * 'AccessControlService', 'AzureActiveDirectory'
     *
     * @param {buffer} [certificateRequest.properties.certificate] The base64
     * encoded certificate raw data string
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<VaultCertificateResponse>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createWithHttpOperationResponse(resourceGroupName: string, vaultName: string, certificateName: string, certificateRequest: models.CertificateRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.VaultCertificateResponse>>;

    /**
     * Uploads a certificate for a resource.
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {string} vaultName The name of the recovery services vault.
     *
     * @param {string} certificateName Certificate friendly name.
     *
     * @param {object} certificateRequest Input parameters for uploading the vault
     * certificate.
     *
     * @param {object} [certificateRequest.properties]
     *
     * @param {string} [certificateRequest.properties.authType] Specifies the
     * authentication type. Possible values include: 'Invalid', 'ACS', 'AAD',
     * 'AccessControlService', 'AzureActiveDirectory'
     *
     * @param {buffer} [certificateRequest.properties.certificate] The base64
     * encoded certificate raw data string
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {VaultCertificateResponse} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {VaultCertificateResponse} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link VaultCertificateResponse} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    create(resourceGroupName: string, vaultName: string, certificateName: string, certificateRequest: models.CertificateRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.VaultCertificateResponse>;
    create(resourceGroupName: string, vaultName: string, certificateName: string, certificateRequest: models.CertificateRequest, callback: ServiceCallback<models.VaultCertificateResponse>): void;
    create(resourceGroupName: string, vaultName: string, certificateName: string, certificateRequest: models.CertificateRequest, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.VaultCertificateResponse>): void;
}

/**
 * @class
 * RegisteredIdentities
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the RecoveryServicesClient.
 */
export interface RegisteredIdentities {


    /**
     * Unregisters the given container from your Recovery Services vault.
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {string} vaultName The name of the recovery services vault.
     *
     * @param {string} identityName Name of the protection container to unregister.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, vaultName: string, identityName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Unregisters the given container from your Recovery Services vault.
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {string} vaultName The name of the recovery services vault.
     *
     * @param {string} identityName Name of the protection container to unregister.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, vaultName: string, identityName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, vaultName: string, identityName: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, vaultName: string, identityName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;
}

/**
 * @class
 * ReplicationUsages
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the RecoveryServicesClient.
 */
export interface ReplicationUsages {


    /**
     * Fetches the replication usages of the vault.
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {string} vaultName The name of the recovery services vault.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ReplicationUsageList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, vaultName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ReplicationUsageList>>;

    /**
     * Fetches the replication usages of the vault.
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {string} vaultName The name of the recovery services vault.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ReplicationUsageList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ReplicationUsageList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ReplicationUsageList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, vaultName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ReplicationUsageList>;
    list(resourceGroupName: string, vaultName: string, callback: ServiceCallback<models.ReplicationUsageList>): void;
    list(resourceGroupName: string, vaultName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ReplicationUsageList>): void;
}

/**
 * @class
 * RecoveryServices
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the RecoveryServicesClient.
 */
export interface RecoveryServices {


    /**
     * @summary API to check for resource name availability.
     * A name is available if no other resource exists that has the same
     * SubscriptionId, Resource Name and Type
     * or if one or more such resources exist, each of these must be GC'd and their
     * time of deletion be more than 24 Hours Ago
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {string} location Location of the resource
     *
     * @param {object} input Contains information about Resource type and Resource
     * name
     *
     * @param {string} [input.type] Describes the Resource type:
     * Microsoft.RecoveryServices/Vaults
     *
     * @param {string} [input.name] Resource name for which availability needs to
     * be checked
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CheckNameAvailabilityResultResource>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    checkNameAvailabilityWithHttpOperationResponse(resourceGroupName: string, location: string, input: models.CheckNameAvailabilityParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CheckNameAvailabilityResultResource>>;

    /**
     * @summary API to check for resource name availability.
     * A name is available if no other resource exists that has the same
     * SubscriptionId, Resource Name and Type
     * or if one or more such resources exist, each of these must be GC'd and their
     * time of deletion be more than 24 Hours Ago
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {string} location Location of the resource
     *
     * @param {object} input Contains information about Resource type and Resource
     * name
     *
     * @param {string} [input.type] Describes the Resource type:
     * Microsoft.RecoveryServices/Vaults
     *
     * @param {string} [input.name] Resource name for which availability needs to
     * be checked
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CheckNameAvailabilityResultResource} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CheckNameAvailabilityResultResource} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CheckNameAvailabilityResultResource} for
     *                      more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    checkNameAvailability(resourceGroupName: string, location: string, input: models.CheckNameAvailabilityParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.CheckNameAvailabilityResultResource>;
    checkNameAvailability(resourceGroupName: string, location: string, input: models.CheckNameAvailabilityParameters, callback: ServiceCallback<models.CheckNameAvailabilityResultResource>): void;
    checkNameAvailability(resourceGroupName: string, location: string, input: models.CheckNameAvailabilityParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CheckNameAvailabilityResultResource>): void;
}

/**
 * @class
 * Vaults
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the RecoveryServicesClient.
 */
export interface Vaults {


    /**
     * Fetches all the resources of the specified type in the subscription.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<VaultList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listBySubscriptionIdWithHttpOperationResponse(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.VaultList>>;

    /**
     * Fetches all the resources of the specified type in the subscription.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {VaultList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {VaultList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link VaultList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listBySubscriptionId(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.VaultList>;
    listBySubscriptionId(callback: ServiceCallback<models.VaultList>): void;
    listBySubscriptionId(options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.VaultList>): void;


    /**
     * Retrieve a list of Vaults.
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<VaultList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByResourceGroupWithHttpOperationResponse(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.VaultList>>;

    /**
     * Retrieve a list of Vaults.
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {VaultList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {VaultList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link VaultList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByResourceGroup(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.VaultList>;
    listByResourceGroup(resourceGroupName: string, callback: ServiceCallback<models.VaultList>): void;
    listByResourceGroup(resourceGroupName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.VaultList>): void;


    /**
     * Get the Vault details.
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {string} vaultName The name of the recovery services vault.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Vault>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, vaultName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Vault>>;

    /**
     * Get the Vault details.
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {string} vaultName The name of the recovery services vault.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Vault} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Vault} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Vault} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, vaultName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Vault>;
    get(resourceGroupName: string, vaultName: string, callback: ServiceCallback<models.Vault>): void;
    get(resourceGroupName: string, vaultName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Vault>): void;


    /**
     * Creates or updates a Recovery Services vault.
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {string} vaultName The name of the recovery services vault.
     *
     * @param {object} vault Recovery Services Vault to be created.
     *
     * @param {object} [vault.properties]
     *
     * @param {object} [vault.properties.upgradeDetails]
     *
     * @param {object} [vault.sku]
     *
     * @param {string} vault.sku.name The Sku name. Possible values include:
     * 'Standard', 'RS0'
     *
     * @param {string} vault.location Resource location.
     *
     * @param {object} [vault.tags] Resource tags.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Vault>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, vaultName: string, vault: models.Vault, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Vault>>;

    /**
     * Creates or updates a Recovery Services vault.
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {string} vaultName The name of the recovery services vault.
     *
     * @param {object} vault Recovery Services Vault to be created.
     *
     * @param {object} [vault.properties]
     *
     * @param {object} [vault.properties.upgradeDetails]
     *
     * @param {object} [vault.sku]
     *
     * @param {string} vault.sku.name The Sku name. Possible values include:
     * 'Standard', 'RS0'
     *
     * @param {string} vault.location Resource location.
     *
     * @param {object} [vault.tags] Resource tags.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Vault} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Vault} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Vault} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, vaultName: string, vault: models.Vault, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Vault>;
    createOrUpdate(resourceGroupName: string, vaultName: string, vault: models.Vault, callback: ServiceCallback<models.Vault>): void;
    createOrUpdate(resourceGroupName: string, vaultName: string, vault: models.Vault, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Vault>): void;


    /**
     * Deletes a vault.
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {string} vaultName The name of the recovery services vault.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, vaultName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Deletes a vault.
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {string} vaultName The name of the recovery services vault.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, vaultName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, vaultName: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, vaultName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Updates the vault.
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {string} vaultName The name of the recovery services vault.
     *
     * @param {object} vault Recovery Services Vault to be created.
     *
     * @param {object} [vault.properties]
     *
     * @param {object} [vault.properties.upgradeDetails]
     *
     * @param {object} [vault.sku]
     *
     * @param {string} vault.sku.name The Sku name. Possible values include:
     * 'Standard', 'RS0'
     *
     * @param {string} [vault.location] Resource location.
     *
     * @param {object} [vault.tags] Resource tags.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Vault>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, vaultName: string, vault: models.PatchVault, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Vault>>;

    /**
     * Updates the vault.
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {string} vaultName The name of the recovery services vault.
     *
     * @param {object} vault Recovery Services Vault to be created.
     *
     * @param {object} [vault.properties]
     *
     * @param {object} [vault.properties.upgradeDetails]
     *
     * @param {object} [vault.sku]
     *
     * @param {string} vault.sku.name The Sku name. Possible values include:
     * 'Standard', 'RS0'
     *
     * @param {string} [vault.location] Resource location.
     *
     * @param {object} [vault.tags] Resource tags.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Vault} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Vault} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Vault} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, vaultName: string, vault: models.PatchVault, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Vault>;
    update(resourceGroupName: string, vaultName: string, vault: models.PatchVault, callback: ServiceCallback<models.Vault>): void;
    update(resourceGroupName: string, vaultName: string, vault: models.PatchVault, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Vault>): void;


    /**
     * Fetches all the resources of the specified type in the subscription.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<VaultList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listBySubscriptionIdNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.VaultList>>;

    /**
     * Fetches all the resources of the specified type in the subscription.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {VaultList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {VaultList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link VaultList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listBySubscriptionIdNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.VaultList>;
    listBySubscriptionIdNext(nextPageLink: string, callback: ServiceCallback<models.VaultList>): void;
    listBySubscriptionIdNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.VaultList>): void;


    /**
     * Retrieve a list of Vaults.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<VaultList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByResourceGroupNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.VaultList>>;

    /**
     * Retrieve a list of Vaults.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {VaultList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {VaultList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link VaultList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByResourceGroupNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.VaultList>;
    listByResourceGroupNext(nextPageLink: string, callback: ServiceCallback<models.VaultList>): void;
    listByResourceGroupNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.VaultList>): void;
}

/**
 * @class
 * Operations
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the RecoveryServicesClient.
 */
export interface Operations {


    /**
     * Returns the list of available operations.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ClientDiscoveryResponse>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ClientDiscoveryResponse>>;

    /**
     * Returns the list of available operations.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ClientDiscoveryResponse} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ClientDiscoveryResponse} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ClientDiscoveryResponse} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ClientDiscoveryResponse>;
    list(callback: ServiceCallback<models.ClientDiscoveryResponse>): void;
    list(options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ClientDiscoveryResponse>): void;


    /**
     * Returns the list of available operations.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ClientDiscoveryResponse>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ClientDiscoveryResponse>>;

    /**
     * Returns the list of available operations.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ClientDiscoveryResponse} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ClientDiscoveryResponse} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ClientDiscoveryResponse} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ClientDiscoveryResponse>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.ClientDiscoveryResponse>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ClientDiscoveryResponse>): void;
}

/**
 * @class
 * VaultExtendedInfoOperations
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the RecoveryServicesClient.
 */
export interface VaultExtendedInfoOperations {


    /**
     * Get the vault extended info.
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {string} vaultName The name of the recovery services vault.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<VaultExtendedInfoResource>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, vaultName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.VaultExtendedInfoResource>>;

    /**
     * Get the vault extended info.
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {string} vaultName The name of the recovery services vault.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {VaultExtendedInfoResource} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {VaultExtendedInfoResource} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link VaultExtendedInfoResource} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, vaultName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.VaultExtendedInfoResource>;
    get(resourceGroupName: string, vaultName: string, callback: ServiceCallback<models.VaultExtendedInfoResource>): void;
    get(resourceGroupName: string, vaultName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.VaultExtendedInfoResource>): void;


    /**
     * Create vault extended info.
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {string} vaultName The name of the recovery services vault.
     *
     * @param {object} resourceResourceExtendedInfoDetails Details of
     * ResourceExtendedInfo
     *
     * @param {string} [resourceResourceExtendedInfoDetails.integrityKey] Integrity
     * key.
     *
     * @param {string} [resourceResourceExtendedInfoDetails.encryptionKey]
     * Encryption key.
     *
     * @param {string}
     * [resourceResourceExtendedInfoDetails.encryptionKeyThumbprint] Encryption key
     * thumbprint.
     *
     * @param {string} [resourceResourceExtendedInfoDetails.algorithm] Algorithm
     * for Vault ExtendedInfo
     *
     * @param {string} [resourceResourceExtendedInfoDetails.eTag] Optional ETag.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<VaultExtendedInfoResource>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, vaultName: string, resourceResourceExtendedInfoDetails: models.VaultExtendedInfoResource, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.VaultExtendedInfoResource>>;

    /**
     * Create vault extended info.
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {string} vaultName The name of the recovery services vault.
     *
     * @param {object} resourceResourceExtendedInfoDetails Details of
     * ResourceExtendedInfo
     *
     * @param {string} [resourceResourceExtendedInfoDetails.integrityKey] Integrity
     * key.
     *
     * @param {string} [resourceResourceExtendedInfoDetails.encryptionKey]
     * Encryption key.
     *
     * @param {string}
     * [resourceResourceExtendedInfoDetails.encryptionKeyThumbprint] Encryption key
     * thumbprint.
     *
     * @param {string} [resourceResourceExtendedInfoDetails.algorithm] Algorithm
     * for Vault ExtendedInfo
     *
     * @param {string} [resourceResourceExtendedInfoDetails.eTag] Optional ETag.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {VaultExtendedInfoResource} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {VaultExtendedInfoResource} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link VaultExtendedInfoResource} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, vaultName: string, resourceResourceExtendedInfoDetails: models.VaultExtendedInfoResource, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.VaultExtendedInfoResource>;
    createOrUpdate(resourceGroupName: string, vaultName: string, resourceResourceExtendedInfoDetails: models.VaultExtendedInfoResource, callback: ServiceCallback<models.VaultExtendedInfoResource>): void;
    createOrUpdate(resourceGroupName: string, vaultName: string, resourceResourceExtendedInfoDetails: models.VaultExtendedInfoResource, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.VaultExtendedInfoResource>): void;


    /**
     * Update vault extended info.
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {string} vaultName The name of the recovery services vault.
     *
     * @param {object} resourceResourceExtendedInfoDetails Details of
     * ResourceExtendedInfo
     *
     * @param {string} [resourceResourceExtendedInfoDetails.integrityKey] Integrity
     * key.
     *
     * @param {string} [resourceResourceExtendedInfoDetails.encryptionKey]
     * Encryption key.
     *
     * @param {string}
     * [resourceResourceExtendedInfoDetails.encryptionKeyThumbprint] Encryption key
     * thumbprint.
     *
     * @param {string} [resourceResourceExtendedInfoDetails.algorithm] Algorithm
     * for Vault ExtendedInfo
     *
     * @param {string} [resourceResourceExtendedInfoDetails.eTag] Optional ETag.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<VaultExtendedInfoResource>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, vaultName: string, resourceResourceExtendedInfoDetails: models.VaultExtendedInfoResource, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.VaultExtendedInfoResource>>;

    /**
     * Update vault extended info.
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {string} vaultName The name of the recovery services vault.
     *
     * @param {object} resourceResourceExtendedInfoDetails Details of
     * ResourceExtendedInfo
     *
     * @param {string} [resourceResourceExtendedInfoDetails.integrityKey] Integrity
     * key.
     *
     * @param {string} [resourceResourceExtendedInfoDetails.encryptionKey]
     * Encryption key.
     *
     * @param {string}
     * [resourceResourceExtendedInfoDetails.encryptionKeyThumbprint] Encryption key
     * thumbprint.
     *
     * @param {string} [resourceResourceExtendedInfoDetails.algorithm] Algorithm
     * for Vault ExtendedInfo
     *
     * @param {string} [resourceResourceExtendedInfoDetails.eTag] Optional ETag.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {VaultExtendedInfoResource} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {VaultExtendedInfoResource} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link VaultExtendedInfoResource} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, vaultName: string, resourceResourceExtendedInfoDetails: models.VaultExtendedInfoResource, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.VaultExtendedInfoResource>;
    update(resourceGroupName: string, vaultName: string, resourceResourceExtendedInfoDetails: models.VaultExtendedInfoResource, callback: ServiceCallback<models.VaultExtendedInfoResource>): void;
    update(resourceGroupName: string, vaultName: string, resourceResourceExtendedInfoDetails: models.VaultExtendedInfoResource, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.VaultExtendedInfoResource>): void;
}

/**
 * @class
 * Usages
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the RecoveryServicesClient.
 */
export interface Usages {


    /**
     * Fetches the usages of the vault.
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {string} vaultName The name of the recovery services vault.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<VaultUsageList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByVaultsWithHttpOperationResponse(resourceGroupName: string, vaultName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.VaultUsageList>>;

    /**
     * Fetches the usages of the vault.
     *
     * @param {string} resourceGroupName The name of the resource group where the
     * recovery services vault is present.
     *
     * @param {string} vaultName The name of the recovery services vault.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {VaultUsageList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {VaultUsageList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link VaultUsageList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByVaults(resourceGroupName: string, vaultName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.VaultUsageList>;
    listByVaults(resourceGroupName: string, vaultName: string, callback: ServiceCallback<models.VaultUsageList>): void;
    listByVaults(resourceGroupName: string, vaultName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.VaultUsageList>): void;
}
