/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * A2A specific switch protection input.
 *
 * @extends models['SwitchProtectionProviderSpecificInput']
 */
class A2ASwitchProtectionInput extends models['SwitchProtectionProviderSpecificInput'] {
  /**
   * Create a A2ASwitchProtectionInput.
   * @member {string} [recoveryContainerId] The recovery container Id.
   * @member {array} [vmDisks] The list of vm disk details.
   * @member {array} [vmManagedDisks] The list of vm managed disk details.
   * @member {string} [recoveryResourceGroupId] The recovery resource group Id.
   * Valid for V2 scenarios.
   * @member {string} [recoveryCloudServiceId] The recovery cloud service Id.
   * Valid for V1 scenarios.
   * @member {string} [recoveryAvailabilitySetId] The recovery availability
   * set.
   * @member {string} [policyId] The Policy Id.
   * @member {string} [recoveryBootDiagStorageAccountId] The boot diagnostic
   * storage account.
   * @member {object} [diskEncryptionInfo] The recovery disk encryption
   * information.
   * @member {object} [diskEncryptionInfo.diskEncryptionKeyInfo] The recovery
   * KeyVault reference for secret.
   * @member {string}
   * [diskEncryptionInfo.diskEncryptionKeyInfo.secretIdentifier] The secret url
   * / identifier.
   * @member {string}
   * [diskEncryptionInfo.diskEncryptionKeyInfo.keyVaultResourceArmId] The
   * KeyVault resource ARM id for secret.
   * @member {object} [diskEncryptionInfo.keyEncryptionKeyInfo] The recovery
   * KeyVault reference for key.
   * @member {string} [diskEncryptionInfo.keyEncryptionKeyInfo.keyIdentifier]
   * The key url / identifier.
   * @member {string}
   * [diskEncryptionInfo.keyEncryptionKeyInfo.keyVaultResourceArmId] The
   * KeyVault resource ARM id for key.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of A2ASwitchProtectionInput
   *
   * @returns {object} metadata of A2ASwitchProtectionInput
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'A2A',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: 'instanceType',
          clientName: 'instanceType'
        },
        uberParent: 'SwitchProtectionProviderSpecificInput',
        className: 'A2ASwitchProtectionInput',
        modelProperties: {
          instanceType: {
            required: true,
            serializedName: 'instanceType',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          recoveryContainerId: {
            required: false,
            serializedName: 'recoveryContainerId',
            type: {
              name: 'String'
            }
          },
          vmDisks: {
            required: false,
            serializedName: 'vmDisks',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'A2AVmDiskInputDetailsElementType',
                  type: {
                    name: 'Composite',
                    className: 'A2AVmDiskInputDetails'
                  }
              }
            }
          },
          vmManagedDisks: {
            required: false,
            serializedName: 'vmManagedDisks',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'A2AVmManagedDiskInputDetailsElementType',
                  type: {
                    name: 'Composite',
                    className: 'A2AVmManagedDiskInputDetails'
                  }
              }
            }
          },
          recoveryResourceGroupId: {
            required: false,
            serializedName: 'recoveryResourceGroupId',
            type: {
              name: 'String'
            }
          },
          recoveryCloudServiceId: {
            required: false,
            serializedName: 'recoveryCloudServiceId',
            type: {
              name: 'String'
            }
          },
          recoveryAvailabilitySetId: {
            required: false,
            serializedName: 'recoveryAvailabilitySetId',
            type: {
              name: 'String'
            }
          },
          policyId: {
            required: false,
            serializedName: 'policyId',
            type: {
              name: 'String'
            }
          },
          recoveryBootDiagStorageAccountId: {
            required: false,
            serializedName: 'recoveryBootDiagStorageAccountId',
            type: {
              name: 'String'
            }
          },
          diskEncryptionInfo: {
            required: false,
            serializedName: 'diskEncryptionInfo',
            type: {
              name: 'Composite',
              className: 'DiskEncryptionInfo'
            }
          }
        }
      }
    };
  }
}

module.exports = A2ASwitchProtectionInput;
