/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { ServiceClient, ServiceClientOptions, ServiceCallback, HttpOperationResponse, ServiceClientCredentials } from 'ms-rest';
import { AzureServiceClient, AzureServiceClientOptions } from 'ms-rest-azure';
import * as models from "./models";
import * as operations from "./operations";

export default class AzureReservationAPI extends AzureServiceClient {
  /**
   * Initializes a new instance of the AzureReservationAPI class.
   * @constructor
   *
   * @class
   * @param {credentials} credentials - Credentials needed for the client to connect to Azure.
   *
   * @param {string} [baseUri] - The base URI of the service.
   *
   * @param {object} [options] - The parameter options
   *
   * @param {Array} [options.filters] - Filters to be added to the request pipeline
   *
   * @param {object} [options.requestOptions] - Options for the underlying request object
   * {@link https://github.com/request/request#requestoptions-callback Options doc}
   *
   * @param {boolean} [options.noRetryPolicy] - If set to true, turn off default retry policy
   *
   * @param {string} [options.acceptLanguage] - The preferred language for the response.
   *
   * @param {number} [options.longRunningOperationRetryTimeout] - The retry timeout in seconds for Long Running Operations. Default value is 30.
   *
   * @param {boolean} [options.generateClientRequestId] - Whether a unique x-ms-client-request-id should be generated. When set to true a unique x-ms-client-request-id value is generated and included in each request. Default is true.
   *
   */
  constructor(credentials: ServiceClientCredentials, baseUri?: string, options?: AzureServiceClientOptions);

  credentials: ServiceClientCredentials;

  apiVersion: string;

  acceptLanguage: string;

  longRunningOperationRetryTimeout: number;

  generateClientRequestId: boolean;

  // Operation groups
  reservationOrder: operations.ReservationOrder;
  reservation: operations.Reservation;
  operation: operations.Operation;


  /**
   * @summary Get the regions and skus that are available for RI purchase for the
   * specified Azure subscription.
   *
   * @param {string} subscriptionId Id of the subscription
   *
   * @param {string} reservedResourceType The type of the resource for which the
   * skus should be provided.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {string} [options.location] Filters the skus based on the location
   * specified in this parameter. This can be an azure region or global
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<Array>} - The deserialized result object.
   *
   * @reject {Error|ServiceError} - The error object.
   */
  getCatalogWithHttpOperationResponse(subscriptionId: string, reservedResourceType: string, options?: { location? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Catalog[]>>;

  /**
   * @summary Get the regions and skus that are available for RI purchase for the
   * specified Azure subscription.
   *
   * @param {string} subscriptionId Id of the subscription
   *
   * @param {string} reservedResourceType The type of the resource for which the
   * skus should be provided.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {string} [options.location] Filters the skus based on the location
   * specified in this parameter. This can be an azure region or global
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {ServiceCallback} [optionalCallback] - The optional callback.
   *
   * @returns {ServiceCallback|Promise} If a callback was passed as the last
   * parameter then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned.
   *
   *                      @resolve {Array} - The deserialized result object.
   *
   *                      @reject {Error|ServiceError} - The error object.
   *
   * {ServiceCallback} optionalCallback(err, result, request, response)
   *
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {Array} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
   */
  getCatalog(subscriptionId: string, reservedResourceType: string, options?: { location? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.Catalog[]>;
  getCatalog(subscriptionId: string, reservedResourceType: string, callback: ServiceCallback<models.Catalog[]>): void;
  getCatalog(subscriptionId: string, reservedResourceType: string, options: { location? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Catalog[]>): void;


  /**
   * @summary Get list of applicable `Reservation`s.
   *
   * Get applicable `Reservation`s that are applied to this subscription.
   *
   * @param {string} subscriptionId Id of the subscription
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<AppliedReservations>} - The deserialized result object.
   *
   * @reject {Error|ServiceError} - The error object.
   */
  getAppliedReservationListWithHttpOperationResponse(subscriptionId: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.AppliedReservations>>;

  /**
   * @summary Get list of applicable `Reservation`s.
   *
   * Get applicable `Reservation`s that are applied to this subscription.
   *
   * @param {string} subscriptionId Id of the subscription
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {ServiceCallback} [optionalCallback] - The optional callback.
   *
   * @returns {ServiceCallback|Promise} If a callback was passed as the last
   * parameter then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned.
   *
   *                      @resolve {AppliedReservations} - The deserialized result object.
   *
   *                      @reject {Error|ServiceError} - The error object.
   *
   * {ServiceCallback} optionalCallback(err, result, request, response)
   *
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {AppliedReservations} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link AppliedReservations} for more information.
   *
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
   */
  getAppliedReservationList(subscriptionId: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.AppliedReservations>;
  getAppliedReservationList(subscriptionId: string, callback: ServiceCallback<models.AppliedReservations>): void;
  getAppliedReservationList(subscriptionId: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.AppliedReservations>): void;
}

export { AzureReservationAPI, models as AzureReservationAPIModels };
