/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is regenerated.
 */

import { BaseResource, CloudError } from "ms-rest-azure";

export {

  BaseResource,
  CloudError
};

/**
 * The policy sku. This property is optional, obsolete, and will be ignored.
 */
export interface PolicySku {
  /**
   * The name of the policy sku. Possible values are A0 and A1.
   */
  name: string;
  /**
   * The policy sku tier. Possible values are Free and Standard.
   */
  tier?: string;
}

/**
 * Identity for the resource.
 */
export interface Identity {
  /**
   * The principal ID of the resource identity.
   */
  readonly principalId?: string;
  /**
   * The tenant ID of the resource identity.
   */
  readonly tenantId?: string;
  /**
   * The identity type. Possible values include: 'SystemAssigned', 'None'
   */
  type?: string;
}

/**
 * The policy assignment.
 */
export interface PolicyAssignment extends BaseResource {
  /**
   * The display name of the policy assignment.
   */
  displayName?: string;
  /**
   * The ID of the policy definition or policy set definition being assigned.
   */
  policyDefinitionId?: string;
  /**
   * The scope for the policy assignment.
   */
  scope?: string;
  /**
   * The policy's excluded scopes.
   */
  notScopes?: string[];
  /**
   * Required if a parameter is used in policy rule.
   */
  parameters?: any;
  /**
   * This message will be part of response in case of policy violation.
   */
  description?: string;
  /**
   * The policy assignment metadata.
   */
  metadata?: any;
  /**
   * The ID of the policy assignment.
   */
  readonly id?: string;
  /**
   * The type of the policy assignment.
   */
  readonly type?: string;
  /**
   * The name of the policy assignment.
   */
  readonly name?: string;
  /**
   * The policy sku. This property is optional, obsolete, and will be ignored.
   */
  sku?: PolicySku;
  /**
   * The location of the policy assignment. Only required when utilizing managed identity.
   */
  location?: string;
  /**
   * The managed identity associated with the policy assignment.
   */
  identity?: Identity;
}

/**
 * Error response indicates Azure Resource Manager is not able to process the incoming request. The
 * reason is provided in the error message.
 */
export interface ErrorResponse {
  /**
   * Http status code.
   */
  httpStatus?: string;
  /**
   * Error code.
   */
  errorCode?: string;
  /**
   * Error message indicating why the operation failed.
   */
  errorMessage?: string;
}

/**
 * The policy definition.
 */
export interface PolicyDefinition extends BaseResource {
  /**
   * The type of policy definition. Possible values are NotSpecified, BuiltIn, and Custom. Possible
   * values include: 'NotSpecified', 'BuiltIn', 'Custom'
   */
  policyType?: string;
  /**
   * The policy definition mode. Possible values are NotSpecified, Indexed, and All. Possible
   * values include: 'NotSpecified', 'Indexed', 'All'
   */
  mode?: string;
  /**
   * The display name of the policy definition.
   */
  displayName?: string;
  /**
   * The policy definition description.
   */
  description?: string;
  /**
   * The policy rule.
   */
  policyRule?: any;
  /**
   * The policy definition metadata.
   */
  metadata?: any;
  /**
   * Required if a parameter is used in policy rule.
   */
  parameters?: any;
  /**
   * The ID of the policy definition.
   */
  readonly id?: string;
  /**
   * The name of the policy definition.
   */
  readonly name?: string;
  /**
   * The type of the resource (Microsoft.Authorization/policyDefinitions).
   */
  readonly type?: string;
}

/**
 * The policy definition reference.
 */
export interface PolicyDefinitionReference {
  /**
   * The ID of the policy definition or policy set definition.
   */
  policyDefinitionId?: string;
  /**
   * Required if a parameter is used in policy rule.
   */
  parameters?: any;
}

/**
 * The policy set definition.
 */
export interface PolicySetDefinition extends BaseResource {
  /**
   * The type of policy definition. Possible values are NotSpecified, BuiltIn, and Custom. Possible
   * values include: 'NotSpecified', 'BuiltIn', 'Custom'
   */
  policyType?: string;
  /**
   * The display name of the policy set definition.
   */
  displayName?: string;
  /**
   * The policy set definition description.
   */
  description?: string;
  /**
   * The policy set definition metadata.
   */
  metadata?: any;
  /**
   * The policy set definition parameters that can be used in policy definition references.
   */
  parameters?: any;
  /**
   * An array of policy definition references.
   */
  policyDefinitions: PolicyDefinitionReference[];
  /**
   * The ID of the policy set definition.
   */
  readonly id?: string;
  /**
   * The name of the policy set definition.
   */
  readonly name?: string;
  /**
   * The type of the resource (Microsoft.Authorization/policySetDefinitions).
   */
  readonly type?: string;
}

/**
 * List of policy assignments.
 */
export interface PolicyAssignmentListResult extends Array<PolicyAssignment> {
  /**
   * The URL to use for getting the next set of results.
   */
  nextLink?: string;
}

/**
 * List of policy definitions.
 */
export interface PolicyDefinitionListResult extends Array<PolicyDefinition> {
  /**
   * The URL to use for getting the next set of results.
   */
  nextLink?: string;
}

/**
 * List of policy set definitions.
 */
export interface PolicySetDefinitionListResult extends Array<PolicySetDefinition> {
  /**
   * The URL to use for getting the next set of results.
   */
  nextLink?: string;
}
