/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * Defines an item in ApplicationHealthPolicyMap.
 *
 */
class ApplicationHealthPolicyMapItem {
  /**
   * Create a ApplicationHealthPolicyMapItem.
   * @property {string} key The key of the application health policy map item.
   * This is the name of the application.
   * @property {object} value The value of the application health policy map
   * item. This is the ApplicationHealthPolicy for this application.
   * @property {boolean} [value.considerWarningAsError] Indicates whether
   * warnings are treated with the same severity as errors.
   * @property {number} [value.maxPercentUnhealthyDeployedApplications] The
   * maximum allowed percentage of unhealthy deployed applications. Allowed
   * values are Byte values from zero to 100.
   * The percentage represents the maximum tolerated percentage of deployed
   * applications that can be unhealthy before the application is considered in
   * error.
   * This is calculated by dividing the number of unhealthy deployed
   * applications over the number of nodes where the application is currently
   * deployed on in the cluster.
   * The computation rounds up to tolerate one failure on small numbers of
   * nodes. Default percentage is zero.
   * @property {object} [value.defaultServiceTypeHealthPolicy] The health
   * policy used by default to evaluate the health of a service type.
   * @property {number}
   * [value.defaultServiceTypeHealthPolicy.maxPercentUnhealthyPartitionsPerService]
   * The maximum allowed percentage of unhealthy partitions per service.
   * Allowed values are Byte values from zero to 100
   *
   * The percentage represents the maximum tolerated percentage of partitions
   * that can be unhealthy before the service is considered in error.
   * If the percentage is respected but there is at least one unhealthy
   * partition, the health is evaluated as Warning.
   * The percentage is calculated by dividing the number of unhealthy
   * partitions over the total number of partitions in the service.
   * The computation rounds up to tolerate one failure on small numbers of
   * partitions. Default percentage is zero.
   * @property {number}
   * [value.defaultServiceTypeHealthPolicy.maxPercentUnhealthyReplicasPerPartition]
   * The maximum allowed percentage of unhealthy replicas per partition.
   * Allowed values are Byte values from zero to 100.
   *
   * The percentage represents the maximum tolerated percentage of replicas
   * that can be unhealthy before the partition is considered in error.
   * If the percentage is respected but there is at least one unhealthy
   * replica, the health is evaluated as Warning.
   * The percentage is calculated by dividing the number of unhealthy replicas
   * over the total number of replicas in the partition.
   * The computation rounds up to tolerate one failure on small numbers of
   * replicas. Default percentage is zero.
   * @property {number}
   * [value.defaultServiceTypeHealthPolicy.maxPercentUnhealthyServices] The
   * maximum allowed percentage of unhealthy services. Allowed values are Byte
   * values from zero to 100.
   *
   * The percentage represents the maximum tolerated percentage of services
   * that can be unhealthy before the application is considered in error.
   * If the percentage is respected but there is at least one unhealthy
   * service, the health is evaluated as Warning.
   * This is calculated by dividing the number of unhealthy services of the
   * specific service type over the total number of services of the specific
   * service type.
   * The computation rounds up to tolerate one failure on small numbers of
   * services. Default percentage is zero.
   * @property {array} [value.serviceTypeHealthPolicyMap] The map with service
   * type health policy per service type name. The map is empty by default.
   */
  constructor() {
  }

  /**
   * Defines the metadata of ApplicationHealthPolicyMapItem
   *
   * @returns {object} metadata of ApplicationHealthPolicyMapItem
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ApplicationHealthPolicyMapItem',
      type: {
        name: 'Composite',
        className: 'ApplicationHealthPolicyMapItem',
        modelProperties: {
          key: {
            required: true,
            serializedName: 'Key',
            type: {
              name: 'String'
            }
          },
          value: {
            required: true,
            serializedName: 'Value',
            type: {
              name: 'Composite',
              className: 'ApplicationHealthPolicy'
            }
          }
        }
      }
    };
  }
}

module.exports = ApplicationHealthPolicyMapItem;
