/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is regenerated.
 */

import { BaseResource, CloudError } from "ms-rest-azure";
import * as moment from "moment";

export {

  BaseResource,
  CloudError
};

/**
 * A private IP address bound to the availability group listener.
 */
export interface PrivateIPAddress {
  /**
   * Private IP address bound to the availability group listener.
   */
  ipAddress?: string;
  /**
   * Subnet used to include private IP.
   */
  subnetResourceId?: string;
}

/**
 * A load balancer configuration for an availability group listener.
 */
export interface LoadBalancerConfiguration {
  /**
   * Private IP address.
   */
  privateIpAddress?: PrivateIPAddress;
  /**
   * Resource id of the public IP.
   */
  publicIpAddressResourceId?: string;
  /**
   * Resource id of the load balancer.
   */
  loadBalancerResourceId?: string;
  /**
   * Probe port.
   */
  probePort?: number;
  /**
   * List of the SQL virtual machine instance resource id's that are enrolled into the availability
   * group listener.
   */
  sqlVirtualMachineInstances?: string[];
}

/**
 * ARM resource.
 */
export interface Resource extends BaseResource {
  /**
   * Resource ID.
   */
  readonly id?: string;
  /**
   * Resource name.
   */
  readonly name?: string;
  /**
   * Resource type.
   */
  readonly type?: string;
}

/**
 * ARM proxy resource.
 */
export interface ProxyResource extends Resource {
}

/**
 * A SQL Server availability group listener.
 */
export interface AvailabilityGroupListener extends ProxyResource {
  /**
   * Provisioning state to track the aysnc operation status.
   */
  readonly provisioningState?: string;
  /**
   * Name of the availability group.
   */
  availabilityGroupName?: string;
  /**
   * List of load balancer configurations for an availability group listener.
   */
  loadBalancerConfigurations?: LoadBalancerConfiguration[];
  /**
   * Create a default availability group if it does not exist.
   */
  createDefaultAvailabilityGroupIfNotExist?: boolean;
  /**
   * Listener port.
   */
  port?: number;
}

/**
 * Display metadata associated with the operation.
 */
export interface OperationDisplay {
  /**
   * The localized friendly form of the resource provider name.
   */
  readonly provider?: string;
  /**
   * The localized friendly form of the resource type related to this action/operation.
   */
  readonly resource?: string;
  /**
   * The localized friendly name for the operation.
   */
  readonly operation?: string;
  /**
   * The localized friendly description for the operation.
   */
  readonly description?: string;
}

/**
 * SQL REST API operation definition.
 */
export interface Operation {
  /**
   * The name of the operation being performed on this particular object.
   */
  readonly name?: string;
  /**
   * The localized display information for this particular operation / action.
   */
  readonly display?: OperationDisplay;
  /**
   * The intended executor of the operation. Possible values include: 'user', 'system'
   */
  readonly origin?: string;
  /**
   * Additional descriptions for the operation.
   */
  readonly properties?: { [propertyName: string]: any };
}

/**
 * Active Directory account details to operate Windows Server Failover Cluster.
 */
export interface WsfcDomainProfile {
  /**
   * Fully qualified name of the domain.
   */
  domainFqdn?: string;
  /**
   * Organizational Unit path in which the nodes and cluster will be present.
   */
  ouPath?: string;
  /**
   * Account name used for creating cluster (at minimum needs permissions to 'Create Computer
   * Objects' in domain).
   */
  clusterBootstrapAccount?: string;
  /**
   * Account name used for operating cluster i.e. will be part of administrators group on all the
   * participating virtual machines in the cluster.
   */
  clusterOperatorAccount?: string;
  /**
   * Account name under which SQL service will run on all participating SQL virtual machines in the
   * cluster.
   */
  sqlServiceAccount?: string;
  /**
   * Optional path for fileshare witness.
   */
  fileShareWitnessPath?: string;
  /**
   * Fully qualified ARM resource id of the witness storage account.
   */
  storageAccountUrl?: string;
  /**
   * Primary key of the witness storage account.
   */
  storageAccountPrimaryKey?: string;
}

/**
 * ARM tracked top level resource.
 */
export interface TrackedResource extends Resource {
  /**
   * Resource location.
   */
  location: string;
  /**
   * Resource tags.
   */
  tags?: { [propertyName: string]: string };
}

/**
 * A SQL virtual machine group.
 */
export interface SqlVirtualMachineGroup extends TrackedResource {
  /**
   * Provisioning state to track the aysnc operation status.
   */
  readonly provisioningState?: string;
  /**
   * SQL image offer. Examples may include SQL2016-WS2016, SQL2017-WS2016.
   */
  sqlImageOffer?: string;
  /**
   * SQL image sku. Possible values include: 'Developer', 'Enterprise'
   */
  sqlImageSku?: string;
  /**
   * Scale type. Possible values include: 'HA'
   */
  readonly scaleType?: string;
  /**
   * Type of cluster manager: Windows Server Failover Cluster (WSFC), implied by the scale type of
   * the group and the OS type. Possible values include: 'WSFC'
   */
  readonly clusterManagerType?: string;
  /**
   * Cluster type. Possible values include: 'Domainful'
   */
  readonly clusterConfiguration?: string;
  /**
   * Cluster Active Directory domain profile.
   */
  wsfcDomainProfile?: WsfcDomainProfile;
}

/**
 * An update to a SQL virtual machine group.
 */
export interface SqlVirtualMachineGroupUpdate {
  /**
   * Resource tags.
   */
  tags?: { [propertyName: string]: string };
}

/**
 * Azure Active Directory identity configuration for a resource.
 */
export interface ResourceIdentity {
  /**
   * The Azure Active Directory principal id.
   */
  readonly principalId?: string;
  /**
   * The identity type. Set this to 'SystemAssigned' in order to automatically create and assign an
   * Azure Active Directory principal for the resource. Possible values include: 'SystemAssigned'
   */
  type?: string;
  /**
   * The Azure Active Directory tenant id.
   */
  readonly tenantId?: string;
}

/**
 * Domain credentials for setting up Windows Server Failover Cluster for SQL availability group.
 */
export interface WsfcDomainCredentials {
  /**
   * Cluster bootstrap account password.
   */
  clusterBootstrapAccountPassword?: string;
  /**
   * Cluster operator account password.
   */
  clusterOperatorAccountPassword?: string;
  /**
   * SQL service account password.
   */
  sqlServiceAccountPassword?: string;
}

/**
 * Set a patching window during which Windows and SQL patches will be applied.
 */
export interface AutoPatchingSettings {
  /**
   * Enable or disable autopatching on SQL virtual machine.
   */
  enable?: boolean;
  /**
   * Day of week to apply the patch on. Possible values include: 'Monday', 'Tuesday', 'Wednesday',
   * 'Thursday', 'Friday', 'Saturday', 'Sunday'
   */
  dayOfWeek?: string;
  /**
   * Hour of the day when patching is initiated. Local VM time.
   */
  maintenanceWindowStartingHour?: number;
  /**
   * Duration of patching.
   */
  maintenanceWindowDuration?: number;
}

/**
 * Configure backups for databases in your SQL virtual machine.
 */
export interface AutoBackupSettings {
  /**
   * Enable or disable autobackup on SQL virtual machine.
   */
  enable?: boolean;
  /**
   * Enable or disable encryption for backup on SQL virtual machine.
   */
  enableEncryption?: boolean;
  /**
   * Retention period of backup: 1-30 days.
   */
  retentionPeriod?: number;
  /**
   * Storage account url where backup will be taken to.
   */
  storageAccountUrl?: string;
  /**
   * Storage account key where backup will be taken to.
   */
  storageAccessKey?: string;
  /**
   * Password for encryption on backup.
   */
  password?: string;
  /**
   * Include or exclude system databases from auto backup.
   */
  backupSystemDbs?: boolean;
  /**
   * Backup schedule type. Possible values include: 'Manual', 'Automated'
   */
  backupScheduleType?: string;
  /**
   * Frequency of full backups. In both cases, full backups begin during the next scheduled time
   * window. Possible values include: 'Daily', 'Weekly'
   */
  fullBackupFrequency?: string;
  /**
   * Start time of a given day during which full backups can take place. 0-23 hours.
   */
  fullBackupStartTime?: number;
  /**
   * Duration of the time window of a given day during which full backups can take place. 1-23
   * hours.
   */
  fullBackupWindowHours?: number;
  /**
   * Frequency of log backups. 5-60 minutes.
   */
  logBackupFrequency?: number;
}

/**
 * Configure your SQL virtual machine to be able to connect to the Azure Key Vault service.
 */
export interface KeyVaultCredentialSettings {
  /**
   * Enable or disable key vault credential setting.
   */
  enable?: boolean;
  /**
   * Credential name.
   */
  credentialName?: string;
  /**
   * Azure Key Vault url.
   */
  azureKeyVaultUrl?: string;
  /**
   * Service principal name to access key vault.
   */
  servicePrincipalName?: string;
  /**
   * Service principal name secret to access key vault.
   */
  servicePrincipalSecret?: string;
}

/**
 * Set the access level and network port settings for SQL Server.
 */
export interface SqlConnectivityUpdateSettings {
  /**
   * SQL Server connectivity option. Possible values include: 'LOCAL', 'PRIVATE', 'PUBLIC'
   */
  connectivityType?: string;
  /**
   * SQL Server port.
   */
  port?: number;
  /**
   * SQL Server sysadmin login to create.
   */
  sqlAuthUpdateUserName?: string;
  /**
   * SQL Server sysadmin login password.
   */
  sqlAuthUpdatePassword?: string;
}

/**
 * Set workload type to optimize storage for SQL Server.
 */
export interface SqlWorkloadTypeUpdateSettings {
  /**
   * SQL Server workload type. Possible values include: 'GENERAL', 'OLTP', 'DW'
   */
  sqlWorkloadType?: string;
}

/**
 * Set disk storage settings for SQL Server.
 */
export interface SqlStorageUpdateSettings {
  /**
   * Virtual machine disk count.
   */
  diskCount?: number;
  /**
   * Disk configuration to apply to SQL Server. Possible values include: 'NEW', 'EXTEND', 'ADD'
   */
  diskConfigurationType?: string;
  /**
   * Device id of the first disk to be updated.
   */
  startingDeviceId?: number;
}

/**
 * Additional SQL Server feature settings.
 */
export interface AdditionalFeaturesServerConfigurations {
  /**
   * Enable or disable R services (SQL 2016 onwards).
   */
  isRServicesEnabled?: boolean;
}

/**
 * Set the connectivity, storage and workload settings.
 */
export interface ServerConfigurationsManagementSettings {
  /**
   * SQL connectivity type settings.
   */
  sqlConnectivityUpdateSettings?: SqlConnectivityUpdateSettings;
  /**
   * SQL workload type settings.
   */
  sqlWorkloadTypeUpdateSettings?: SqlWorkloadTypeUpdateSettings;
  /**
   * SQL storage update settings.
   */
  sqlStorageUpdateSettings?: SqlStorageUpdateSettings;
  /**
   * Additional SQL feature settings.
   */
  additionalFeaturesServerConfigurations?: AdditionalFeaturesServerConfigurations;
}

/**
 * A SQL virtual machine.
 */
export interface SqlVirtualMachine extends TrackedResource {
  /**
   * Azure Active Directory identity of the server.
   */
  identity?: ResourceIdentity;
  /**
   * ARM Resource id of underlying virtual machine created from SQL marketplace image.
   */
  virtualMachineResourceId?: string;
  /**
   * Provisioning state to track the aysnc operation status.
   */
  readonly provisioningState?: string;
  /**
   * SQL image offer. Examples include SQL2016-WS2016, SQL2017-WS2016.
   */
  readonly sqlImageOffer?: string;
  /**
   * SQL Server license type. Possible values include: 'PAYG', 'AHUB'
   */
  sqlServerLicenseType?: string;
  /**
   * SQL image sku. Possible values include: 'Developer', 'Express', 'Standard', 'Enterprise',
   * 'Web'
   */
  readonly sqlImageSku?: string;
  /**
   * ARM resource id of the SQL virtual machine group this SQL virtual machine is or will be part
   * of.
   */
  sqlVirtualMachineGroupResourceId?: string;
  /**
   * Domain credentials for setting up Windows Server Failover Cluster for SQL availability group.
   */
  wsfcDomainCredentials?: WsfcDomainCredentials;
  /**
   * Auto patching settings for applying critical security updates to SQL virtual machine.
   */
  autoPatchingSettings?: AutoPatchingSettings;
  /**
   * Auto backup settings for SQL Server.
   */
  autoBackupSettings?: AutoBackupSettings;
  /**
   * Key vault credential settings.
   */
  keyVaultCredentialSettings?: KeyVaultCredentialSettings;
  /**
   * SQL Server configuration management settings.
   */
  serverConfigurationsManagementSettings?: ServerConfigurationsManagementSettings;
}

/**
 * An update to a SQL virtual machine.
 */
export interface SqlVirtualMachineUpdate {
  /**
   * Resource tags.
   */
  tags?: { [propertyName: string]: string };
}

/**
 * A list of availability group listeners.
 */
export interface AvailabilityGroupListenerListResult extends Array<AvailabilityGroupListener> {
  /**
   * Link to retrieve next page of results.
   */
  readonly nextLink?: string;
}

/**
 * Result of the request to list SQL operations.
 */
export interface OperationListResult extends Array<Operation> {
  /**
   * Link to retrieve next page of results.
   */
  readonly nextLink?: string;
}

/**
 * A list of SQL virtual machine groups.
 */
export interface SqlVirtualMachineGroupListResult extends Array<SqlVirtualMachineGroup> {
  /**
   * Link to retrieve next page of results.
   */
  readonly nextLink?: string;
}

/**
 * A list of SQL virtual machines.
 */
export interface SqlVirtualMachineListResult extends Array<SqlVirtualMachine> {
  /**
   * Link to retrieve next page of results.
   */
  readonly nextLink?: string;
}
