/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const msRest = require('ms-rest');
const msRestAzure = require('ms-rest-azure');
const WebResource = msRest.WebResource;


/**
 * Creates an Azure subscription
 *
 * @param {string} enrollmentAccountName The name of the enrollment account to
 * which the subscription will be billed.
 *
 * @param {object} body The subscription creation parameters.
 *
 * @param {string} [body.displayName] The display name of the subscription.
 *
 * @param {array} [body.owners] The list of principals that should be granted
 * Owner access on the subscription. Principals should be of type User, Service
 * Principal or Security Group.
 *
 * @param {string} [body.offerType] The offer type of the subscription. For
 * example, MS-AZR-0017P (EnterpriseAgreement) and MS-AZR-0148P
 * (EnterpriseAgreement devTest) are available. Only valid when creating a
 * subscription in a enrollment account scope. Possible values include:
 * 'MS-AZR-0017P', 'MS-AZR-0148P'
 *
 * @param {object} [body.additionalParameters] Additional, untyped parameters
 * to support custom subscription creation scenarios.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link SubscriptionCreationResult} for more
 *                      information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _createSubscriptionInEnrollmentAccount(enrollmentAccountName, body, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }

  if (!callback) {
    throw new Error('callback cannot be null.');
  }

  // Send request
  this.beginCreateSubscriptionInEnrollmentAccount(enrollmentAccountName, body, options, (err, parsedResult, httpRequest, response) => {
    if (err) return callback(err);

    let initialResult = new msRest.HttpOperationResponse();
    initialResult.request = httpRequest;
    initialResult.response = response;
    initialResult.body = response.body;
    client.getLongRunningOperationResult(initialResult, options, (err, pollingResult) => {
      if (err) return callback(err);

      // Create Result
      let result = null;

      httpRequest = pollingResult.request;
      response = pollingResult.response;
      let responseBody = pollingResult.body;
      if (responseBody === '') responseBody = null;

      // Deserialize Response
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['SubscriptionCreationResult']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }

      return callback(null, result, httpRequest, response);
    });
  });
}

/**
 * Creates an Azure subscription
 *
 * @param {string} enrollmentAccountName The name of the enrollment account to
 * which the subscription will be billed.
 *
 * @param {object} body The subscription creation parameters.
 *
 * @param {string} [body.displayName] The display name of the subscription.
 *
 * @param {array} [body.owners] The list of principals that should be granted
 * Owner access on the subscription. Principals should be of type User, Service
 * Principal or Security Group.
 *
 * @param {string} [body.offerType] The offer type of the subscription. For
 * example, MS-AZR-0017P (EnterpriseAgreement) and MS-AZR-0148P
 * (EnterpriseAgreement devTest) are available. Only valid when creating a
 * subscription in a enrollment account scope. Possible values include:
 * 'MS-AZR-0017P', 'MS-AZR-0148P'
 *
 * @param {object} [body.additionalParameters] Additional, untyped parameters
 * to support custom subscription creation scenarios.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link SubscriptionCreationResult} for more
 *                      information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _beginCreateSubscriptionInEnrollmentAccount(enrollmentAccountName, body, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let apiVersion = '2018-03-01-preview';
  // Validate
  try {
    if (enrollmentAccountName === null || enrollmentAccountName === undefined || typeof enrollmentAccountName.valueOf() !== 'string') {
      throw new Error('enrollmentAccountName cannot be null or undefined and it must be of type string.');
    }
    if (body === null || body === undefined) {
      throw new Error('body cannot be null or undefined.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'providers/Microsoft.Billing/enrollmentAccounts/{enrollmentAccountName}/providers/Microsoft.Subscription/createSubscription';
  requestUrl = requestUrl.replace('{enrollmentAccountName}', encodeURIComponent(enrollmentAccountName));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'POST';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (body !== null && body !== undefined) {
      let requestModelMapper = new client.models['SubscriptionCreationParameters']().mapper();
      requestModel = client.serialize(requestModelMapper, body, 'body');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(body, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200 && statusCode !== 202) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          let internalError = null;
          if (parsedErrorResponse.error) internalError = parsedErrorResponse.error;
          error.code = internalError ? internalError.code : parsedErrorResponse.code;
          error.message = internalError ? internalError.message : parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['ErrorResponse']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['SubscriptionCreationResult']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/** Class representing a SubscriptionFactory. */
class SubscriptionFactory {
  /**
   * Create a SubscriptionFactory.
   * @param {SubscriptionClient} client Reference to the service client.
   */
  constructor(client) {
    this.client = client;
    this._createSubscriptionInEnrollmentAccount = _createSubscriptionInEnrollmentAccount;
    this._beginCreateSubscriptionInEnrollmentAccount = _beginCreateSubscriptionInEnrollmentAccount;
  }

  /**
   * Creates an Azure subscription
   *
   * @param {string} enrollmentAccountName The name of the enrollment account to
   * which the subscription will be billed.
   *
   * @param {object} body The subscription creation parameters.
   *
   * @param {string} [body.displayName] The display name of the subscription.
   *
   * @param {array} [body.owners] The list of principals that should be granted
   * Owner access on the subscription. Principals should be of type User, Service
   * Principal or Security Group.
   *
   * @param {string} [body.offerType] The offer type of the subscription. For
   * example, MS-AZR-0017P (EnterpriseAgreement) and MS-AZR-0148P
   * (EnterpriseAgreement devTest) are available. Only valid when creating a
   * subscription in a enrollment account scope. Possible values include:
   * 'MS-AZR-0017P', 'MS-AZR-0148P'
   *
   * @param {object} [body.additionalParameters] Additional, untyped parameters
   * to support custom subscription creation scenarios.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<SubscriptionCreationResult>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  createSubscriptionInEnrollmentAccountWithHttpOperationResponse(enrollmentAccountName, body, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._createSubscriptionInEnrollmentAccount(enrollmentAccountName, body, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Creates an Azure subscription
   *
   * @param {string} enrollmentAccountName The name of the enrollment account to
   * which the subscription will be billed.
   *
   * @param {object} body The subscription creation parameters.
   *
   * @param {string} [body.displayName] The display name of the subscription.
   *
   * @param {array} [body.owners] The list of principals that should be granted
   * Owner access on the subscription. Principals should be of type User, Service
   * Principal or Security Group.
   *
   * @param {string} [body.offerType] The offer type of the subscription. For
   * example, MS-AZR-0017P (EnterpriseAgreement) and MS-AZR-0148P
   * (EnterpriseAgreement devTest) are available. Only valid when creating a
   * subscription in a enrollment account scope. Possible values include:
   * 'MS-AZR-0017P', 'MS-AZR-0148P'
   *
   * @param {object} [body.additionalParameters] Additional, untyped parameters
   * to support custom subscription creation scenarios.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {SubscriptionCreationResult} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link SubscriptionCreationResult} for more
   *                      information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  createSubscriptionInEnrollmentAccount(enrollmentAccountName, body, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._createSubscriptionInEnrollmentAccount(enrollmentAccountName, body, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._createSubscriptionInEnrollmentAccount(enrollmentAccountName, body, options, optionalCallback);
    }
  }

  /**
   * Creates an Azure subscription
   *
   * @param {string} enrollmentAccountName The name of the enrollment account to
   * which the subscription will be billed.
   *
   * @param {object} body The subscription creation parameters.
   *
   * @param {string} [body.displayName] The display name of the subscription.
   *
   * @param {array} [body.owners] The list of principals that should be granted
   * Owner access on the subscription. Principals should be of type User, Service
   * Principal or Security Group.
   *
   * @param {string} [body.offerType] The offer type of the subscription. For
   * example, MS-AZR-0017P (EnterpriseAgreement) and MS-AZR-0148P
   * (EnterpriseAgreement devTest) are available. Only valid when creating a
   * subscription in a enrollment account scope. Possible values include:
   * 'MS-AZR-0017P', 'MS-AZR-0148P'
   *
   * @param {object} [body.additionalParameters] Additional, untyped parameters
   * to support custom subscription creation scenarios.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<SubscriptionCreationResult>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  beginCreateSubscriptionInEnrollmentAccountWithHttpOperationResponse(enrollmentAccountName, body, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._beginCreateSubscriptionInEnrollmentAccount(enrollmentAccountName, body, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Creates an Azure subscription
   *
   * @param {string} enrollmentAccountName The name of the enrollment account to
   * which the subscription will be billed.
   *
   * @param {object} body The subscription creation parameters.
   *
   * @param {string} [body.displayName] The display name of the subscription.
   *
   * @param {array} [body.owners] The list of principals that should be granted
   * Owner access on the subscription. Principals should be of type User, Service
   * Principal or Security Group.
   *
   * @param {string} [body.offerType] The offer type of the subscription. For
   * example, MS-AZR-0017P (EnterpriseAgreement) and MS-AZR-0148P
   * (EnterpriseAgreement devTest) are available. Only valid when creating a
   * subscription in a enrollment account scope. Possible values include:
   * 'MS-AZR-0017P', 'MS-AZR-0148P'
   *
   * @param {object} [body.additionalParameters] Additional, untyped parameters
   * to support custom subscription creation scenarios.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {SubscriptionCreationResult} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link SubscriptionCreationResult} for more
   *                      information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  beginCreateSubscriptionInEnrollmentAccount(enrollmentAccountName, body, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._beginCreateSubscriptionInEnrollmentAccount(enrollmentAccountName, body, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._beginCreateSubscriptionInEnrollmentAccount(enrollmentAccountName, body, options, optionalCallback);
    }
  }

}

module.exports = SubscriptionFactory;
